@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2019-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2019-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -23,9 +23,7 @@
 #ifndef ROCPRIM_BENCHMARK_BLOCK_SORT_PARALLEL_HPP_
 #define ROCPRIM_BENCHMARK_BLOCK_SORT_PARALLEL_HPP_
 
-#include <cstddef>
-#include <string>
-#include <vector>
+#include "benchmark_utils.hpp"
 
 // Google Benchmark
 #include <benchmark/benchmark.h>
@@ -34,9 +32,15 @@
 #include <hip/hip_runtime.h>
 
 // rocPRIM
-#include <rocprim/detail/various.hpp>
+#include <rocprim/block/block_load_func.hpp>
+#include <rocprim/block/block_sort.hpp>
+#include <rocprim/block/block_store_func.hpp>
+#include <rocprim/type_traits.hpp>
 
-#include "benchmark_utils.hpp"
+#include <string>
+#include <vector>
+
+#include <cstddef>
 
 template<class KeyType,
          class ValueType,
@@ -187,11 +191,11 @@ public:
     static constexpr unsigned int warmup_size       = 5;
     static constexpr bool         debug_synchronous = false;
 
-    auto dispatch_block_sort(std::false_type /*stable_sort*/,
+    static auto dispatch_block_sort(std::false_type /*stable_sort*/,
                              size_t            size,
                              const hipStream_t stream,
                              KeyType*          d_input,
-                             KeyType*          d_output) const
+                             KeyType*          d_output)
     {
         hipLaunchKernelGGL(
             HIP_KERNEL_NAME(
@@ -204,11 +208,11 @@ public:
             d_output);
     }
 
-    auto dispatch_block_sort(std::true_type /*stable_sort*/,
+    static auto dispatch_block_sort(std::true_type /*stable_sort*/,
                              size_t            size,
                              const hipStream_t stream,
                              KeyType*          d_input,
-                             KeyType*          d_output) const
+                             KeyType*          d_output)
     {
         hipLaunchKernelGGL(HIP_KERNEL_NAME(stable_sort_kernel<KeyType,
                                                               ValueType,
@@ -223,21 +227,21 @@ public:
                            d_output);
     }
 
-    void run(benchmark::State& state, const std::size_t N, const hipStream_t stream) const override
+    void run(benchmark::State&   state,
+             size_t              bytes,
+             const managed_seed& seed,
+             hipStream_t         stream) const override
     {
+        // Calculate the number of elements N
+        size_t N = bytes / sizeof(KeyType);
+
         const auto size = items_per_block * ((N + items_per_block - 1) / items_per_block);
 
-        std::vector<KeyType> input;
-        if(std::is_floating_point<KeyType>::value)
-        {
-            input = get_random_data<KeyType>(size, (KeyType)-1000, (KeyType) + 1000);
-        }
-        else
-        {
-            input = get_random_data<KeyType>(size,
-                                             std::numeric_limits<KeyType>::min(),
-                                             std::numeric_limits<KeyType>::max());
-        }
+        std::vector<KeyType> input = get_random_data<KeyType>(size,
+                                                              generate_limits<KeyType>::min(),
+                                                              generate_limits<KeyType>::max(),
+                                                              seed.get_0());
+
         KeyType* d_input;
         KeyType* d_output;
         HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&d_input), size * sizeof(KeyType)));
