@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,9 +20,8 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 
-#include <cstddef>
-#include <iostream>
-#include <string>
+#include "benchmark_device_adjacent_difference.parallel.hpp"
+#include "benchmark_utils.hpp"
 
 // Google Benchmark
 #include <benchmark/benchmark.h>
@@ -31,23 +30,24 @@
 #include <hip/hip_runtime_api.h>
 
 // rocPRIM
-#include <rocprim/detail/various.hpp>
 #include <rocprim/device/device_adjacent_difference.hpp>
 
 // CmdParser
 #include "cmdparser.hpp"
 
-#include "benchmark_device_adjacent_difference.parallel.hpp"
-#include "benchmark_utils.hpp"
+#include <iostream>
+#include <string>
 
-#ifndef DEFAULT_N
-constexpr std::size_t DEFAULT_N = 1024 * 1024 * 128;
+#include <cstddef>
+
+#ifndef DEFAULT_BYTES
+constexpr std::size_t DEFAULT_BYTES = 1024LL * 1024LL * 1024LL * 2LL;
 #endif
 
 #define CREATE_BENCHMARK(T, left, in_place)                                     \
     {                                                                           \
         const device_adjacent_difference_benchmark<T, left, in_place> instance; \
-        REGISTER_BENCHMARK(benchmarks, size, stream, instance);                 \
+        REGISTER_BENCHMARK(benchmarks, size, seed, stream, instance);           \
     }
 
 // clang-format off
@@ -61,12 +61,13 @@ constexpr std::size_t DEFAULT_N = 1024 * 1024 * 128;
 int main(int argc, char* argv[])
 {
     cli::Parser parser(argc, argv);
-    parser.set_optional<size_t>("size", "size", DEFAULT_N, "number of values");
+    parser.set_optional<size_t>("size", "size", DEFAULT_BYTES, "size in bytes");
     parser.set_optional<int>("trials", "trials", -1, "number of iterations");
     parser.set_optional<std::string>("name_format",
                                      "name_format",
                                      "human",
                                      "either: json,human,txt");
+    parser.set_optional<std::string>("seed", "seed", "random", get_seed_message());
 #ifdef BENCHMARK_CONFIG_TUNING
     // optionally run an evenly split subset of benchmarks, when making multiple program invocations
     parser.set_optional<int>("parallel_instance",
@@ -85,6 +86,8 @@ int main(int argc, char* argv[])
     const size_t size   = parser.get<size_t>("size");
     const int    trials = parser.get<int>("trials");
     bench_naming::set_format(parser.get<std::string>("name_format"));
+    const std::string  seed_type = parser.get<std::string>("seed");
+    const managed_seed seed(seed_type);
 
     // HIP
     const hipStream_t stream = 0; // default
@@ -92,6 +95,7 @@ int main(int argc, char* argv[])
     // Benchmark info
     add_common_benchmark_info();
     benchmark::AddCustomContext("size", std::to_string(size));
+    benchmark::AddCustomContext("seed", seed_type);
 
     std::vector<benchmark::internal::Benchmark*> benchmarks = {};
 #ifdef BENCHMARK_CONFIG_TUNING
@@ -101,6 +105,7 @@ int main(int argc, char* argv[])
                                                         parallel_instance,
                                                         parallel_instances,
                                                         size,
+                                                        seed,
                                                         stream);
 #else // BENCHMARK_CONFIG_TUNING
     using custom_float2  = custom_type<float, float>;
