@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,28 +20,30 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 
+#include "benchmark_device_histogram.parallel.hpp"
+#include "benchmark_utils.hpp"
+
+// CmdParser
+#include "cmdparser.hpp"
+
+// Google Benchmark
+#include <benchmark/benchmark.h>
+
+// HIP API
+#include <cstddef>
+#include <hip/hip_runtime.h>
+
+// rocPRIM
+#include <rocprim/device/device_histogram.hpp>
+
 #include <iostream>
 #include <limits>
 #include <locale>
 #include <string>
 #include <vector>
 
-// Google Benchmark
-#include "benchmark/benchmark.h"
-// CmdParser
-#include "cmdparser.hpp"
-
-// HIP API
-#include <hip/hip_runtime.h>
-
-// rocPRIM
-#include <rocprim/rocprim.hpp>
-
-#include "benchmark_device_histogram.parallel.hpp"
-#include "benchmark_utils.hpp"
-
-#ifndef DEFAULT_N
-const size_t DEFAULT_N = 1024 * 1024 * 32;
+#ifndef DEFAULT_BYTES
+const size_t DEFAULT_BYTES = 1024 * 1024 * 32 * 4;
 #endif
 
 namespace rp = rocprim;
@@ -66,12 +68,16 @@ const int entropy_reductions[] = {0, 2, 4, 6};
 
 template<class T>
 void run_even_benchmark(benchmark::State& state,
-                        size_t            bins,
-                        size_t            scale,
-                        int               entropy_reduction,
-                        hipStream_t       stream,
-                        size_t            size)
+                        size_t            bytes,
+                        const managed_seed&,
+                        hipStream_t stream,
+                        size_t      bins,
+                        size_t      scale,
+                        int         entropy_reduction)
 {
+    // Calculate the number of elements
+    size_t size = bytes / sizeof(T);
+
     using counter_type = unsigned int;
     using level_type =
         typename std::conditional_t<std::is_integral<T>::value && sizeof(T) < sizeof(int), int, T>;
@@ -167,12 +173,16 @@ void run_even_benchmark(benchmark::State& state,
 
 template<class T, unsigned int Channels, unsigned int ActiveChannels>
 void run_multi_even_benchmark(benchmark::State& state,
-                              size_t            bins,
-                              size_t            scale,
-                              int               entropy_reduction,
-                              hipStream_t       stream,
-                              size_t            size)
+                              size_t            bytes,
+                              const managed_seed&,
+                              hipStream_t stream,
+                              size_t      bins,
+                              size_t      scale,
+                              int         entropy_reduction)
 {
+    // Calculate the number of elements
+    size_t size = bytes / sizeof(T);
+
     using counter_type = unsigned int;
     using level_type =
         typename std::conditional_t<std::is_integral<T>::value && sizeof(T) < sizeof(int), int, T>;
@@ -281,14 +291,20 @@ void run_multi_even_benchmark(benchmark::State& state,
 }
 
 template<class T>
-void run_range_benchmark(benchmark::State& state, size_t bins, hipStream_t stream, size_t size)
+void run_range_benchmark(
+    benchmark::State& state, size_t bytes, const managed_seed& seed, hipStream_t stream, size_t bins)
 {
+    // Calculate the number of elements
+    size_t size = bytes / sizeof(T);
+
     using counter_type = unsigned int;
     using level_type =
         typename std::conditional_t<std::is_integral<T>::value && sizeof(T) < sizeof(int), int, T>;
 
     // Generate data
-    std::vector<T> input = get_random_data<T>(size, 0, bins);
+    const auto     random_range = limit_random_range<T>(0, bins);
+    std::vector<T> input
+        = get_random_data<T>(size, random_range.first, random_range.second, seed.get_0());
 
     std::vector<level_type> levels(bins + 1);
     for(size_t i = 0; i < levels.size(); i++)
@@ -382,11 +398,12 @@ void run_range_benchmark(benchmark::State& state, size_t bins, hipStream_t strea
 }
 
 template<class T, unsigned int Channels, unsigned int ActiveChannels>
-void run_multi_range_benchmark(benchmark::State& state,
-                               size_t            bins,
-                               hipStream_t       stream,
-                               size_t            size)
+void run_multi_range_benchmark(
+    benchmark::State& state, size_t bytes, const managed_seed& seed, hipStream_t stream, size_t bins)
 {
+    // Calculate the number of elements
+    size_t size = bytes / sizeof(T);
+
     using counter_type = unsigned int;
     using level_type =
         typename std::conditional_t<std::is_integral<T>::value && sizeof(T) < sizeof(int), int, T>;
@@ -405,7 +422,11 @@ void run_multi_range_benchmark(benchmark::State& state,
     }
 
     // Generate data
-    std::vector<T> input = get_random_data<T>(size * Channels, 0, bins);
+    const auto     random_range = limit_random_range<T>(0, bins);
+    std::vector<T> input        = get_random_data<T>(size * Channels,
+                                              random_range.first,
+                                              random_range.second,
+                                              seed.get_0());
 
     T*            d_input;
     level_type*   d_levels[ActiveChannels];
@@ -511,7 +532,7 @@ void run_multi_range_benchmark(benchmark::State& state,
                                   + ",bins:" + std::to_string(BINS) + ",cfg:default_config}")  \
             .c_str(),                                                                          \
         [=](benchmark::State& state)                                                           \
-        { run_even_benchmark<T>(state, BINS, SCALE, entropy_reduction, stream, size); }));
+        { run_even_benchmark<T>(state, bytes, seed, stream, BINS, SCALE, entropy_reduction); }));
 
 #define BENCHMARK_EVEN_TYPE(VECTOR, T, S)      \
     CREATE_EVEN_BENCHMARK(VECTOR, T, 10, S);   \
@@ -520,8 +541,9 @@ void run_multi_range_benchmark(benchmark::State& state,
     CREATE_EVEN_BENCHMARK(VECTOR, T, 10000, S);
 
 void add_even_benchmarks(std::vector<benchmark::internal::Benchmark*>& benchmarks,
-                         hipStream_t                                   stream,
-                         size_t                                        size)
+                         size_t                                        bytes,
+                         const managed_seed&                           seed,
+                         hipStream_t                                   stream)
 {
     for(int entropy_reduction : entropy_reductions)
     {
@@ -547,21 +569,26 @@ void add_even_benchmarks(std::vector<benchmark::internal::Benchmark*>& benchmark
         [=](benchmark::State& state)                                                          \
         {                                                                                     \
             run_multi_even_benchmark<T, CHANNELS, ACTIVE_CHANNELS>(state,                     \
+                                                                   bytes,                      \
+                                                                   seed,                      \
+                                                                   stream,                    \
                                                                    BINS,                      \
                                                                    SCALE,                     \
-                                                                   entropy_reduction,         \
-                                                                   stream,                    \
-                                                                   size);                     \
+                                                                   entropy_reduction);        \
         })
 
-#define BENCHMARK_MULTI_EVEN_TYPE(C, A, T, S)                                                  \
-    CREATE_MULTI_EVEN_BENCHMARK(C, A, T, 10, S), CREATE_MULTI_EVEN_BENCHMARK(C, A, T, 100, S), \
-        CREATE_MULTI_EVEN_BENCHMARK(C, A, T, 1000, S),                                         \
-        CREATE_MULTI_EVEN_BENCHMARK(C, A, T, 10000, S)
+// clang-format off
+#define BENCHMARK_MULTI_EVEN_TYPE(C, A, T, S)      \
+    CREATE_MULTI_EVEN_BENCHMARK(C, A, T, 10, S),   \
+    CREATE_MULTI_EVEN_BENCHMARK(C, A, T, 100, S),  \
+    CREATE_MULTI_EVEN_BENCHMARK(C, A, T, 1000, S), \
+    CREATE_MULTI_EVEN_BENCHMARK(C, A, T, 10000, S)
+// clang-format on
 
 void add_multi_even_benchmarks(std::vector<benchmark::internal::Benchmark*>& benchmarks,
-                               hipStream_t                                   stream,
-                               size_t                                        size)
+                               size_t                                        bytes,
+                               const managed_seed&                           seed,
+                               hipStream_t                                   stream)
 {
     for(int entropy_reduction : entropy_reductions)
     {
@@ -581,15 +608,20 @@ void add_multi_even_benchmarks(std::vector<benchmark::internal::Benchmark*>& ben
         bench_naming::format_name("{lvl:device,algo:histogram_range,value_type:" #T ",bins:" \
                                   + std::to_string(BINS) + ",cfg:default_config}")           \
             .c_str(),                                                                        \
-        [=](benchmark::State& state) { run_range_benchmark<T>(state, BINS, stream, size); })
+        [=](benchmark::State& state) { run_range_benchmark<T>(state, bytes, seed, stream, BINS); })
 
-#define BENCHMARK_RANGE_TYPE(T)                                    \
-    CREATE_RANGE_BENCHMARK(T, 10), CREATE_RANGE_BENCHMARK(T, 100), \
-        CREATE_RANGE_BENCHMARK(T, 1000), CREATE_RANGE_BENCHMARK(T, 10000)
+// clang-format off
+#define BENCHMARK_RANGE_TYPE(T)      \
+    CREATE_RANGE_BENCHMARK(T, 10),   \
+    CREATE_RANGE_BENCHMARK(T, 100),  \
+    CREATE_RANGE_BENCHMARK(T, 1000), \
+    CREATE_RANGE_BENCHMARK(T, 10000)
+// clang-format on
 
 void add_range_benchmarks(std::vector<benchmark::internal::Benchmark*>& benchmarks,
-                          hipStream_t                                   stream,
-                          size_t                                        size)
+                          size_t                                        bytes,
+                          const managed_seed&                           seed,
+                          hipStream_t                                   stream)
 {
     std::vector<benchmark::internal::Benchmark*> bs = {
         BENCHMARK_RANGE_TYPE(long long),
@@ -611,16 +643,26 @@ void add_range_benchmarks(std::vector<benchmark::internal::Benchmark*>& benchmar
                                   ",bins:"                                                    \
                                   + std::to_string(BINS) + ",cfg:default_config}")            \
             .c_str(),                                                                         \
-        [=](benchmark::State& state)                                                          \
-        { run_multi_range_benchmark<T, CHANNELS, ACTIVE_CHANNELS>(state, BINS, stream, size); })
+        [=](benchmark::State& state) {                                                        \
+            run_multi_range_benchmark<T, CHANNELS, ACTIVE_CHANNELS>(state,                    \
+                                                                    bytes,                     \
+                                                                    seed,                     \
+                                                                    stream,                   \
+                                                                    BINS);                    \
+        })
 
-#define BENCHMARK_MULTI_RANGE_TYPE(C, A, T)                                                \
-    CREATE_MULTI_RANGE_BENCHMARK(C, A, T, 10), CREATE_MULTI_RANGE_BENCHMARK(C, A, T, 100), \
-        CREATE_MULTI_RANGE_BENCHMARK(C, A, T, 1000), CREATE_MULTI_RANGE_BENCHMARK(C, A, T, 10000)
+// clang-format off
+#define BENCHMARK_MULTI_RANGE_TYPE(C, A, T)      \
+    CREATE_MULTI_RANGE_BENCHMARK(C, A, T, 10),   \
+    CREATE_MULTI_RANGE_BENCHMARK(C, A, T, 100),  \
+    CREATE_MULTI_RANGE_BENCHMARK(C, A, T, 1000), \
+    CREATE_MULTI_RANGE_BENCHMARK(C, A, T, 10000)
+// clang-format on
 
 void add_multi_range_benchmarks(std::vector<benchmark::internal::Benchmark*>& benchmarks,
-                                hipStream_t                                   stream,
-                                size_t                                        size)
+                                size_t                                        bytes,
+                                const managed_seed&                           seed,
+                                hipStream_t                                   stream)
 {
     std::vector<benchmark::internal::Benchmark*> bs = {
         BENCHMARK_MULTI_RANGE_TYPE(4, 4, int),
@@ -636,12 +678,13 @@ void add_multi_range_benchmarks(std::vector<benchmark::internal::Benchmark*>& be
 int main(int argc, char* argv[])
 {
     cli::Parser parser(argc, argv);
-    parser.set_optional<size_t>("size", "size", DEFAULT_N, "number of values");
+    parser.set_optional<size_t>("size", "size", DEFAULT_BYTES, "number of bytes");
     parser.set_optional<int>("trials", "trials", -1, "number of iterations");
     parser.set_optional<std::string>("name_format",
                                      "name_format",
                                      "human",
                                      "either: json,human,txt");
+    parser.set_optional<std::string>("seed", "seed", "random", get_seed_message());
 #ifdef BENCHMARK_CONFIG_TUNING
     // optionally run an evenly split subset of benchmarks, when making multiple program invocations
     parser.set_optional<int>("parallel_instance",
@@ -657,16 +700,19 @@ int main(int argc, char* argv[])
 
     // Parse argv
     benchmark::Initialize(&argc, argv);
-    const size_t size   = parser.get<size_t>("size");
+    const size_t bytes   = parser.get<size_t>("size");
     const int    trials = parser.get<int>("trials");
     bench_naming::set_format(parser.get<std::string>("name_format"));
+    const std::string  seed_type = parser.get<std::string>("seed");
+    const managed_seed seed(seed_type);
 
     // HIP
     hipStream_t stream = 0; // default
 
     // Benchmark info
     add_common_benchmark_info();
-    benchmark::AddCustomContext("size", std::to_string(size));
+    benchmark::AddCustomContext("bytes", std::to_string(bytes));
+    benchmark::AddCustomContext("seed", seed_type);
 
     // Add benchmarks
     std::vector<benchmark::internal::Benchmark*> benchmarks;
@@ -676,13 +722,14 @@ int main(int argc, char* argv[])
     config_autotune_register::register_benchmark_subset(benchmarks,
                                                         parallel_instance,
                                                         parallel_instances,
-                                                        size,
+                                                        bytes,
+                                                        seed,
                                                         stream);
 #else // BENCHMARK_CONFIG_TUNING
-    add_even_benchmarks(benchmarks, stream, size);
-    add_multi_even_benchmarks(benchmarks, stream, size);
-    add_range_benchmarks(benchmarks, stream, size);
-    add_multi_range_benchmarks(benchmarks, stream, size);
+    add_even_benchmarks(benchmarks, bytes, seed, stream);
+    add_multi_even_benchmarks(benchmarks, bytes, seed, stream);
+    add_range_benchmarks(benchmarks, bytes, seed, stream);
+    add_multi_range_benchmarks(benchmarks, bytes, seed, stream);
 #endif // BENCHMARK_CONFIG_TUNING
 
     // Use manual timing
