@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2018-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,19 +20,21 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 
-#include <cstdio>
-#include <cstdlib>
+#include "benchmark_utils.hpp"
+// CmdParser
+#include "cmdparser.hpp"
+
+// Google Benchmark
+#include <benchmark/benchmark.h>
+// rocPRIM
+#include <rocprim/block/block_load.hpp>
+#include <rocprim/block/block_store.hpp>
+
 #include <iostream>
 #include <string>
 
-// Google Benchmark
-#include "benchmark/benchmark.h"
-// CmdParser
-#include "cmdparser.hpp"
-// rocPRIM
-#include <rocprim/rocprim.hpp>
-
-#include "benchmark_utils.hpp"
+#include <cstdio>
+#include <cstdlib>
 
 enum memory_operation_method
 {
@@ -65,7 +67,7 @@ template<class T, unsigned int ItemsPerThread, unsigned int BlockSize>
 struct operation<no_operation, T, ItemsPerThread, BlockSize>
 {
     ROCPRIM_HOST_DEVICE inline
-    void operator()(T (&)[ItemsPerThread], void* = nullptr, unsigned int = 0, T* = nullptr)
+    void operator()(T (&)[ItemsPerThread], void* = nullptr, unsigned int = 0, T* = nullptr) const
     {
         // No operation
     }
@@ -80,7 +82,7 @@ struct operation<custom_operation, T, ItemsPerThread, BlockSize>
     ROCPRIM_HOST_DEVICE inline
     void operator()(T (&input)[ItemsPerThread],
                     void* shared_storage = nullptr, unsigned int shared_storage_size = 0,
-                    T* global_mem_output = nullptr)
+                    T* global_mem_output = nullptr) const
     {
         (void) shared_storage;
         (void) shared_storage_size;
@@ -105,7 +107,7 @@ struct operation<block_scan, T, ItemsPerThread, BlockSize>
     ROCPRIM_HOST_DEVICE inline
     void operator()(T (&input)[ItemsPerThread],
                     void* shared_storage = nullptr, unsigned int shared_storage_size = 0,
-                    T* global_mem_output = nullptr)
+                    T* global_mem_output = nullptr) const
     {
         (void) global_mem_output;
         using block_scan_type = typename rocprim::block_scan<
@@ -164,8 +166,8 @@ struct operation<atomics_inter_warp_collision, T, ItemsPerThread, BlockSize>
         (void) shared_storage;
         (void) shared_storage_size;
         (void) input;
-        unsigned int index = (threadIdx.x % warpSize) * ItemsPerThread +
-                             blockIdx.x * blockDim.x * ItemsPerThread;
+        unsigned int index = (threadIdx.x % rocprim::arch::wavefront::min_size()) * ItemsPerThread
+                             + blockIdx.x * blockDim.x * ItemsPerThread;
         ROCPRIM_UNROLL
         for(unsigned int i = 0; i < ItemsPerThread; i++)
         {
@@ -323,31 +325,22 @@ void operation_kernel(T* input, T* output, CustomOp op)
     store.store(output + offset, items, storage.store);
 }
 
-template<
-    class T,
-    unsigned int BlockSize,
-    unsigned int ItemsPerThread,
-    memory_operation_method MemOp,
-    kernel_operation KernelOp = no_operation
->
-void run_benchmark(benchmark::State& state,
-                   size_t size,
-                   const hipStream_t stream)
+template<class T,
+         unsigned int            BlockSize,
+         unsigned int            ItemsPerThread,
+         memory_operation_method MemOp,
+         kernel_operation        KernelOp = no_operation>
+void run_benchmark(benchmark::State&   state,
+                   size_t              size,
+                   const managed_seed& seed,
+                   const hipStream_t   stream)
 {
-    const size_t grid_size = size / (BlockSize * ItemsPerThread);
-    std::vector<T> input;
-    if(std::is_floating_point<T>::value)
-    {
-        input = get_random_data<T>(size, (T)-1000, (T)+1000);
-    }
-    else
-    {
-        input = get_random_data<T>(
-            size,
-            std::numeric_limits<T>::min(),
-            std::numeric_limits<T>::max()
-        );
-    }
+    const size_t   grid_size = size / (BlockSize * ItemsPerThread);
+    std::vector<T> input     = get_random_data<T>(size,
+                                              generate_limits<T>::min(),
+                                              generate_limits<T>::max(),
+                                              seed.get_0());
+
     T * d_input;
     T * d_output;
     HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&d_input), size * sizeof(T)));
@@ -416,22 +409,14 @@ void run_benchmark(benchmark::State& state,
 
 template<class T>
 void run_benchmark_memcpy(benchmark::State& state,
-                           size_t size,
-                           const hipStream_t stream)
+                          size_t            size,
+                          const managed_seed&,
+                          const hipStream_t stream)
 {
-    std::vector<T> input;
-    if(std::is_floating_point<T>::value)
-    {
-        input = get_random_data<T>(size, (T)-1000, (T)+1000);
-    }
-    else
-    {
-        input = get_random_data<T>(
-            size,
-            std::numeric_limits<T>::min(),
-            std::numeric_limits<T>::max()
-        );
-    }
+    // Allocate device buffers
+    // Note: since this benchmark only tests performance by memcpying between device buffers,
+    // we don't really need to transfer data into these from the host - whatever happens
+    // to be in device memory will do.
     T * d_input;
     T * d_output;
     HIP_CHECK(hipMalloc(reinterpret_cast<void**>(&d_input), size * sizeof(T)));
@@ -487,6 +472,7 @@ void run_benchmark_memcpy(benchmark::State& state,
             .c_str(),                                                                     \
         run_benchmark<T, BLOCK_SIZE, IPT, METHOD, OPERATION>,                             \
         SIZE,                                                                             \
+        seed,                                                                             \
         stream)
 
 #define CREATE_BENCHMARK_MEMCPY(T, SIZE)                                              \
@@ -496,6 +482,7 @@ void run_benchmark_memcpy(benchmark::State& state,
             .c_str(),                                                                 \
         run_benchmark_memcpy<T>,                                                      \
         SIZE,                                                                         \
+        seed,                                                                         \
         stream)
 
 template<class T>
@@ -512,18 +499,22 @@ int main(int argc, char *argv[])
                                      "name_format",
                                      "human",
                                      "either: json,human,txt");
+    parser.set_optional<std::string>("seed", "seed", "random", get_seed_message());
     parser.run_and_exit_if_error();
 
     // Parse argv
     benchmark::Initialize(&argc, argv);
     const int trials = parser.get<int>("trials");
     bench_naming::set_format(parser.get<std::string>("name_format"));
+    const std::string  seed_type = parser.get<std::string>("seed");
+    const managed_seed seed(seed_type);
 
     // HIP
     hipStream_t stream = 0; // default
 
     // Benchmark info
     add_common_benchmark_info();
+    benchmark::AddCustomContext("seed", seed_type);
 
     // Add benchmarks
     std::vector<benchmark::internal::Benchmark*> benchmarks =
