@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -23,9 +23,7 @@
 #ifndef ROCPRIM_BENCHMARK_DETAIL_BENCHMARK_DEVICE_MERGE_SORT_BLOCK_SORT_PARALLEL_HPP_
 #define ROCPRIM_BENCHMARK_DETAIL_BENCHMARK_DEVICE_MERGE_SORT_BLOCK_SORT_PARALLEL_HPP_
 
-#include <cstddef>
-#include <string>
-#include <vector>
+#include "benchmark_utils.hpp"
 
 // Google Benchmark
 #include <benchmark/benchmark.h>
@@ -34,9 +32,12 @@
 #include <hip/hip_runtime.h>
 
 // rocPRIM
-#include <rocprim/rocprim.hpp>
+#include <rocprim/device/device_merge_sort.hpp>
 
-#include "benchmark_utils.hpp"
+#include <string>
+#include <vector>
+
+#include <cstddef>
 
 namespace rp = rocprim;
 
@@ -59,8 +60,7 @@ std::string config_name()
 {
     const rocprim::detail::merge_sort_block_sort_config_params config = Config();
     return "{bs:" + std::to_string(config.block_sort_config.block_size)
-           + ",ipt:" + std::to_string(config.block_sort_config.items_per_thread)
-           + ",method:" + std::string(get_block_sort_method_name(config.block_sort_method)) + "}";
+           + ",ipt:" + std::to_string(config.block_sort_config.items_per_thread) + "}";
 }
 
 template<>
@@ -88,25 +88,22 @@ struct device_merge_sort_block_sort_benchmark : public config_autotune_interface
 
     // keys benchmark
     template<typename val = Value>
-    auto do_run(benchmark::State& state, size_t size, const hipStream_t stream) const ->
+    auto do_run(benchmark::State&   state,
+                size_t              bytes,
+                const managed_seed& seed,
+                hipStream_t         stream) const ->
         typename std::enable_if<std::is_same<val, ::rocprim::empty_type>::value, void>::type
     {
         using key_type = Key;
 
+        // Calculate the number of elements 
+        size_t size = bytes / sizeof(key_type);
         // Generate data
-        std::vector<key_type> keys_input;
-        if(std::is_floating_point<key_type>::value)
-        {
-            keys_input = get_random_data<key_type>(size,
-                                                   static_cast<key_type>(-1000),
-                                                   static_cast<key_type>(1000));
-        }
-        else
-        {
-            keys_input = get_random_data<key_type>(size,
-                                                   std::numeric_limits<key_type>::min(),
-                                                   std::numeric_limits<key_type>::max());
-        }
+        std::vector<key_type> keys_input
+            = get_random_data<key_type>(size,
+                                        generate_limits<key_type>::min(),
+                                        generate_limits<key_type>::max(),
+                                        seed.get_0());
 
         key_type* d_keys_input;
         key_type* d_keys_output;
@@ -180,26 +177,23 @@ struct device_merge_sort_block_sort_benchmark : public config_autotune_interface
 
     // pairs benchmark
     template<typename val = Value>
-    auto do_run(benchmark::State& state, size_t size, const hipStream_t stream) const ->
+    auto do_run(benchmark::State&   state,
+                size_t              bytes,
+                const managed_seed& seed,
+                hipStream_t         stream) const ->
         typename std::enable_if<!std::is_same<val, ::rocprim::empty_type>::value, void>::type
     {
         using key_type   = Key;
         using value_type = Value;
 
+        // Calculate the number of elements 
+        size_t size = bytes / sizeof(key_type);
         // Generate data
-        std::vector<key_type> keys_input;
-        if(std::is_floating_point<key_type>::value)
-        {
-            keys_input = get_random_data<key_type>(size,
-                                                   static_cast<key_type>(-1000),
-                                                   static_cast<key_type>(1000));
-        }
-        else
-        {
-            keys_input = get_random_data<key_type>(size,
-                                                   std::numeric_limits<key_type>::min(),
-                                                   std::numeric_limits<key_type>::max());
-        }
+        std::vector<key_type> keys_input
+            = get_random_data<key_type>(size,
+                                        generate_limits<key_type>::min(),
+                                        generate_limits<key_type>::max(),
+                                        seed.get_0());
 
         std::vector<value_type> values_input(size);
         std::iota(values_input.begin(), values_input.end(), 0);
@@ -289,9 +283,12 @@ struct device_merge_sort_block_sort_benchmark : public config_autotune_interface
         HIP_CHECK(hipFree(d_values_output));
     }
 
-    void run(benchmark::State& state, size_t size, hipStream_t stream) const override
+    void run(benchmark::State&   state,
+             size_t              bytes,
+             const managed_seed& seed,
+             hipStream_t         stream) const override
     {
-        do_run(state, size, stream);
+        do_run(state, bytes, seed, stream);
     }
 };
 
