@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,8 +20,10 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 
-#include <cstddef>
-#include <string>
+#include "benchmark_device_reduce.parallel.hpp"
+#include "benchmark_utils.hpp"
+// CmdParser
+#include "cmdparser.hpp"
 
 // Google Benchmark
 #include <benchmark/benchmark.h>
@@ -29,34 +31,30 @@
 // HIP API
 #include <hip/hip_runtime.h>
 
-// rocPRIM HIP API
-#include <rocprim/rocprim.hpp>
+#include <string>
 
-// CmdParser
-#include "cmdparser.hpp"
+#include <cstddef>
 
-#include "benchmark_utils.hpp"
-#include "benchmark_device_reduce.parallel.hpp"
-
-#ifndef DEFAULT_N
-const size_t DEFAULT_N = 1024 * 1024 * 128;
+#ifndef DEFAULT_BYTES
+const size_t DEFAULT_BYTES = 1024 * 1024 * 128 * 4;
 #endif
 
-#define CREATE_BENCHMARK(T, REDUCE_OP)                          \
-    {                                                           \
-        const device_reduce_benchmark<T, REDUCE_OP> instance;   \
-        REGISTER_BENCHMARK(benchmarks, size, stream, instance); \
+#define CREATE_BENCHMARK(T, REDUCE_OP)                                \
+    {                                                                 \
+        const device_reduce_benchmark<T, REDUCE_OP> instance;         \
+        REGISTER_BENCHMARK(benchmarks, bytes, seed, stream, instance); \
     }
 
 int main(int argc, char *argv[])
 {
     cli::Parser parser(argc, argv);
-    parser.set_optional<size_t>("size", "size", DEFAULT_N, "number of values");
+    parser.set_optional<size_t>("size", "size", DEFAULT_BYTES, "number of bytes");
     parser.set_optional<int>("trials", "trials", -1, "number of iterations");
     parser.set_optional<std::string>("name_format",
                                      "name_format",
                                      "human",
                                      "either: json,human,txt");
+    parser.set_optional<std::string>("seed", "seed", "random", get_seed_message());
 #ifdef BENCHMARK_CONFIG_TUNING
     // optionally run an evenly split subset of benchmarks, when making multiple program invocations
     parser.set_optional<int>("parallel_instance",
@@ -72,16 +70,19 @@ int main(int argc, char *argv[])
 
     // Parse argv
     benchmark::Initialize(&argc, argv);
-    const size_t size = parser.get<size_t>("size");
+    const size_t bytes = parser.get<size_t>("size");
     const int trials = parser.get<int>("trials");
     bench_naming::set_format(parser.get<std::string>("name_format"));
+    const std::string  seed_type = parser.get<std::string>("seed");
+    const managed_seed seed(seed_type);
 
     // HIP
     hipStream_t stream = 0; // default
 
     // Benchmark info
     add_common_benchmark_info();
-    benchmark::AddCustomContext("size", std::to_string(size));
+    benchmark::AddCustomContext("bytes", std::to_string(bytes));
+    benchmark::AddCustomContext("seed", seed_type);
 
     // Add benchmarks
     std::vector<benchmark::internal::Benchmark*> benchmarks = {};
@@ -91,7 +92,8 @@ int main(int argc, char *argv[])
     config_autotune_register::register_benchmark_subset(benchmarks,
                                                         parallel_instance,
                                                         parallel_instances,
-                                                        size,
+                                                        bytes,
+                                                        seed,
                                                         stream);
 #else
     using custom_float2 = custom_type<float, float>;
