@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -23,9 +23,7 @@
 #ifndef ROCPRIM_BENCHMARK_DEVICE_REDUCE_PARALLEL_HPP_
 #define ROCPRIM_BENCHMARK_DEVICE_REDUCE_PARALLEL_HPP_
 
-#include <cstddef>
-#include <string>
-#include <vector>
+#include "benchmark_utils.hpp"
 
 // Google Benchmark
 #include <benchmark/benchmark.h>
@@ -34,9 +32,13 @@
 #include <hip/hip_runtime.h>
 
 // rocPRIM HIP API
-#include <rocprim/rocprim.hpp>
+#include <rocprim/device/detail/device_config_helper.hpp>
+#include <rocprim/device/device_reduce.hpp>
 
-#include "benchmark_utils.hpp"
+#include <string>
+#include <vector>
+
+#include <cstddef>
 
 constexpr const char* get_reduce_method_name(rocprim::block_reduce_algorithm alg)
 {
@@ -82,12 +84,18 @@ struct device_reduce_benchmark : public config_autotune_interface
     static constexpr unsigned int batch_size = 10;
     static constexpr unsigned int warmup_size = 5;
 
-    void run(benchmark::State& state,
-             size_t size,
-             const hipStream_t stream) const override
+    void run(benchmark::State&   state,
+             size_t              bytes,
+             const managed_seed& seed,
+             hipStream_t         stream) const override
     {
+        // Calculate the number of elements 
+        size_t size = bytes / sizeof(T);
+
         BinaryFunction reduce_op{};
-        std::vector<T> input = get_random_data<T>(size, T(0), T(1000));
+        const auto     random_range = limit_random_range<T>(0, 1000);
+        std::vector<T> input
+            = get_random_data<T>(size, random_range.first, random_range.second, seed.get_0());
 
         T * d_input;
         T * d_output;
