@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,26 +20,27 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 
+#include "benchmark_utils.hpp"
+// CmdParser
+#include "cmdparser.hpp"
+
+// Google Benchmark
+#include <benchmark/benchmark.h>
+
+// HIP API
+#include <hip/hip_runtime.h>
+
+// rocPRIM
+#include <rocprim/device/device_segmented_reduce.hpp>
+
 #include <iostream>
 #include <limits>
 #include <locale>
 #include <string>
 #include <vector>
 
-// Google Benchmark
-#include "benchmark/benchmark.h"
-// CmdParser
-#include "cmdparser.hpp"
-#include "benchmark_utils.hpp"
-
-// HIP API
-#include <hip/hip_runtime.h>
-
-// rocPRIM
-#include <rocprim/rocprim.hpp>
-
-#ifndef DEFAULT_N
-const size_t DEFAULT_N = 1024 * 1024 * 32;
+#ifndef DEFAULT_BYTES
+const size_t DEFAULT_BYTES = 1024 * 1024 * 32 * 4;
 #endif
 
 namespace rp = rocprim;
@@ -48,14 +49,20 @@ const unsigned int batch_size = 10;
 const unsigned int warmup_size = 5;
 
 template<class T>
-void run_benchmark(benchmark::State& state, size_t desired_segments, hipStream_t stream, size_t size)
+void run_benchmark(benchmark::State&   state,
+                   size_t              desired_segments,
+                   size_t              bytes,
+                   const managed_seed& seed,
+                   hipStream_t         stream)
 {
     using offset_type = int;
     using value_type = T;
 
+    // Calculate the number of elements 
+    size_t size = bytes / sizeof(T);
+
     // Generate data
-    const unsigned int seed = 123;
-    std::default_random_engine gen(seed);
+    engine_type gen(seed.get_0());
 
     const double avg_segment_length = static_cast<double>(size) / desired_segments;
     std::uniform_real_distribution<double> segment_length_dis(0, avg_segment_length * 2);
@@ -188,8 +195,9 @@ void run_benchmark(benchmark::State& state, size_t desired_segments, hipStream_t
             .c_str(),                                                                  \
         run_benchmark<T>,                                                              \
         SEGMENTS,                                                                      \
-        stream,                                                                        \
-        size)
+        bytes,                                                                          \
+        seed,                                                                          \
+        stream)
 
 #define BENCHMARK_TYPE(type) \
     CREATE_BENCHMARK(type, 1), \
@@ -199,8 +207,9 @@ void run_benchmark(benchmark::State& state, size_t desired_segments, hipStream_t
     CREATE_BENCHMARK(type, 10000)
 
 void add_benchmarks(std::vector<benchmark::internal::Benchmark*>& benchmarks,
-                    hipStream_t stream,
-                    size_t size)
+                    size_t                                        bytes,
+                    const managed_seed&                           seed,
+                    hipStream_t                                   stream)
 {
     using custom_float2 = custom_type<float, float>;
     using custom_double2 = custom_type<double, double>;
@@ -223,30 +232,35 @@ void add_benchmarks(std::vector<benchmark::internal::Benchmark*>& benchmarks,
 int main(int argc, char *argv[])
 {
     cli::Parser parser(argc, argv);
-    parser.set_optional<size_t>("size", "size", DEFAULT_N, "number of values");
+    parser.set_optional<size_t>("size", "size", DEFAULT_BYTES, "number of bytes");
     parser.set_optional<int>("trials", "trials", -1, "number of iterations");
     parser.set_optional<std::string>("name_format",
                                      "name_format",
                                      "human",
                                      "either: json,human,txt");
+    // fixed seed as a random seed adds a lot of variance
+    parser.set_optional<std::string>("seed", "seed", "321", get_seed_message());
     parser.run_and_exit_if_error();
 
     // Parse argv
     benchmark::Initialize(&argc, argv);
-    const size_t size = parser.get<size_t>("size");
+    const size_t bytes = parser.get<size_t>("size");
     const int trials = parser.get<int>("trials");
     bench_naming::set_format(parser.get<std::string>("name_format"));
+    const std::string  seed_type = parser.get<std::string>("seed");
+    const managed_seed seed(seed_type);
 
     // HIP
     hipStream_t stream = 0; // default
 
     // Benchmark info
     add_common_benchmark_info();
-    benchmark::AddCustomContext("size", std::to_string(size));
+    benchmark::AddCustomContext("bytes", std::to_string(bytes));
+    benchmark::AddCustomContext("seed", seed_type);
 
     // Add benchmarks
     std::vector<benchmark::internal::Benchmark*> benchmarks;
-    add_benchmarks(benchmarks, stream, size);
+    add_benchmarks(benchmarks, bytes, seed, stream);
 
     // Use manual timing
     for(auto& b : benchmarks)
