@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,23 +21,29 @@
 #ifndef ROCPRIM_BENCHMARK_UTILS_HPP_
 #define ROCPRIM_BENCHMARK_UTILS_HPP_
 
+#include <benchmark/benchmark.h>
+
+// rocPRIM
+#include <rocprim/block/block_load.hpp>
+#include <rocprim/block/block_scan.hpp>
+#include <rocprim/device/config_types.hpp>
+#include <rocprim/device/detail/device_config_helper.hpp> // partition_config_params
+#include <rocprim/intrinsics/arch.hpp>
+#include <rocprim/types.hpp>
+
 #include <algorithm>
 #include <iostream>
+#include <iterator>
 #include <memory>
+#include <numeric>
 #include <random>
 #include <regex>
 #include <sstream>
 #include <string>
 #include <type_traits>
+#include <utility>
 #include <vector>
 
-#ifdef WIN32
-#include <numeric>
-#endif
-
-#include "benchmark/benchmark.h"
-#include <rocprim/rocprim.hpp>
-
 #define HIP_CHECK(condition)                                                                \
     {                                                                                       \
         hipError_t error = condition;                                                       \
@@ -52,6 +58,49 @@
 #define TUNING_SHARED_MEMORY_MAX 65536u
 // Support half operators on host side
 
+inline const char* get_seed_message()
+{
+    return "seed for input generation, either an unsigned integer value for determinisic results "
+           "or 'random' for different inputs for each repetition";
+}
+
+/// \brief Provides a sequence of seeds.
+class managed_seed
+{
+public:
+    /// \param[in] seed_string Either "random" to get random seeds,
+    ///   or an unsigned integer to get (a sequence) of deterministic seeds.
+    managed_seed(const std::string& seed_string)
+    {
+        is_random = seed_string == "random";
+        if(!is_random)
+        {
+            const unsigned int seed = std::stoul(seed_string);
+            std::seed_seq      seq{seed};
+            seq.generate(seeds.begin(), seeds.end());
+        }
+    }
+
+    unsigned int get_0() const
+    {
+        return is_random ? std::random_device{}() : seeds[0];
+    }
+
+    unsigned int get_1() const
+    {
+        return is_random ? std::random_device{}() : seeds[1];
+    }
+
+    unsigned int get_2() const
+    {
+        return is_random ? std::random_device{}() : seeds[2];
+    }
+
+private:
+    std::array<unsigned int, 3> seeds;
+    bool                        is_random;
+};
+
 ROCPRIM_HOST inline
 rocprim::native_half half_to_native(const rocprim::half& x)
 {
@@ -118,15 +167,20 @@ struct is_valid_for_int_distribution :
         std::is_same<unsigned long long, T>::value
     > {};
 
-using engine_type = std::default_random_engine;
+template<typename Iterator>
+using it_value_t = typename std::iterator_traits<Iterator>::value_type;
 
-// get_random_data() generates only part of sequence and replicates it,
+using engine_type = std::minstd_rand;
+
+// generate_random_data_n() generates only part of sequence and replicates it,
 // because benchmarks usually do not need "true" random sequence.
-template<class T, class U, class V>
-inline auto get_random_data(size_t size, U min, V max, size_t max_random_size = 1024 * 1024)
-    -> typename std::enable_if<rocprim::is_integral<T>::value, std::vector<T>>::type
+template<class OutputIter, class U, class V, class Generator>
+inline auto generate_random_data_n(
+    OutputIter it, size_t size, U min, V max, Generator& gen, size_t max_random_size = 1024 * 1024)
+    -> typename std::enable_if_t<rocprim::is_integral<it_value_t<OutputIter>>::value, OutputIter>
 {
-    engine_type gen{std::random_device{}()};
+    using T = it_value_t<OutputIter>;
+
     using dis_type = typename std::conditional<
         is_valid_for_int_distribution<T>::value,
         T,
@@ -135,48 +189,50 @@ inline auto get_random_data(size_t size, U min, V max, size_t max_random_size =
             unsigned int>::type
         >::type;
     std::uniform_int_distribution<dis_type> distribution((T)min, (T)max);
-    std::vector<T> data(size);
-    std::generate(
-        data.begin(), data.begin() + std::min(size, max_random_size),
-        [&]() { return distribution(gen); }
-    );
+    std::generate_n(it, std::min(size, max_random_size), [&]() { return distribution(gen); });
     for(size_t i = max_random_size; i < size; i += max_random_size)
     {
-        std::copy_n(data.begin(), std::min(size - i, max_random_size), data.begin() + i);
+        std::copy_n(it, std::min(size - i, max_random_size), it + i);
     }
-    return data;
+    return it + size;
 }
 
-template<class T, class U, class V>
-inline auto get_random_data(size_t size, U min, V max, size_t max_random_size = 1024 * 1024)
-    -> typename std::enable_if<rocprim::is_floating_point<T>::value, std::vector<T>>::type
+template<class OutputIterator, class U, class V, class Generator>
+inline auto generate_random_data_n(OutputIterator it,
+                                   size_t         size,
+                                   U              min,
+                                   V              max,
+                                   Generator&     gen,
+                                   size_t         max_random_size = 1024 * 1024)
+    -> std::enable_if_t<rocprim::is_floating_point<it_value_t<OutputIterator>>::value,
+                        OutputIterator>
 {
-    engine_type gen{std::random_device{}()};
+    using T = typename std::iterator_traits<OutputIterator>::value_type;
+
     // Generate floats when T is half
-    using dis_type = typename std::conditional<std::is_same<rocprim::half, T>::value, float, T>::type;
+    using dis_type = std::conditional_t<std::is_same<rocprim::half, T>::value
+                                            || std::is_same<rocprim::bfloat16, T>::value,
+                                        float,
+                                        T>;
     std::uniform_real_distribution<dis_type> distribution((dis_type)min, (dis_type)max);
-    std::vector<T> data(size);
-    std::generate(
-        data.begin(), data.begin() + std::min(size, max_random_size),
-        [&]() { return distribution(gen); }
-    );
+    std::generate_n(it, std::min(size, max_random_size), [&]() { return distribution(gen); });
     for(size_t i = max_random_size; i < size; i += max_random_size)
     {
-        std::copy_n(data.begin(), std::min(size - i, max_random_size), data.begin() + i);
+        std::copy_n(it, std::min(size - i, max_random_size), it + i);
     }
-    return data;
+    return it + size;
 }
 
 template<class T>
-inline std::vector<T> get_random_data01(size_t size, float p, size_t max_random_size = 1024 * 1024)
+inline std::vector<T>
+    get_random_data01(size_t size, float p, unsigned int seed, size_t max_random_size = 1024 * 1024)
 {
-    engine_type gen{std::random_device{}()};
+    engine_type                 gen(seed);
     std::bernoulli_distribution distribution(p);
-    std::vector<T> data(size);
-    std::generate(
-        data.begin(), data.begin() + std::min(size, max_random_size),
-        [&]() { return distribution(gen); }
-    );
+    std::vector<T>              data(size);
+    std::generate(data.begin(),
+                  data.begin() + std::min(size, max_random_size),
+                  [&]() { return distribution(gen); });
     for(size_t i = max_random_size; i < size; i += max_random_size)
     {
         std::copy_n(data.begin(), std::min(size - i, max_random_size), data.begin() + i);
@@ -184,12 +240,6 @@ inline std::vector<T> get_random_data01(size_t size, float p, size_t max_random_
     return data;
 }
 
-template<class T>
-inline T get_random_value(T min, T max)
-{
-    return get_random_data(1, min, max)[0];
-}
-
 template<class T, class U = T>
 struct custom_type
 {
@@ -226,82 +276,292 @@ struct custom_type
     {
         return x == rhs.x && y == rhs.y;
     }
+
+    ROCPRIM_HOST_DEVICE custom_type& operator+=(const custom_type& rhs)
+    {
+        this->x += rhs.x;
+        this->y += rhs.y;
+        return *this;
+    }
 };
 
 template<typename>
-struct is_custom_type : std::false_type {};
+struct is_custom_type : std::false_type
+{};
 
 template<class T, class U>
-struct is_custom_type<custom_type<T,U>> : std::true_type {};
+struct is_custom_type<custom_type<T, U>> : std::true_type
+{};
+
+template<typename T, typename U>
+struct is_comparable
+{
+private:
+    // A dummy template function that attempts to compare two objects of types T and U
+    template<typename V, typename W>
+    static auto test(V&& v, W&& w)
+        -> decltype(std::declval<V>() < std::declval<W>(), std::true_type{});
+
+    // Fallback if the above template function is not valid
+    template<typename, typename>
+    static std::false_type test(...);
+
+public:
+    // Final result
+    static constexpr bool value = decltype(test<T, U>(std::declval<T>(), std::declval<U>()))::value;
+};
+
+template<typename T, typename U, typename V>
+struct is_comparable<custom_type<U, V>, T>
+    : std::conditional_t<rocprim::is_arithmetic<T>::value
+                             || !std::is_same<T, custom_type<U, V>>::value,
+                         std::false_type,
+                         std::true_type>
+{};
+
+template<class CustomType>
+struct custom_type_decomposer
+{
+    static_assert(is_custom_type<CustomType>::value,
+                  "custom_type_decomposer can only be used with instantiations of custom_type");
+
+    using T = typename CustomType::first_type;
+    using U = typename CustomType::second_type;
+
+    __host__ __device__ ::rocprim::tuple<T&, U&> operator()(CustomType& key) const
+    {
+        return ::rocprim::tuple<T&, U&>{key.x, key.y};
+    }
+};
+
+template<class T, class enable = void>
+struct generate_limits;
 
 template<class T>
-inline auto get_random_data(size_t size, T min, T max, size_t max_random_size = 1024 * 1024)
-    -> typename std::enable_if<is_custom_type<T>::value, std::vector<T>>::type
+struct generate_limits<T, std::enable_if_t<rocprim::is_integral<T>::value>>
 {
+    static inline T min()
+    {
+        return rocprim::numeric_limits<T>::min();
+    }
+    static inline T max()
+    {
+        return rocprim::numeric_limits<T>::max();
+    }
+};
+
+template<class T>
+struct generate_limits<T, std::enable_if_t<is_custom_type<T>::value>>
+{
+    using F = typename T::first_type;
+    using S = typename T::second_type;
+    static inline T min()
+    {
+        return T(generate_limits<F>::min(), generate_limits<S>::min());
+    }
+    static inline T max()
+    {
+        return T(generate_limits<F>::max(), generate_limits<S>::max());
+    }
+};
+
+template<class T>
+struct generate_limits<T, std::enable_if_t<rocprim::is_floating_point<T>::value>>
+{
+    static inline T min()
+    {
+        return T(-1000);
+    }
+    static inline T max()
+    {
+        return T(1000);
+    }
+};
+
+template<class OutputIterator, class Generator>
+inline auto generate_random_data_n(OutputIterator             it,
+                                   size_t                     size,
+                                   it_value_t<OutputIterator> min,
+                                   it_value_t<OutputIterator> max,
+                                   Generator&                 gen,
+                                   size_t                     max_random_size = 1024 * 1024)
+    -> std::enable_if_t<is_custom_type<it_value_t<OutputIterator>>::value, OutputIterator>
+{
+    using T = it_value_t<OutputIterator>;
+
     using first_type = typename T::first_type;
     using second_type = typename T::second_type;
-    std::vector<T> data(size);
-    auto fdata = get_random_data<first_type>(size, min.x, max.x, max_random_size);
-    auto sdata = get_random_data<second_type>(size, min.y, max.y, max_random_size);
+
+    std::vector<first_type>  fdata(size);
+    std::vector<second_type> sdata(size);
+    generate_random_data_n(fdata.begin(), size, min.x, max.x, gen, max_random_size);
+    generate_random_data_n(sdata.begin(), size, min.y, max.y, gen, max_random_size);
+
     for(size_t i = 0; i < size; i++)
     {
-        data[i] = T(fdata[i], sdata[i]);
+        it[i] = T(fdata[i], sdata[i]);
     }
+    return it + size;
+}
+
+template<class OutputIterator, class Generator>
+inline auto generate_random_data_n(OutputIterator             it,
+                                   size_t                     size,
+                                   it_value_t<OutputIterator> min,
+                                   it_value_t<OutputIterator> max,
+                                   Generator&                 gen,
+                                   size_t                     max_random_size = 1024 * 1024)
+    -> std::enable_if_t<!is_custom_type<it_value_t<OutputIterator>>::value
+                            && !std::is_same<decltype(max.x), void>::value,
+                        OutputIterator>
+{
+    using T = it_value_t<OutputIterator>;
+
+    using field_type = decltype(max.x);
+    std::vector<field_type> field_data(size);
+    generate_random_data_n(field_data.begin(), size, min.x, max.x, gen, max_random_size);
+    for(size_t i = 0; i < size; i++)
+    {
+        it[i] = T(field_data[i]);
+    }
+    return it + size;
+}
+
+template<class T, class U, class V>
+inline std::vector<T> get_random_data(
+    size_t size, U min, V max, unsigned int seed, size_t max_random_size = 1024 * 1024)
+{
+    std::vector<T> data(size);
+    engine_type    gen(seed);
+    generate_random_data_n(data.begin(), size, min, max, gen, max_random_size);
     return data;
 }
 
-template<class T>
-inline auto get_random_data(size_t size, T min, T max, size_t max_random_size = 1024 * 1024)
-    -> typename std::enable_if<!is_custom_type<T>::value && !std::is_same<decltype(max.x), void>::value, std::vector<T>>::type
+template<typename T, typename U>
+auto limit_cast(U value) -> T
 {
-    // NOTE 1: post-increment operator required, because HIP has different typedefs for vector field types
-    //         when using HCC or HIP-Clang. Using HIP-Clang members are accessed as fields of a struct via
-    //         a union, but in HCC mode they are proxy types (Scalar_accessor). STL algorithms don't
-    //         always tolerate proxies. Unfortunately, Scalar_accessor doesn't have any member typedefs to
-    //         conveniently obtain the inner stored type. All operations on it (operator+, operator+=,
-    //         CTOR, etc.) return a reference to an accessor, it is only the post-increment operator that
-    //         returns a copy of the stored type, hence we take the decltype of that.
-    //
-    // NOTE 2: decltype() is unevaluated context. We don't really modify max, just compute the type of the
-    //         expression if we were to actually call it.
-    using field_type = decltype(max.x++);
-    std::vector<T> data(size);
-    auto field_data = get_random_data<field_type>(size, min.x, max.x, max_random_size);
-    for(size_t i = 0; i < size; i++)
+    static_assert(rocprim::is_arithmetic<T>::value && rocprim::is_arithmetic<U>::value
+                      && is_comparable<T, U>::value,
+                  "Cannot use limit_cast with chosen types of T and U");
+
+    using common_type = typename std::common_type<T, U>::type;
+    if(rocprim::is_unsigned<T>::value)
     {
-        data[i] = T(field_data[i]);
+        if(value < 0)
+        {
+            return rocprim::numeric_limits<T>::min();
+        }
+        if(static_cast<common_type>(value)
+           > static_cast<common_type>(rocprim::numeric_limits<T>::max()))
+        {
+            return rocprim::numeric_limits<T>::max();
+        }
     }
-    return data;
+    else if(rocprim::is_signed<T>::value && rocprim::is_unsigned<U>::value)
+    {
+        if(value > rocprim::numeric_limits<T>::max())
+        {
+            return rocprim::numeric_limits<T>::max();
+        }
+    }
+    else if(rocprim::is_floating_point<T>::value)
+    {
+        return static_cast<T>(value);
+    }
+    else // Both T and U are signed
+    {
+        if(value < static_cast<common_type>(rocprim::numeric_limits<T>::min()))
+        {
+            return rocprim::numeric_limits<T>::min();
+        }
+        else if(value > static_cast<common_type>(rocprim::numeric_limits<T>::max()))
+        {
+            return rocprim::numeric_limits<T>::max();
+        }
+    }
+    return static_cast<T>(value);
 }
 
-inline bool is_warp_size_supported(const unsigned int required_warp_size)
+// This overload below is selected for non-standard float types, e.g. half, which cannot be compared with the limit types.
+template<class T, class U, class V>
+inline auto limit_random_range(U range_start, V range_end)
+    -> std::enable_if_t<!is_custom_type<T>::value
+                            && (!is_comparable<T, U>::value || !is_comparable<T, V>::value),
+                        std::pair<T, T>>
 {
-    return ::rocprim::host_warp_size() >= required_warp_size;
+    return {static_cast<T>(range_start), static_cast<T>(range_end)};
+}
+
+template<typename T, typename U, typename V>
+auto limit_random_range(U range_start, V range_end)
+    -> std::enable_if_t<(is_custom_type<T>::value && is_comparable<typename T::first_type, U>::value
+                         && is_comparable<typename T::second_type, U>::value
+                         && is_comparable<typename T::first_type, V>::value
+                         && is_comparable<typename T::second_type, V>::value
+                         && rocprim::is_arithmetic<typename T::first_type>::value
+                         && rocprim::is_arithmetic<typename T::second_type>::value
+                         && rocprim::is_arithmetic<U>::value && rocprim::is_arithmetic<V>::value),
+                        std::pair<T, T>>
+{
+
+    return {
+        T{limit_cast<typename T::first_type>(range_start),
+          limit_cast<typename T::second_type>(range_start)},
+        T{  limit_cast<typename T::first_type>(range_end),
+          limit_cast<typename T::second_type>(range_end)  }
+    };
+}
+
+template<class T, class U, class V>
+inline auto limit_random_range(U range_start, V range_end)
+    -> std::enable_if_t<!is_custom_type<T>::value && is_comparable<T, U>::value
+                            && is_comparable<T, V>::value,
+                        std::pair<T, T>>
+{
+
+    if(is_comparable<V, U>::value)
+    {
+        using common_type = typename std::common_type<T, U>::type;
+        if(static_cast<common_type>(range_start) > static_cast<common_type>(range_end))
+        {
+            throw std::range_error("limit_random_range: Incorrect range used!");
+        }
+    }
+
+    T start = limit_cast<T>(range_start);
+    T end   = limit_cast<T>(range_end);
+    return std::make_pair(start, end);
+}
+
+inline bool is_warp_size_supported(const unsigned int required_warp_size, const int device_id)
+{
+    unsigned int warp_size;
+    HIP_CHECK(::rocprim::host_warp_size(device_id, warp_size));
+    return warp_size >= required_warp_size;
 }
 
 template<unsigned int LogicalWarpSize>
-struct DeviceSelectWarpSize
-{
-    static constexpr unsigned int value = ::rocprim::device_warp_size() >= LogicalWarpSize
-        ? LogicalWarpSize
-        : ::rocprim::device_warp_size();
-};
+__device__ constexpr bool device_test_enabled_for_warp_size_v
+    = ::rocprim::arch::wavefront::min_size() >= LogicalWarpSize;
 
+/// \brief Get segments of uniform random size in [1, max_segment_length] with random key.
 template<typename T>
 std::vector<T>
-    get_random_segments(const size_t size, const size_t max_segment_length, const int seed_value)
+    get_random_segments(const size_t size, const size_t max_segment_length, unsigned int seed)
 {
     static_assert(rocprim::is_arithmetic<T>::value, "Key type must be arithmetic");
 
-    std::default_random_engine            prng(seed_value);
-    std::uniform_int_distribution<size_t> segment_length_distribution(max_segment_length);
+    engine_type                           prng(seed);
+    std::uniform_int_distribution<size_t> segment_length_distribution(
+        std::numeric_limits<size_t>::min(),
+        max_segment_length);
     // std::uniform_real_distribution cannot handle rocprim::half, use float instead
     using dis_type =
         typename std::conditional<std::is_same<rocprim::half, T>::value, float, T>::type;
     using key_distribution_type = std::conditional_t<rocprim::is_integral<T>::value,
                                                      std::uniform_int_distribution<dis_type>,
                                                      std::uniform_real_distribution<dis_type>>;
-    key_distribution_type key_distribution(std::numeric_limits<T>::max());
+    key_distribution_type key_distribution(rocprim::numeric_limits<T>::max());
     std::vector<T>        keys(size);
 
     size_t keys_start_index = 0;
@@ -316,6 +576,51 @@ std::vector<T>
     return keys;
 }
 
+/// \brief Get segments of uniform random size in [1, max_segment_length] with unique incrementing key.
+template<typename T>
+std::vector<T>
+    get_random_segments_iota(const size_t size, const size_t max_segment_length, unsigned int seed)
+{
+    engine_type                           prng(seed);
+    std::uniform_int_distribution<size_t> segment_length_distribution(1, max_segment_length);
+
+    std::vector<T> keys(size);
+
+    size_t segment_index    = 0;
+    size_t keys_start_index = 0;
+    while(keys_start_index < size)
+    {
+        const size_t new_segment_length = segment_length_distribution(prng);
+        const size_t new_segment_end    = std::min(size, keys_start_index + new_segment_length);
+        const T      key                = segment_index++;
+        std::fill(keys.begin() + keys_start_index, keys.begin() + new_segment_end, key);
+        keys_start_index += new_segment_length;
+    }
+    return keys;
+}
+
+template<class T, class U, class V>
+inline auto get_random_value(U min, V max, size_t seed_value)
+    -> std::enable_if_t<rocprim::is_arithmetic<T>::value, T>
+{
+    T           result;
+    engine_type gen(seed_value);
+    generate_random_data_n(&result, 1, min, max, gen);
+    return result;
+}
+
+template<class T>
+inline auto get_random_value(T min, T max, size_t seed_value)
+    -> std::enable_if_t<is_custom_type<T>::value, T>
+{
+    typename T::first_type  result_first;
+    typename T::second_type result_second;
+    engine_type             gen(seed_value);
+    generate_random_data_n(&result_first, 1, min.x, max.x, gen);
+    generate_random_data_n(&result_second, 1, min.y, max.y, gen);
+    return T{result_first, result_second};
+}
+
 template <typename T, T, typename>
 struct make_index_range_impl;
 
@@ -345,13 +650,16 @@ void static_for_each(Args&&... args)
                                                                  std::forward<Args>(args)...);
 }
 
-#define REGISTER_BENCHMARK(benchmarks, size, stream, instance)                     \
-    benchmark::internal::Benchmark* benchmark = benchmark::RegisterBenchmark(      \
-        instance.name().c_str(),                                                   \
-        [instance](benchmark::State& state, size_t size, const hipStream_t stream) \
-        { instance.run(state, size, stream); },                                    \
-        size,                                                                      \
-        stream);                                                                   \
+#define REGISTER_BENCHMARK(benchmarks, size, seed, stream, instance)                     \
+    benchmark::internal::Benchmark* benchmark = benchmark::RegisterBenchmark(            \
+        instance.name().c_str(),                                                         \
+        [instance](benchmark::State&   state,                                            \
+                   size_t              _size,                                            \
+                   const managed_seed& _seed,                                            \
+                   hipStream_t         _stream) { instance.run(state, _size, _seed, _stream); }, \
+        size,                                                                            \
+        seed,                                                                            \
+        stream);                                                                         \
     benchmarks.emplace_back(benchmark)
 
 struct config_autotune_interface
@@ -361,8 +669,8 @@ struct config_autotune_interface
     {
         return name();
     };
-    virtual ~config_autotune_interface()                           = default;
-    virtual void run(benchmark::State&, size_t, hipStream_t) const = 0;
+    virtual ~config_autotune_interface()                                                = default;
+    virtual void run(benchmark::State&, size_t, const managed_seed&, hipStream_t) const = 0;
 };
 
 struct config_autotune_register
@@ -387,10 +695,11 @@ struct config_autotune_register
 
     // Register a subset of all created benchmarks for the current parallel instance and add to vector.
     static void register_benchmark_subset(std::vector<benchmark::internal::Benchmark*>& benchmarks,
-                                          int               parallel_instance_index,
-                                          int               parallel_instance_count,
-                                          size_t            size,
-                                          const hipStream_t stream)
+                                          int                 parallel_instance_index,
+                                          int                 parallel_instance_count,
+                                          size_t              size,
+                                          const managed_seed& seed,
+                                          hipStream_t         stream)
     {
         std::vector<std::unique_ptr<config_autotune_interface>>& configs = vector();
         // sorting to get a consistent order because order of initialization of static variables is undefined by the C++ standard.
@@ -407,9 +716,13 @@ struct config_autotune_register
             config_autotune_interface*                  tuning_benchmark = uniq_ptr.get();
             benchmark::internal::Benchmark*             benchmark = benchmark::RegisterBenchmark(
                 tuning_benchmark->name().c_str(),
-                [tuning_benchmark](benchmark::State& state, size_t size, const hipStream_t stream)
-                { tuning_benchmark->run(state, size, stream); },
+                [tuning_benchmark](benchmark::State&   state,
+                                   size_t              size,
+                                   const managed_seed& seed,
+                                   hipStream_t         stream)
+                { tuning_benchmark->run(state, size, seed, stream); },
                 size,
+                seed,
                 stream);
             benchmarks.emplace_back(benchmark);
         }
@@ -441,7 +754,7 @@ public:
         static format storage = human;
         return storage;
     }
-    static void set_format(std::string argument)
+    static void set_format(const std::string& argument)
     {
         format result = human;
         if(argument == "json")
@@ -616,6 +929,11 @@ struct Traits
 };
 
 // Explicit definitions
+template<>
+inline const char* Traits<char>::name()
+{
+    return "char";
+}
 template <>
 inline const char* Traits<int>::name() { return "int"; }
 template <>
@@ -640,12 +958,17 @@ inline const char* Traits<rocprim::half>::name()
     return "rocprim::half";
 }
 template<>
+inline const char* Traits<rocprim::bfloat16>::name()
+{
+    return "rocprim::bfloat16";
+}
+template<>
 inline const char* Traits<long long>::name()
 {
     return "int64_t";
 }
 // On MSVC `int64_t` and `long long` are the same, leading to multiple definition errors
-#ifndef WIN32
+#ifndef _WIN32
 template <>
 inline const char* Traits<int64_t>::name() { return "int64_t"; }
 #endif
@@ -669,6 +992,11 @@ inline const char* Traits<custom_type<double, double>>::name()
     return "custom_type<double,double>";
 }
 template<>
+inline const char* Traits<custom_type<int, double>>::name()
+{
+    return "custom_type<int,double>";
+}
+template<>
 inline const char* Traits<custom_type<char, double>>::name()
 {
     return "custom_type<char,double>";
@@ -689,6 +1017,11 @@ inline const char* Traits<custom_type<long long, double>>::name()
     return "custom_type<int64_t,double>";
 }
 template<>
+inline const char* Traits<custom_type<float, int16_t>>::name()
+{
+    return "custom_type<float,int16_t>";
+}
+template<>
 inline const char* Traits<rocprim::empty_type>::name()
 {
     return "empty_type";
@@ -801,7 +1134,7 @@ inline void add_common_benchmark_info()
     num("hdp_arch_has_dynamic_parallelism", arch.hasDynamicParallelism);
 }
 
-inline const char* get_block_scan_method_name(rocprim::block_scan_algorithm alg)
+inline const char* get_block_scan_algorithm_name(rocprim::block_scan_algorithm alg)
 {
     switch(alg)
     {
@@ -811,7 +1144,45 @@ inline const char* get_block_scan_method_name(rocprim::block_scan_algorithm alg)
             return "block_scan_algorithm::reduce_then_scan";
             // Not using `default: ...` because it kills effectiveness of -Wswitch
     }
-    return "unknown_algorithm";
+    return "default_algorithm";
+}
+
+inline const char* get_block_load_method_name(rocprim::block_load_method method)
+{
+    switch(method)
+    {
+        case rocprim::block_load_method::block_load_direct:
+            return "block_load_method::block_load_direct";
+        case rocprim::block_load_method::block_load_striped:
+            return "block_load_method::block_load_striped";
+        case rocprim::block_load_method::block_load_vectorize:
+            return "block_load_method::block_load_vectorize";
+        case rocprim::block_load_method::block_load_transpose:
+            return "block_load_method::block_load_transpose";
+        case rocprim::block_load_method::block_load_warp_transpose:
+            return "block_load_method::block_load_warp_transpose";
+    }
+    return "default_method";
+}
+
+template<std::size_t Size, std::size_t Alignment>
+struct alignas(Alignment) custom_aligned_type
+{
+    unsigned char data[Size];
+};
+
+template<typename Config>
+std::string partition_config_name()
+{
+    const rocprim::detail::partition_config_params config = Config();
+    return "{bs:" + std::to_string(config.kernel_config.block_size)
+           + ",ipt:" + std::to_string(config.kernel_config.items_per_thread) + "}";
+}
+
+template<>
+inline std::string partition_config_name<rocprim::default_config>()
+{
+    return "default_config";
 }
 
 #endif // ROCPRIM_BENCHMARK_UTILS_HPP_
