@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -20,26 +20,28 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 
-#include <iostream>
-
-#include <vector>
-#include <limits>
-#include <string>
-#include <cstdio>
-#include <cstdlib>
-
-// Google Benchmark
-#include "benchmark/benchmark.h"
+#include "benchmark_utils.hpp"
 // CmdParser
 #include "cmdparser.hpp"
-#include "benchmark_utils.hpp"
+
+// Google Benchmark
+#include <benchmark/benchmark.h>
 
 // HIP API
 #include <hip/hip_runtime.h>
+#include <rocprim/device/config_types.hpp>
 #include <rocprim/warp/warp_exchange.hpp>
 
-#ifndef DEFAULT_N
-const size_t DEFAULT_N = 1024 * 1024 * 32;
+#include <iostream>
+#include <limits>
+#include <string>
+#include <vector>
+
+#include <cstdio>
+#include <cstdlib>
+
+#ifndef DEFAULT_BYTES
+const size_t DEFAULT_BYTES = 1024 * 1024 * 32 * 4;
 #endif
 
 struct BlockedToStripedOp
@@ -52,7 +54,7 @@ struct BlockedToStripedOp
     ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
     void operator()(warp_exchange_type warp_exchange,
                     T (&items)[ItemsPerThread],
-                    typename warp_exchange_type::storage_type& storage)
+                    typename warp_exchange_type::storage_type& storage) const
     {
         warp_exchange.blocked_to_striped(items, items, storage);
     }
@@ -68,7 +70,7 @@ struct StripedToBlockedOp
     ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
     void operator()(warp_exchange_type warp_exchange,
                     T (&items)[ItemsPerThread],
-                    typename warp_exchange_type::storage_type& storage)
+                    typename warp_exchange_type::storage_type& storage) const
     {
         warp_exchange.striped_to_blocked(items, items, storage);
     }
@@ -84,7 +86,7 @@ struct BlockedToStripedShuffleOp
     ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
     void operator()(warp_exchange_type warp_exchange,
                     T (&items)[ItemsPerThread],
-                    typename warp_exchange_type::storage_type& /*storage*/)
+                    typename warp_exchange_type::storage_type& /*storage*/) const
     {
         warp_exchange.blocked_to_striped_shuffle(items, items);
     }
@@ -100,7 +102,7 @@ struct StripedToBlockedShuffleOp
     ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
     void operator()(warp_exchange_type warp_exchange,
                     T (&items)[ItemsPerThread],
-                    typename warp_exchange_type::storage_type& /*storage*/)
+                    typename warp_exchange_type::storage_type& /*storage*/) const
     {
         warp_exchange.striped_to_blocked_shuffle(items, items);
     }
@@ -118,22 +120,20 @@ struct ScatterToStripedOp
     void operator()(warp_exchange_type warp_exchange,
                     T (&thread_data)[ItemsPerThread],
                     const OffsetT (&ranks)[ItemsPerThread],
-                    typename warp_exchange_type::storage_type& storage)
+                    typename warp_exchange_type::storage_type& storage) const
     {
         warp_exchange.scatter_to_striped(thread_data, thread_data, ranks, storage);
     }
 };
 
-template<
-    class T,
-    unsigned int BlockSize,
-    unsigned int ItemsPerThread,
-    unsigned int LogicalWarpSize,
-    class Op
->
-__global__
-__launch_bounds__(BlockSize)
-auto warp_exchange_kernel(T* d_output, unsigned int trials) -> typename std::enable_if<!std::is_same<Op, ScatterToStripedOp>::value, void>::type
+template<unsigned int BlockSize,
+         unsigned int ItemsPerThread,
+         unsigned int LogicalWarpSize,
+         class Op,
+         class T>
+__device__ auto warp_exchange_benchmark(T* d_output, unsigned int trials)
+    -> std::enable_if_t<device_test_enabled_for_warp_size_v<LogicalWarpSize>
+                        && !std::is_same<Op, ScatterToStripedOp>::value>
 {
     T thread_data[ItemsPerThread];
 
@@ -141,16 +141,12 @@ auto warp_exchange_kernel(T* d_output, unsigned int trials) -> typename std::ena
     for(unsigned int i = 0; i < ItemsPerThread; i++)
     {
         // generate unique value each data-element
-        thread_data[i] = static_cast<T>(hipThreadIdx_x*ItemsPerThread+i);
+        thread_data[i] = static_cast<T>(threadIdx.x * ItemsPerThread + i);
     }
 
-    using warp_exchange_type = ::rocprim::warp_exchange<
-        T,
-        ItemsPerThread,
-        DeviceSelectWarpSize<LogicalWarpSize>::value
-    >;
+    using warp_exchange_type = ::rocprim::warp_exchange<T, ItemsPerThread, LogicalWarpSize>;
     constexpr unsigned int warps_in_block = BlockSize / LogicalWarpSize;
-    const unsigned int warp_id = hipThreadIdx_x / LogicalWarpSize;
+    const unsigned int     warp_id        = threadIdx.x / LogicalWarpSize;
     ROCPRIM_SHARED_MEMORY typename warp_exchange_type::storage_type storage[warps_in_block];
 
     ROCPRIM_NO_UNROLL
@@ -163,44 +159,37 @@ auto warp_exchange_kernel(T* d_output, unsigned int trials) -> typename std::ena
     ROCPRIM_UNROLL
     for(unsigned int i = 0; i < ItemsPerThread; i++)
     {
-        const unsigned int global_idx =
-            (BlockSize * hipBlockIdx_x + hipThreadIdx_x) * ItemsPerThread + i;
+        const unsigned int global_idx = (BlockSize * blockIdx.x + threadIdx.x) * ItemsPerThread + i;
         d_output[global_idx] = thread_data[i];
     }
 }
 
-template<
-    class T,
-    unsigned int BlockSize,
-    unsigned int ItemsPerThread,
-    unsigned int LogicalWarpSize,
-    class Op
-    >
-__global__
-__launch_bounds__(BlockSize)
-    auto warp_exchange_kernel(T* d_output, unsigned int trials) -> typename std::enable_if<std::is_same<Op, ScatterToStripedOp>::value, void>::type
+template<unsigned int BlockSize,
+         unsigned int ItemsPerThread,
+         unsigned int LogicalWarpSize,
+         class Op,
+         class T>
+__device__ auto warp_exchange_benchmark(T* d_output, unsigned int trials)
+    -> std::enable_if_t<device_test_enabled_for_warp_size_v<LogicalWarpSize>
+                        && std::is_same<Op, ScatterToStripedOp>::value>
 {
     T thread_data[ItemsPerThread];
     unsigned int thread_ranks[ItemsPerThread];
     constexpr unsigned int warps_in_block = BlockSize / LogicalWarpSize;
-    const unsigned int warp_id = hipThreadIdx_x / LogicalWarpSize;
-    const unsigned int lane_id = hipThreadIdx_x % LogicalWarpSize;
+    const unsigned int     warp_id        = threadIdx.x / LogicalWarpSize;
+    const unsigned int     lane_id        = threadIdx.x % LogicalWarpSize;
 
     ROCPRIM_UNROLL
     for(unsigned int i = 0; i < ItemsPerThread; i++)
     {
         // generate unique value each data-element
-        thread_data[i] = static_cast<T>(hipThreadIdx_x*ItemsPerThread+i);
+        thread_data[i] = static_cast<T>(threadIdx.x * ItemsPerThread + i);
         // generate unique destination location for each data-element
         const unsigned int s_lane_id = i % 2 == 0 ? LogicalWarpSize - 1 - lane_id : lane_id;
         thread_ranks[i] = s_lane_id*ItemsPerThread+i; // scatter values in warp across whole storage
     }
 
-    using warp_exchange_type = ::rocprim::warp_exchange<
-        T,
-        ItemsPerThread,
-        DeviceSelectWarpSize<LogicalWarpSize>::value
-        >;
+    using warp_exchange_type = ::rocprim::warp_exchange<T, ItemsPerThread, LogicalWarpSize>;
     ROCPRIM_SHARED_MEMORY typename warp_exchange_type::storage_type storage[warps_in_block];
 
     ROCPRIM_NO_UNROLL
@@ -213,12 +202,30 @@ __launch_bounds__(BlockSize)
     ROCPRIM_UNROLL
     for(unsigned int i = 0; i < ItemsPerThread; i++)
     {
-        const unsigned int global_idx =
-            (BlockSize * hipBlockIdx_x + hipThreadIdx_x) * ItemsPerThread + i;
+        const unsigned int global_idx = (BlockSize * blockIdx.x + threadIdx.x) * ItemsPerThread + i;
         d_output[global_idx] = thread_data[i];
     }
 }
 
+template<unsigned int BlockSize,
+         unsigned int ItemsPerThread,
+         unsigned int LogicalWarpSize,
+         class Op,
+         class T>
+__device__ auto warp_exchange_benchmark(T* /*d_output*/, unsigned int /*trials*/)
+    -> std::enable_if_t<!device_test_enabled_for_warp_size_v<LogicalWarpSize>>
+{}
+
+template<unsigned int BlockSize,
+         unsigned int ItemsPerThread,
+         unsigned int LogicalWarpSize,
+         class Op,
+         class T>
+__global__ __launch_bounds__(BlockSize) void warp_exchange_kernel(T* d_output, unsigned int trials)
+{
+    warp_exchange_benchmark<BlockSize, ItemsPerThread, LogicalWarpSize, Op>(d_output, trials);
+}
+
 template<
     class T,
     unsigned int BlockSize,
@@ -226,8 +233,11 @@ template<
     unsigned int LogicalWarpSize,
     class Op
 >
-void run_benchmark(benchmark::State& state, hipStream_t stream, size_t N)
+void run_benchmark(benchmark::State& state, hipStream_t stream, size_t bytes)
 {
+    // Calculate the number of elements 
+    size_t N = bytes / sizeof(T);
+
     constexpr unsigned int trials = 200;
     constexpr unsigned int items_per_block = BlockSize * ItemsPerThread;
     const unsigned int size = items_per_block * ((N + items_per_block - 1) / items_per_block);
@@ -245,18 +255,8 @@ void run_benchmark(benchmark::State& state, hipStream_t stream, size_t N)
         // Record start event
         HIP_CHECK(hipEventRecord(start, stream));
 
-        hipLaunchKernelGGL(
-            HIP_KERNEL_NAME(warp_exchange_kernel<
-                    T,
-                    BlockSize,
-                    ItemsPerThread,
-                    LogicalWarpSize,
-                    Op
-                >
-            ),
-            dim3(size / items_per_block), dim3(BlockSize), 0, stream,
-            d_output, trials
-        );
+        warp_exchange_kernel<BlockSize, ItemsPerThread, LogicalWarpSize, Op>
+            <<<dim3(size / items_per_block), dim3(BlockSize), 0, stream>>>(d_output, trials);
 
         HIP_CHECK(hipPeekAtLastError());
 
@@ -286,12 +286,12 @@ void run_benchmark(benchmark::State& state, hipStream_t stream, size_t N)
                                      .c_str(),                                                    \
                                  &run_benchmark<T, BS, IT, WS, OP>,                               \
                                  stream,                                                          \
-                                 size)
+                                 bytes)
 
 int main(int argc, char *argv[])
 {
     cli::Parser parser(argc, argv);
-    parser.set_optional<size_t>("size", "size", DEFAULT_N, "number of values");
+    parser.set_optional<size_t>("size", "size", DEFAULT_BYTES, "number of bytes");
     parser.set_optional<int>("trials", "trials", -1, "number of iterations");
     parser.set_optional<std::string>("name_format",
                                      "name_format",
@@ -301,7 +301,7 @@ int main(int argc, char *argv[])
 
     // Parse argv
     benchmark::Initialize(&argc, argv);
-    const size_t size = parser.get<size_t>("size");
+    const size_t bytes = parser.get<size_t>("size");
     const int trials = parser.get<int>("trials");
     bench_naming::set_format(parser.get<std::string>("name_format"));
 
@@ -310,69 +310,77 @@ int main(int argc, char *argv[])
 
     // Benchmark info
     add_common_benchmark_info();
-    benchmark::AddCustomContext("size", std::to_string(size));
+    benchmark::AddCustomContext("bytes", std::to_string(bytes));
 
     // Add benchmarks
     std::vector<benchmark::internal::Benchmark*> benchmarks{
-        CREATE_BENCHMARK(int, 256,  1, 16, BlockedToStripedOp),
-        CREATE_BENCHMARK(int, 256,  1, 32, BlockedToStripedOp),
-        CREATE_BENCHMARK(int, 256,  4, 16, BlockedToStripedOp),
-        CREATE_BENCHMARK(int, 256,  4, 32, BlockedToStripedOp),
+        CREATE_BENCHMARK(int, 256, 1, 16, BlockedToStripedOp),
+        CREATE_BENCHMARK(int, 256, 1, 32, BlockedToStripedOp),
+        CREATE_BENCHMARK(int, 256, 4, 16, BlockedToStripedOp),
+        CREATE_BENCHMARK(int, 256, 4, 32, BlockedToStripedOp),
         CREATE_BENCHMARK(int, 256, 16, 16, BlockedToStripedOp),
         CREATE_BENCHMARK(int, 256, 16, 32, BlockedToStripedOp),
+        CREATE_BENCHMARK(int, 256, 32, 32, BlockedToStripedOp),
 
-        CREATE_BENCHMARK(int, 256,  1, 16, StripedToBlockedOp),
-        CREATE_BENCHMARK(int, 256,  1, 32, StripedToBlockedOp),
-        CREATE_BENCHMARK(int, 256,  4, 16, StripedToBlockedOp),
-        CREATE_BENCHMARK(int, 256,  4, 32, StripedToBlockedOp),
+        CREATE_BENCHMARK(int, 256, 1, 16, StripedToBlockedOp),
+        CREATE_BENCHMARK(int, 256, 1, 32, StripedToBlockedOp),
+        CREATE_BENCHMARK(int, 256, 4, 16, StripedToBlockedOp),
+        CREATE_BENCHMARK(int, 256, 4, 32, StripedToBlockedOp),
         CREATE_BENCHMARK(int, 256, 16, 16, StripedToBlockedOp),
         CREATE_BENCHMARK(int, 256, 16, 32, StripedToBlockedOp),
+        CREATE_BENCHMARK(int, 256, 32, 32, StripedToBlockedOp),
 
-        CREATE_BENCHMARK(int, 256,  1, 16, BlockedToStripedShuffleOp),
-        CREATE_BENCHMARK(int, 256,  1, 32, BlockedToStripedShuffleOp),
-        CREATE_BENCHMARK(int, 256,  4, 16, BlockedToStripedShuffleOp),
-        CREATE_BENCHMARK(int, 256,  4, 32, BlockedToStripedShuffleOp),
+        CREATE_BENCHMARK(int, 256, 1, 16, BlockedToStripedShuffleOp),
+        CREATE_BENCHMARK(int, 256, 1, 32, BlockedToStripedShuffleOp),
+        CREATE_BENCHMARK(int, 256, 4, 16, BlockedToStripedShuffleOp),
+        CREATE_BENCHMARK(int, 256, 4, 32, BlockedToStripedShuffleOp),
         CREATE_BENCHMARK(int, 256, 16, 16, BlockedToStripedShuffleOp),
         CREATE_BENCHMARK(int, 256, 16, 32, BlockedToStripedShuffleOp),
+        CREATE_BENCHMARK(int, 256, 32, 32, BlockedToStripedShuffleOp),
 
-        CREATE_BENCHMARK(int, 256,  1, 16, StripedToBlockedShuffleOp),
-        CREATE_BENCHMARK(int, 256,  1, 32, StripedToBlockedShuffleOp),
-        CREATE_BENCHMARK(int, 256,  4, 16, StripedToBlockedShuffleOp),
-        CREATE_BENCHMARK(int, 256,  4, 32, StripedToBlockedShuffleOp),
+        CREATE_BENCHMARK(int, 256, 1, 16, StripedToBlockedShuffleOp),
+        CREATE_BENCHMARK(int, 256, 1, 32, StripedToBlockedShuffleOp),
+        CREATE_BENCHMARK(int, 256, 4, 16, StripedToBlockedShuffleOp),
+        CREATE_BENCHMARK(int, 256, 4, 32, StripedToBlockedShuffleOp),
         CREATE_BENCHMARK(int, 256, 16, 16, StripedToBlockedShuffleOp),
         CREATE_BENCHMARK(int, 256, 16, 32, StripedToBlockedShuffleOp),
+        CREATE_BENCHMARK(int, 256, 32, 32, StripedToBlockedShuffleOp),
 
-        CREATE_BENCHMARK(int, 256,  1, 16, ScatterToStripedOp),
-        CREATE_BENCHMARK(int, 256,  1, 32, ScatterToStripedOp),
-        CREATE_BENCHMARK(int, 256,  4, 16, ScatterToStripedOp),
-        CREATE_BENCHMARK(int, 256,  4, 32, ScatterToStripedOp),
+        CREATE_BENCHMARK(int, 256, 1, 16, ScatterToStripedOp),
+        CREATE_BENCHMARK(int, 256, 1, 32, ScatterToStripedOp),
+        CREATE_BENCHMARK(int, 256, 4, 16, ScatterToStripedOp),
+        CREATE_BENCHMARK(int, 256, 4, 32, ScatterToStripedOp),
         CREATE_BENCHMARK(int, 256, 16, 16, ScatterToStripedOp),
-        CREATE_BENCHMARK(int, 256, 16, 32, ScatterToStripedOp)
-    };
+        CREATE_BENCHMARK(int, 256, 16, 32, ScatterToStripedOp)};
 
-    if(is_warp_size_supported(64))
+    int hip_device = 0;
+    HIP_CHECK(::rocprim::detail::get_device_from_stream(stream, hip_device));
+    if(is_warp_size_supported(64, hip_device))
     {
         std::vector<benchmark::internal::Benchmark*> additional_benchmarks{
-            CREATE_BENCHMARK(int, 256,  1, 64, BlockedToStripedOp),
-            CREATE_BENCHMARK(int, 256,  4, 64, BlockedToStripedOp),
+            CREATE_BENCHMARK(int, 256, 1, 64, BlockedToStripedOp),
+            CREATE_BENCHMARK(int, 256, 4, 64, BlockedToStripedOp),
             CREATE_BENCHMARK(int, 256, 16, 64, BlockedToStripedOp),
+            CREATE_BENCHMARK(int, 256, 64, 64, BlockedToStripedOp),
 
-            CREATE_BENCHMARK(int, 256,  1, 64, StripedToBlockedOp),
-            CREATE_BENCHMARK(int, 256,  4, 64, StripedToBlockedOp),
+            CREATE_BENCHMARK(int, 256, 1, 64, StripedToBlockedOp),
+            CREATE_BENCHMARK(int, 256, 4, 64, StripedToBlockedOp),
             CREATE_BENCHMARK(int, 256, 16, 64, StripedToBlockedOp),
+            CREATE_BENCHMARK(int, 256, 64, 64, StripedToBlockedOp),
 
-            CREATE_BENCHMARK(int, 256,  1, 64, BlockedToStripedShuffleOp),
-            CREATE_BENCHMARK(int, 256,  4, 64, BlockedToStripedShuffleOp),
+            CREATE_BENCHMARK(int, 256, 1, 64, BlockedToStripedShuffleOp),
+            CREATE_BENCHMARK(int, 256, 4, 64, BlockedToStripedShuffleOp),
             CREATE_BENCHMARK(int, 256, 16, 64, BlockedToStripedShuffleOp),
+            CREATE_BENCHMARK(int, 256, 64, 64, BlockedToStripedShuffleOp),
 
-            CREATE_BENCHMARK(int, 256,  1, 64, StripedToBlockedShuffleOp),
-            CREATE_BENCHMARK(int, 256,  4, 64, StripedToBlockedShuffleOp),
+            CREATE_BENCHMARK(int, 256, 1, 64, StripedToBlockedShuffleOp),
+            CREATE_BENCHMARK(int, 256, 4, 64, StripedToBlockedShuffleOp),
             CREATE_BENCHMARK(int, 256, 16, 64, StripedToBlockedShuffleOp),
+            CREATE_BENCHMARK(int, 256, 64, 64, StripedToBlockedShuffleOp),
 
-            CREATE_BENCHMARK(int, 256,  1, 64, ScatterToStripedOp),
-            CREATE_BENCHMARK(int, 256,  4, 64, ScatterToStripedOp),
-            CREATE_BENCHMARK(int, 256, 16, 64, ScatterToStripedOp)
-        };
+            CREATE_BENCHMARK(int, 256, 1, 64, ScatterToStripedOp),
+            CREATE_BENCHMARK(int, 256, 4, 64, ScatterToStripedOp),
+            CREATE_BENCHMARK(int, 256, 16, 64, ScatterToStripedOp)};
         benchmarks.insert(
             benchmarks.end(),
             additional_benchmarks.begin(),
