@@ -1,7 +1,7 @@
 // The MIT License (MIT)
 //
 // Copyright (c) 2015 - 2016 Florian Rappl
-// Modifications Copyright (c) 2019, Advanced Micro Devices, Inc.  All rights reserved.
+// Modifications Copyright (c) 2019-2024, Advanced Micro Devices, Inc.  All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -27,12 +27,12 @@
 */
 
 #pragma once
+#include <functional>
 #include <iostream>
+#include <sstream>
 #include <stdexcept>
 #include <string>
 #include <vector>
-#include <sstream>
-#include <functional>
 
 namespace cli {
     struct CallbackArgs {
@@ -96,7 +96,7 @@ namespace cli {
                 CmdBase(name, alternative, description, required, dominant, ArgumentCountChecker<T>::Variadic) {
             }
 
-            virtual bool parse(std::ostream& output, std::ostream& error) {
+            virtual bool parse(std::ostream& output, std::ostream& error) override {
                 try {
                     CallbackArgs args { arguments, output, error };
                     value = callback(args);
@@ -106,7 +106,7 @@ namespace cli {
                 }
             }
 
-            virtual std::string print_value() const {
+            virtual std::string print_value() const override {
                 return "";
             }
 
@@ -118,10 +118,10 @@ namespace cli {
         class CmdArgument final : public CmdBase {
         public:
             explicit CmdArgument(const std::string& name, const std::string& alternative, const std::string& description, bool required, bool dominant) :
-                CmdBase(name, alternative, description, required, dominant, ArgumentCountChecker<T>::Variadic) {
+                CmdBase(name, alternative, description, required, dominant, ArgumentCountChecker<T>::Variadic), value(T()) {
             }
 
-            virtual bool parse(std::ostream&, std::ostream&) {
+            virtual bool parse(std::ostream&, std::ostream&) override {
                 try {
                     value = Parser::parse(arguments, value);
                     return true;
@@ -130,7 +130,7 @@ namespace cli {
                 }
             }
 
-            virtual std::string print_value() const {
+            virtual std::string print_value() const override {
                 return stringify(value);
             }
 
@@ -306,7 +306,7 @@ namespace cli {
         }
 
         template<typename T>
-        void set_optional(const std::string& name, const std::string& alternative, T defaultValue, const std::string& description = "", bool dominant = false) {
+        void set_optional(const std::string& name, const std::string& alternative, const T& defaultValue, const std::string& description = "", bool dominant = false) {
             auto command = new CmdArgument<T> { name, alternative, description, false, dominant };
             command->value = defaultValue;
             _commands.push_back(command);
