@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -381,9 +381,6 @@ public:
     }
 };
 
-/// @}
-// end of group blockmodule
-
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 
 template<
@@ -773,9 +770,15 @@ private:
     using block_exchange_type = block_exchange<T, BlockSizeX, ItemsPerThread, BlockSizeY, BlockSizeZ>;
 
 public:
-    static_assert(BlockSize % ::rocprim::device_warp_size() == 0,
-                 "BlockSize must be a multiple of hardware warpsize");
+    ROCPRIM_DETAIL_DEVICE_STATIC_ASSERT(BlockSize % ::rocprim::arch::wavefront::min_size() == 0,
+                                        "BlockSize must be a multiple of hardware warpsize");
 
+    ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
+    block_load()
+    {
+        assert(BlockSize % ::rocprim::arch::wavefront::size() == 0);
+    }
+     
     using storage_type = typename block_exchange_type::storage_type;
 
     template<class InputIterator>
@@ -888,4 +891,7 @@ public:
 
 END_ROCPRIM_NAMESPACE
 
+/// @}
+// end of group blockmodule
+
 #endif // ROCPRIM_BLOCK_BLOCK_LOAD_HPP_
