@@ -1,4 +1,4 @@
-// Copyright (c) 2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -24,9 +24,8 @@
 #include "../config.hpp"
 #include "../functional.hpp"
 
-#include "../detail/radix_sort.hpp"
-
 #include "block_scan.hpp"
+#include "config.hpp"
 
 #include "detail/block_radix_rank_basic.hpp"
 #include "detail/block_radix_rank_match.hpp"
@@ -62,7 +61,8 @@ struct select_block_radix_rank_impl<block_radix_rank_algorithm::basic>
     template<unsigned int BlockSizeX,
              unsigned int RadixBits,
              unsigned int BlockSizeY,
-             unsigned int BlockSizeZ>
+             unsigned int BlockSizeZ,
+             block_padding_hint>
     using type = block_radix_rank<BlockSizeX, RadixBits, false, BlockSizeY, BlockSizeZ>;
 };
 
@@ -72,7 +72,8 @@ struct select_block_radix_rank_impl<block_radix_rank_algorithm::basic_memoize>
     template<unsigned int BlockSizeX,
              unsigned int RadixBits,
              unsigned int BlockSizeY,
-             unsigned int BlockSizeZ>
+             unsigned int BlockSizeZ,
+             block_padding_hint>
     using type = block_radix_rank<BlockSizeX, RadixBits, true, BlockSizeY, BlockSizeZ>;
 };
 
@@ -82,8 +83,9 @@ struct select_block_radix_rank_impl<block_radix_rank_algorithm::match>
     template<unsigned int BlockSizeX,
              unsigned int RadixBits,
              unsigned int BlockSizeY,
-             unsigned int BlockSizeZ>
-    using type = block_radix_rank_match<BlockSizeX, RadixBits, BlockSizeY, BlockSizeZ>;
+             unsigned int BlockSizeZ,
+             block_padding_hint PaddingHint>
+    using type = block_radix_rank_match<BlockSizeX, RadixBits, BlockSizeY, BlockSizeZ, PaddingHint>;
 };
 } // namespace detail
 
@@ -98,6 +100,7 @@ struct select_block_radix_rank_impl<block_radix_rank_algorithm::match>
 /// the same values from shared memory twice, at the expense of more register usage.
 /// \tparam BlockSizeY - the number of threads in a block's y dimension, defaults to 1.
 /// \tparam BlockSizeZ - the number of threads in a block's z dimension, defaults to 1.
+/// \tparam PaddingHint - a hint that decides when to use padding. May not always be applicable.
 ///
 /// \par Overview
 /// * Key type must be an arithmetic type (that is, an integral type or a floating point type).
@@ -137,19 +140,21 @@ struct select_block_radix_rank_impl<block_radix_rank_algorithm::match>
 /// \endcode
 template<unsigned int               BlockSizeX,
          unsigned int               RadixBits,
-         block_radix_rank_algorithm Algorithm  = block_radix_rank_algorithm::default_algorithm,
-         unsigned int               BlockSizeY = 1,
-         unsigned int               BlockSizeZ = 1>
+         block_radix_rank_algorithm Algorithm   = block_radix_rank_algorithm::default_algorithm,
+         unsigned int               BlockSizeY  = 1,
+         unsigned int               BlockSizeZ  = 1,
+         block_padding_hint         PaddingHint = block_padding_hint::avoid_conflicts>
 class block_radix_rank
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
     : private detail::select_block_radix_rank_impl<
-          Algorithm>::template type<BlockSizeX, RadixBits, BlockSizeY, BlockSizeZ>
+          Algorithm>::template type<BlockSizeX, RadixBits, BlockSizeY, BlockSizeZ, PaddingHint>
 #endif
 {
     using base_type = typename detail::select_block_radix_rank_impl<
-        Algorithm>::template type<BlockSizeX, RadixBits, BlockSizeY, BlockSizeZ>;
+        Algorithm>::template type<BlockSizeX, RadixBits, BlockSizeY, BlockSizeZ, PaddingHint>;
 
 public:
+    /// \brief The number of digits each thread will process.
     static constexpr unsigned int digits_per_thread = base_type::digits_per_thread;
 
     /// \brief Struct used to allocate a temporary memory that is required for thread
@@ -203,6 +208,7 @@ public:
     ///     ...
     /// }
     /// \endcode
+    /// \endparblock
     template<typename Key, unsigned ItemsPerThread>
     ROCPRIM_DEVICE void rank_keys(const Key (&keys)[ItemsPerThread],
                                   unsigned int (&ranks)[ItemsPerThread],
@@ -277,6 +283,7 @@ public:
     ///     ...
     /// }
     /// \endcode
+    /// \endparblock
     template<typename Key, unsigned ItemsPerThread>
     ROCPRIM_DEVICE void rank_keys_desc(const Key (&keys)[ItemsPerThread],
                                        unsigned int (&ranks)[ItemsPerThread],
@@ -359,6 +366,7 @@ public:
     ///     ...
     /// }
     /// \endcode
+    /// \endparblock
     template<typename Key, unsigned ItemsPerThread, typename DigitExtractor>
     ROCPRIM_DEVICE void rank_keys(const Key (&keys)[ItemsPerThread],
                                   unsigned int (&ranks)[ItemsPerThread],
@@ -380,7 +388,6 @@ public:
     /// a key.
     /// \param [in] keys - reference to an array of keys provided by a thread.
     /// \param [out] ranks - reference to an array where the final ranks are written to.
-    /// \param [in] storage - reference to a temporary storage object of type \p storage_type.
     /// \param [in] digit_extractor - function object used to convert a key to a digit.
     /// The signature of the \p digit_extractor should be equivalent to the following:
     /// <tt>unsigned int f(const Key &key);</tt>. The signature does not need to have
@@ -445,6 +452,7 @@ public:
     ///     ...
     /// }
     /// \endcode
+    /// \endparblock
     template<typename Key, unsigned ItemsPerThread, typename DigitExtractor>
     ROCPRIM_DEVICE void rank_keys_desc(const Key (&keys)[ItemsPerThread],
                                        unsigned int (&ranks)[ItemsPerThread],
@@ -466,7 +474,6 @@ public:
     /// a key.
     /// \param [in] keys - reference to an array of keys provided by a thread.
     /// \param [out] ranks - reference to an array where the final ranks are written to.
-    /// \param [in] storage - reference to a temporary storage object of type \p storage_type.
     /// \param [in] digit_extractor - function object used to convert a key to a digit.
     /// The signature of the \p digit_extractor should be equivalent to the following:
     /// <tt>unsinged int f(const Key &key);</tt>. The signature does not need to have
