@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -47,7 +47,7 @@ class block_scan_warp_scan
     static constexpr unsigned int BlockSize = BlockSizeX * BlockSizeY * BlockSizeZ;
     // Select warp size
     static constexpr unsigned int warp_size_ =
-        detail::get_min_warp_size(BlockSize, ::rocprim::device_warp_size());
+        detail::get_min_warp_size(BlockSize, ::rocprim::arch::wavefront::min_size());
     // Number of warps in block
     static constexpr unsigned int warps_no_ = (BlockSize + warp_size_ - 1) / warp_size_;
 
@@ -77,7 +77,9 @@ class block_scan_warp_scan
     };
 
 public:
+    ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_WITH_PUSH
     using storage_type = detail::raw_storage<storage_type_>;
+    ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_POP
 
     template<class BinaryFunction>
     ROCPRIM_DEVICE ROCPRIM_INLINE
@@ -502,7 +504,7 @@ private:
                              T& output,
                              storage_type& storage,
                              BinaryFunction scan_op)
-        -> typename std::enable_if<(BlockSize_ > ::rocprim::device_warp_size())>::type
+        -> typename std::enable_if<(BlockSize_ > ::rocprim::arch::wavefront::min_size())>::type
     {
         storage_type_& storage_ = storage.get();
         // Perform warp scan
@@ -531,7 +533,7 @@ private:
                              T& output,
                              storage_type& storage,
                              BinaryFunction scan_op)
-        -> typename std::enable_if<!(BlockSize_ > ::rocprim::device_warp_size())>::type
+        -> typename std::enable_if<!(BlockSize_ > ::rocprim::arch::wavefront::min_size())>::type
     {
         (void) storage;
         (void) flat_tid;
@@ -558,7 +560,7 @@ private:
                              T init,
                              storage_type& storage,
                              BinaryFunction scan_op)
-        -> typename std::enable_if<(BlockSize_ > ::rocprim::device_warp_size())>::type
+        -> typename std::enable_if<(BlockSize_ > ::rocprim::arch::wavefront::min_size())>::type
     {
         storage_type_& storage_ = storage.get();
         // Perform warp scan on input values
@@ -598,7 +600,7 @@ private:
                              T init,
                              storage_type& storage,
                              BinaryFunction scan_op)
-        -> typename std::enable_if<!(BlockSize_ > ::rocprim::device_warp_size())>::type
+        -> typename std::enable_if<!(BlockSize_ > ::rocprim::arch::wavefront::min_size())>::type
     {
         (void) flat_tid;
         (void) storage;
@@ -633,7 +635,7 @@ private:
                              T& output,
                              storage_type& storage,
                              BinaryFunction scan_op)
-        -> typename std::enable_if<(BlockSize_ > ::rocprim::device_warp_size())>::type
+        -> typename std::enable_if<(BlockSize_ > ::rocprim::arch::wavefront::min_size())>::type
     {
         storage_type_& storage_ = storage.get();
         // Perform warp scan on input values
@@ -669,7 +671,7 @@ private:
                              T& output,
                              storage_type& storage,
                              BinaryFunction scan_op)
-        -> typename std::enable_if<!(BlockSize_ > ::rocprim::device_warp_size())>::type
+        -> typename std::enable_if<!(BlockSize_ > ::rocprim::arch::wavefront::min_size())>::type
     {
         (void) flat_tid;
         (void) storage;
