@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2018-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -29,7 +29,6 @@
 #include <cassert>
 
 #include "../config.hpp"
-#include "../intrinsics/thread.hpp"
 #include "../detail/various.hpp"
 
 /// \addtogroup primitivesmodule_deviceconfigs
@@ -49,7 +48,7 @@ struct default_config
     // merge_sort_config
     using block_sort_config  = default_config;
     using block_merge_config = default_config;
-    // radix_sort_config_v2
+    // radix_sort_config
     using single_sort_config = default_config;
     using merge_sort_config  = default_config;
     using onesweep_config    = default_config;
@@ -101,10 +100,13 @@ template<
     unsigned int SharedMemoryPerThread,
     // Most kernels require block sizes not smaller than warp
     unsigned int MinBlockSize, 
+    // If kernels require more than MaxBlockSize * SharedMemoryPerThread bytes
+    // (eg. to store some kind of block-wide state), that size can be specified here
+    unsigned int ExtraSharedMemory = 0,
     // Can fit in shared memory?
     // Although GPUs have 64KiB, 32KiB is used here as a "soft" limit,
     // because some additional memory may be required in kernels
-    bool = (MaxBlockSize * SharedMemoryPerThread <= (1u << 15))
+    bool = (MaxBlockSize * SharedMemoryPerThread + ExtraSharedMemory <= (1u << 15))
 >
 struct limit_block_size
 {
@@ -113,16 +115,18 @@ struct limit_block_size
         limit_block_size<
             detail::next_power_of_two(MaxBlockSize) / 2,
             SharedMemoryPerThread,
-            MinBlockSize
+            MinBlockSize,
+            ExtraSharedMemory
         >::value;
 };
 
 template<
     unsigned int MaxBlockSize,
     unsigned int SharedMemoryPerThread,
-    unsigned int MinBlockSize
+    unsigned int MinBlockSize,
+    unsigned int ExtraSharedMemory
 >
-struct limit_block_size<MaxBlockSize, SharedMemoryPerThread, MinBlockSize, true>
+struct limit_block_size<MaxBlockSize, SharedMemoryPerThread, MinBlockSize, ExtraSharedMemory, true>
 {
     static_assert(MaxBlockSize >= MinBlockSize, "Data is too large, it cannot fit in shared memory");
 
@@ -155,6 +159,7 @@ using default_or_custom_config =
         Config
     >::type;
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 enum class target_arch : unsigned int
 {
     // This must be zero, to initialize the device -> architecture cache
@@ -164,10 +169,15 @@ enum class target_arch : unsigned int
     gfx906  = 906,
     gfx908  = 908,
     gfx90a  = 910,
+    gfx942  = 942,
     gfx1030 = 1030,
+    gfx1100 = 1100,
     gfx1102 = 1102,
+    gfx1200 = 1200,
+    gfx1201 = 1201,
     unknown = std::numeric_limits<unsigned int>::max(),
 };
+#endif // DOXYGEN_SHOULD_SKIP_THIS
 
 /**
  * \brief Checks if the first `n` characters of `rhs` are equal to `lhs`
@@ -196,16 +206,29 @@ constexpr bool prefix_equals(const char* lhs, const char* rhs, std::size_t n)
 
 constexpr target_arch get_target_arch_from_name(const char* const arch_name, const std::size_t n)
 {
-    constexpr const char* target_names[]
-        = {"gfx803", "gfx900", "gfx906", "gfx908", "gfx90a", "gfx1030", "gfx1102"};
+    constexpr const char* target_names[]         = {"gfx803",
+                                                    "gfx900",
+                                                    "gfx906",
+                                                    "gfx908",
+                                                    "gfx90a",
+                                                    "gfx942",
+                                                    "gfx1030",
+                                                    "gfx1100",
+                                                    "gfx1102",
+                                                    "gfx1200",
+                                                    "gfx1201"};
     constexpr target_arch target_architectures[] = {
         target_arch::gfx803,
         target_arch::gfx900,
         target_arch::gfx906,
         target_arch::gfx908,
         target_arch::gfx90a,
+        target_arch::gfx942,
         target_arch::gfx1030,
+        target_arch::gfx1100,
         target_arch::gfx1102,
+        target_arch::gfx1200,
+        target_arch::gfx1201,
     };
     static_assert(sizeof(target_names) / sizeof(target_names[0])
                       == sizeof(target_architectures) / sizeof(target_architectures[0]),
@@ -225,16 +248,14 @@ constexpr target_arch get_target_arch_from_name(const char* const arch_name, con
 /**
  * \brief Get the current architecture in device compilation.
  * 
- * This function will always return `unkown` when called from the host, host could should instead
+ * This function will always return `unknown` when called from the host, host could should instead
  * call host_target_arch to query the current device from the HIP API.
  * 
  * \return target_arch the architecture currently being compiled for on the device.
  */
 constexpr target_arch device_target_arch()
 {
-#ifdef WIN32
-    return target_arch::unknown;
-#elif defined(__amdgcn_processor__)
+#if defined(__amdgcn_processor__)
     // The terminating zero is not counted in the length of the string
     return get_target_arch_from_name(__amdgcn_processor__,
                                      sizeof(__amdgcn_processor__) - sizeof('\0'));
@@ -260,10 +281,18 @@ auto dispatch_target_arch(const target_arch target_arch)
             return Config::template architecture_config<target_arch::gfx908>::params;
         case target_arch::gfx90a:
             return Config::template architecture_config<target_arch::gfx90a>::params;
+        case target_arch::gfx942:
+            return Config::template architecture_config<target_arch::gfx942>::params;
         case target_arch::gfx1030:
             return Config::template architecture_config<target_arch::gfx1030>::params;
+        case target_arch::gfx1100:
+            return Config::template architecture_config<target_arch::gfx1100>::params;
         case target_arch::gfx1102:
             return Config::template architecture_config<target_arch::gfx1102>::params;
+        case target_arch::gfx1200:
+            return Config::template architecture_config<target_arch::gfx1200>::params;
+        case target_arch::gfx1201:
+            return Config::template architecture_config<target_arch::gfx1201>::params;
         case target_arch::invalid:
             assert(false && "Invalid target architecture selected at runtime.");
     }
@@ -318,11 +347,18 @@ inline hipError_t get_device_arch(int device_id, target_arch& arch)
     return hipSuccess;
 }
 
-#ifndef WIN32
 inline hipError_t get_device_from_stream(const hipStream_t stream, int& device_id)
 {
     static constexpr hipStream_t default_stream = 0;
-    if(stream == default_stream || stream == hipStreamPerThread)
+
+    // hipStreamLegacy is supported in HIP >= 6.1.0
+#if (HIP_VERSION_MAJOR >= 6 && HIP_VERSION_MINOR >= 1)
+    const bool is_legacy_stream = (stream == hipStreamLegacy);
+#else
+    const bool is_legacy_stream = false;
+#endif
+
+    if (stream == default_stream || stream == hipStreamPerThread || is_legacy_stream);
     {
         const hipError_t result = hipGetDevice(&device_id);
         if(result != hipSuccess)
@@ -343,15 +379,9 @@ inline hipError_t get_device_from_stream(const hipStream_t stream, int& device_i
 #endif
     return hipSuccess;
 }
-#endif
 
 inline hipError_t host_target_arch(const hipStream_t stream, target_arch& arch)
 {
-#ifdef WIN32
-    (void)stream;
-    arch = target_arch::unknown;
-    return hipSuccess;
-#else
     int              device_id;
     const hipError_t result = get_device_from_stream(stream, device_id);
     if(result != hipSuccess)
@@ -360,11 +390,48 @@ inline hipError_t host_target_arch(const hipStream_t stream, target_arch& arch)
     }
 
     return get_device_arch(device_id, arch);
-#endif
 }
 
 } // end namespace detail
 
+/// \brief Returns a number of threads in a hardware warp for the actual device.
+/// At host side this constant is available at runtime only.
+/// \param device_id - the device that should be queried.
+/// \param warp_size - out parameter for the warp size.
+/// \return hipError_t any error that might occur.
+///
+/// It is constant for a device.
+ROCPRIM_HOST inline hipError_t host_warp_size(const int device_id, unsigned int& warp_size)
+{
+    warp_size = -1;
+    hipDeviceProp_t device_prop;
+    hipError_t      success = hipGetDeviceProperties(&device_prop, device_id);
+
+    if(success == hipSuccess)
+    {
+        warp_size = device_prop.warpSize;
+    }
+    return success;
+};
+
+/// \brief Returns the number of threads in a hardware warp for the device associated with the stream.
+/// At host side this constant is available at runtime only.
+/// \param stream - the stream, whose device should be queried.
+/// \param warp_size - out parameter for the warp size.
+/// \return hipError_t any error that might occur.
+///
+/// It is constant for a device.
+ROCPRIM_HOST inline hipError_t host_warp_size(const hipStream_t stream, unsigned int& warp_size)
+{
+    int        hip_device;
+    hipError_t success = detail::get_device_from_stream(stream, hip_device);
+    if(success == hipSuccess)
+    {
+        return host_warp_size(hip_device, warp_size);
+    }
+    return success;
+};
+
 END_ROCPRIM_NAMESPACE
 
 /// @}
