@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -494,22 +494,16 @@ ROCPRIM_DEVICE ROCPRIM_INLINE void
             unsigned int bin;
             if(sample_to_bin_op[channel](values[i].values[channel], bin))
             {
-                const unsigned int pos                 = flat_id * ItemsPerThread + i;
-                lane_mask_type     same_bin_lanes_mask = ::rocprim::ballot(pos < valid_count);
-                for(unsigned int b = 0; b < bins_bits[channel]; b++)
-                {
-                    const unsigned int   bit_set      = bin & (1u << b);
-                    const lane_mask_type bit_set_mask = ::rocprim::ballot(bit_set);
-                    same_bin_lanes_mask &= (bit_set ? bit_set_mask : ~bit_set_mask);
-                }
-                const unsigned int same_bin_count = ::rocprim::bit_count(same_bin_lanes_mask);
-                const unsigned int prev_same_bin_count
-                    = ::rocprim::masked_bit_count(same_bin_lanes_mask);
-                if(prev_same_bin_count == 0)
+                const unsigned int pos = flat_id * ItemsPerThread + i;
+                lane_mask_type     same_bin_lanes_mask
+                    = ::rocprim::match_any(bin, bins_bits[channel], pos < valid_count);
+
+                if(::rocprim::group_elect(same_bin_lanes_mask))
                 {
                     // Write the number of lanes having this bin,
                     // if the current lane is the first (and maybe only) lane with this bin.
-                    ::rocprim::detail::atomic_add(&histogram[channel][bin], same_bin_count);
+                    ::rocprim::detail::atomic_add(&histogram[channel][bin],
+                                                  ::rocprim::bit_count(same_bin_lanes_mask));
                 }
             }
         }
