@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -40,17 +40,17 @@ namespace detail
 {
 
 ROCPRIM_DEVICE ROCPRIM_INLINE
-range_t compute_range(const unsigned int id,
-                      const unsigned int size1,
-                      const unsigned int size2,
-                      const unsigned int spacing,
-                      const unsigned int p1,
-                      const unsigned int p2)
+range_t<> compute_range(const unsigned int id,
+                        const unsigned int size1,
+                        const unsigned int size2,
+                        const unsigned int spacing,
+                        const unsigned int p1,
+                        const unsigned int p2)
 {
     unsigned int diag1 = id * spacing;
     unsigned int diag2 = min(size1 + size2, diag1 + spacing);
 
-    return range_t{p1, p2, diag1 - p1, diag2 - p2};
+    return range_t<>{p1, p2, diag1 - p1, diag2 - p2};
 }
 
 template<
@@ -68,24 +68,23 @@ void partition_kernel_impl(IndexIterator indices,
                            const unsigned int spacing,
                            BinaryFunction compare_function)
 {
-    const unsigned int flat_id = ::rocprim::detail::block_thread_id<0>();
-    const unsigned int flat_block_id = ::rocprim::detail::block_id<0>();
+    const unsigned int flat_id         = ::rocprim::detail::block_thread_id<0>();
+    const unsigned int flat_block_id   = ::rocprim::detail::block_id<0>();
     const unsigned int flat_block_size = ::rocprim::detail::block_size<0>();
+    const unsigned int input_size      = input1_size + input2_size;
+    const unsigned int id              = flat_block_id * flat_block_size + flat_id;
+    const unsigned int partition_id    = id * spacing;
+    const unsigned int partitions      = (input_size + spacing - 1) / spacing;
 
-    unsigned int id = flat_block_id * flat_block_size + flat_id;
+    if(id > partitions)
+    {
+        return;
+    }
 
-    unsigned int partition_id = id * spacing;
     size_t diag = min(static_cast<size_t>(partition_id), input1_size + input2_size);
 
-    unsigned int begin =
-        merge_path(
-            keys_input1,
-            keys_input2,
-            input1_size,
-            input2_size,
-            diag,
-            compare_function
-        );
+    unsigned int begin
+        = merge_path(keys_input1, keys_input2, input1_size, input2_size, diag, compare_function);
 
     indices[id] = begin;
 }
@@ -122,22 +121,20 @@ void load(unsigned int flat_id,
     ::rocprim::syncthreads();
 }
 
-template<
-    unsigned int BlockSize,
-    class KeysInputIterator1,
-    class KeysInputIterator2,
-    class KeyType,
-    unsigned int ItemsPerThread,
-    class BinaryFunction
->
+template<unsigned int BlockSize,
+         class KeysInputIterator1,
+         class KeysInputIterator2,
+         class KeyType,
+         unsigned int ItemsPerThread,
+         class BinaryFunction>
 ROCPRIM_DEVICE ROCPRIM_INLINE
-void merge_keys(unsigned int flat_id,
+void merge_keys(unsigned int       flat_id,
                 KeysInputIterator1 keys_input1,
                 KeysInputIterator2 keys_input2,
                 KeyType (&key_inputs)[ItemsPerThread],
                 unsigned int (&index)[ItemsPerThread],
-                KeyType * keys_shared,
-                range_t range,
+                KeyType*       keys_shared,
+                range_t<>      range,
                 BinaryFunction compare_function)
 {
     load<BlockSize, ItemsPerThread>(
@@ -145,11 +142,7 @@ void merge_keys(unsigned int flat_id,
         keys_shared, range.count1(), range.count2()
     );
 
-    range_t range_local =
-        range_t {
-            0, range.count1(), range.count1(),
-            (range.count1() + range.count2())
-        };
+    range_t<> range_local{0, range.count1(), range.count1(), (range.count1() + range.count2())};
 
     unsigned int diag = ItemsPerThread * flat_id;
     unsigned int partition =
@@ -162,18 +155,12 @@ void merge_keys(unsigned int flat_id,
             compare_function
         );
 
-    range_t range_partition =
-        range_t {
-            range_local.begin1 + partition,
-            range_local.end1,
-            range_local.begin2 + diag - partition,
-            range_local.end2
-        };
+    range_t<> range_partition{range_local.begin1 + partition,
+                              range_local.end1,
+                              range_local.begin2 + diag - partition,
+                              range_local.end2};
 
-    serial_merge(
-        keys_shared, key_inputs, index, range_partition,
-        compare_function
-    );
+    serial_merge<false>(keys_shared, key_inputs, index, range_partition, compare_function);
 }
 
 template<
@@ -296,7 +283,9 @@ void merge_kernel_impl(IndexIterator indices,
 
     ROCPRIM_SHARED_MEMORY union
     {
+        ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_WITH_PUSH
         typename detail::raw_storage<key_type[input_block_size]> keys_shared;
+        ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_POP
         typename keys_store_type::storage_type keys_store;
     } storage;
 
@@ -310,14 +299,13 @@ void merge_kernel_impl(IndexIterator indices,
     const unsigned int valid_in_last_block = count - block_offset;
     const bool         is_incomplete_block = valid_in_last_block < items_per_block;
 
-    const unsigned int p1 = indices[flat_block_id];
-    const unsigned int p2 = indices[flat_block_id + 1];
+    const unsigned int partitions = (count + items_per_block - 1) / items_per_block;
 
-    range_t range =
-        compute_range(
-            flat_block_id, input1_size, input2_size, items_per_block,
-            p1, p2
-        );
+    const unsigned int p1 = indices[rocprim::min(flat_block_id, partitions)];
+    const unsigned int p2 = indices[rocprim::min(flat_block_id + 1, partitions)];
+
+    range_t<> range
+        = compute_range(flat_block_id, input1_size, input2_size, items_per_block, p1, p2);
 
     merge_keys<BlockSize>(
         flat_id, keys_input1, keys_input2, input, index,
