@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -254,7 +254,9 @@ struct block_permute_values_impl<Value,
         Value values[items_per_block];
     };
 
+    ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_WITH_PUSH
     using storage_type = raw_storage<storage_type_>;
+    ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_POP
 
     template<typename ValuesInputIterator, typename ValuesOutputIterator>
     ROCPRIM_DEVICE void permute(unsigned int (&ranks)[ItemsPerThread],
@@ -318,145 +320,13 @@ struct block_permute_values_impl<Value,
 
 template<typename Key,
          typename Value,
-         unsigned int         BlockSize,
-         unsigned int         ItemsPerThread,
-         block_sort_algorithm Algo,
+         unsigned int BlockSize,
+         unsigned int ItemsPerThread,
          typename Enable = void>
-struct block_sort_impl
-{
-    using stable_key_type = rocprim::tuple<Key, unsigned int>;
-
-    using keys_load_type
-        = block_load<Key, BlockSize, ItemsPerThread, block_load_method::block_load_transpose>;
-
-    using sort_type
-        = block_sort<stable_key_type, BlockSize, ItemsPerThread, rocprim::empty_type, Algo>;
-
-    using keys_store_type
-        = block_store<Key, BlockSize, ItemsPerThread, block_store_method::block_store_transpose>;
-
-    using values_permute_type = block_permute_values_impl<Value, BlockSize, ItemsPerThread>;
-
-    union storage_type
-    {
-        typename keys_load_type::storage_type      load_keys;
-        typename sort_type::storage_type           sort;
-        typename keys_store_type::storage_type     store_keys;
-        typename values_permute_type::storage_type permute_values;
-    };
-
-    template<typename KeysInputIterator,
-             typename KeysOutputIterator,
-             typename ValuesInputIterator,
-             typename ValuesOutputIterator,
-             typename BinaryFunction>
-    ROCPRIM_DEVICE  ROCPRIM_FORCE_INLINE
-    void sort(const unsigned int   valid_in_last_block,
-              const bool           is_incomplete_block,
-              KeysInputIterator    keys_input,
-              KeysOutputIterator   keys_output,
-              ValuesInputIterator  values_input,
-              ValuesOutputIterator values_output,
-              BinaryFunction       compare_function,
-              storage_type&        storage)
-    {
-        // By default, the block sort algorithm is not stable. We can make it stable
-        // by adding an index to each key.
-
-        Key keys[ItemsPerThread];
-
-        if(is_incomplete_block)
-        {
-            keys_load_type().load(keys_input, keys, valid_in_last_block, storage.load_keys);
-        }
-        else
-        {
-            keys_load_type().load(keys_input, keys, storage.load_keys);
-        }
-
-        const auto flat_id = block_thread_id<0>();
-
-        stable_key_type stable_keys[ItemsPerThread];
-        ROCPRIM_UNROLL
-        for(unsigned int i = 0; i < ItemsPerThread; ++i)
-        {
-            stable_keys[i] = rocprim::make_tuple(keys[i], flat_id * ItemsPerThread + i);
-        }
-
-        syncthreads();
-
-        // Special compare function that enforces sorting is stable.
-        auto stable_compare_function
-            = [compare_function](const stable_key_type& a,
-                                 const stable_key_type& b) ROCPRIM_FORCE_INLINE mutable
-        {
-            const bool ab = compare_function(rocprim::get<0>(a), rocprim::get<0>(b));
-            return ab
-                   || (!compare_function(rocprim::get<0>(b), rocprim::get<0>(a))
-                       && (rocprim::get<1>(a) < rocprim::get<1>(b)));
-        };
-
-        if(is_incomplete_block)
-        {
-            // Special compare function that enforces sorting is stable, and that out-of-bounds elements
-            // are not compared.
-            auto stable_oob_compare_function
-                = [stable_compare_function, valid_in_last_block](const stable_key_type& a,
-                                                                 const stable_key_type& b) mutable
-            {
-                const bool a_oob = rocprim::get<1>(a) >= valid_in_last_block;
-                const bool b_oob = rocprim::get<1>(b) >= valid_in_last_block;
-                return a_oob || b_oob ? !a_oob : stable_compare_function(a, b);
-            };
-
-            // Note: rocprim::block_sort with an algorithm that is not stable_merge_sort does not implement sorting
-            // a misaligned amount of items.
-            sort_type().sort(stable_keys, storage.sort, stable_oob_compare_function);
-
-            unsigned int ranks[ItemsPerThread];
-            ROCPRIM_UNROLL
-            for(unsigned int i = 0; i < ItemsPerThread; ++i)
-            {
-                keys[i]  = rocprim::get<0>(stable_keys[i]);
-                ranks[i] = rocprim::get<1>(stable_keys[i]);
-            }
-
-            syncthreads();
-            keys_store_type().store(keys_output, keys, valid_in_last_block, storage.store_keys);
-            values_permute_type().permute(ranks,
-                                          values_input,
-                                          values_output,
-                                          valid_in_last_block,
-                                          storage.permute_values);
-        }
-        else
-        {
-            sort_type().sort(stable_keys, storage.sort, stable_compare_function);
-
-            unsigned int ranks[ItemsPerThread];
-            ROCPRIM_UNROLL
-            for(unsigned int i = 0; i < ItemsPerThread; ++i)
-            {
-                keys[i]  = rocprim::get<0>(stable_keys[i]);
-                ranks[i] = rocprim::get<1>(stable_keys[i]);
-            }
-
-            syncthreads();
-            keys_store_type().store(keys_output, keys, storage.store_keys);
-            values_permute_type().permute(ranks,
-                                          values_input,
-                                          values_output,
-                                          storage.permute_values);
-        }
-    }
-};
+struct block_sort_impl;
 
 template<typename Key, unsigned int BlockSize, unsigned int ItemsPerThread>
-struct block_sort_impl<Key,
-                       rocprim::empty_type,
-                       BlockSize,
-                       ItemsPerThread,
-                       block_sort_algorithm::stable_merge_sort>
+struct block_sort_impl<Key, rocprim::empty_type, BlockSize, ItemsPerThread>
 {
     using keys_load_type
         = block_load<Key, BlockSize, ItemsPerThread, block_load_method::block_load_transpose>;
@@ -512,12 +382,12 @@ struct block_sort_impl<Key,
     }
 };
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 template<typename Key, typename Value, unsigned int BlockSize, unsigned int ItemsPerThread>
 struct block_sort_impl<Key,
                        Value,
                        BlockSize,
                        ItemsPerThread,
-                       block_sort_algorithm::stable_merge_sort,
                        std::enable_if_t<(sizeof(Value) <= sizeof(int))>>
 {
     using keys_load_type
@@ -593,13 +463,11 @@ struct block_sort_impl<Key,
         }
     }
 };
-
 template<typename Key, typename Value, unsigned int BlockSize, unsigned int ItemsPerThread>
 struct block_sort_impl<Key,
                        Value,
                        BlockSize,
                        ItemsPerThread,
-                       block_sort_algorithm::stable_merge_sort,
                        std::enable_if_t<(sizeof(Value) > sizeof(int))>>
 {
     using keys_load_type
@@ -675,35 +543,41 @@ struct block_sort_impl<Key,
         }
     }
 };
+#endif // DOXYGEN_SHOULD_SKIP_THIS
 
-template<unsigned int         BlockSize,
-         unsigned int         ItemsPerThread,
-         block_sort_algorithm Algo,
+template<unsigned int BlockSize,
+         unsigned int ItemsPerThread,
          class KeysInputIterator,
          class KeysOutputIterator,
          class ValuesInputIterator,
          class ValuesOutputIterator,
          class OffsetT,
-         class BinaryFunction,
-         class ValueType = typename std::iterator_traits<ValuesInputIterator>::value_type>
-ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE auto block_sort_kernel_impl(KeysInputIterator    keys_input,
-                                                                KeysOutputIterator   keys_output,
-                                                                ValuesInputIterator  values_input,
-                                                                ValuesOutputIterator values_output,
-                                                                const OffsetT        input_size,
-                                                                BinaryFunction compare_function)
+         class BinaryFunction>
+ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE
+void block_sort_kernel_impl(KeysInputIterator    keys_input,
+                            KeysOutputIterator   keys_output,
+                            ValuesInputIterator  values_input,
+                            ValuesOutputIterator values_output,
+                            const OffsetT        input_size,
+                            const unsigned int   num_blocks,
+                            BinaryFunction       compare_function)
 {
     using key_type   = typename std::iterator_traits<KeysInputIterator>::value_type;
     using value_type = typename std::iterator_traits<ValuesInputIterator>::value_type;
 
-    const unsigned int     flat_block_id   = block_id<0>();
     constexpr unsigned int items_per_block = BlockSize * ItemsPerThread;
 
-    const OffsetT      block_offset        = flat_block_id * items_per_block;
+    const unsigned int flat_block_id = ::rocprim::flat_block_id();
+    if(flat_block_id >= num_blocks)
+    {
+        return;
+    }
+
+    const OffsetT      block_offset        = static_cast<OffsetT>(flat_block_id) * items_per_block;
     const unsigned int valid_in_last_block = input_size - block_offset;
     const bool         is_incomplete_block = flat_block_id == (input_size / items_per_block);
 
-    using sort_impl = block_sort_impl<key_type, value_type, BlockSize, ItemsPerThread, Algo>;
+    using sort_impl = block_sort_impl<key_type, value_type, BlockSize, ItemsPerThread>;
 
     ROCPRIM_SHARED_MEMORY typename sort_impl::storage_type storage;
 
@@ -725,38 +599,40 @@ template<unsigned int BlockSize,
          class ValuesOutputIterator,
          class OffsetT,
          class BinaryFunction>
-ROCPRIM_DEVICE ROCPRIM_INLINE void block_merge_oddeven_kernel(KeysInputIterator    keys_input,
-                                                              KeysOutputIterator   keys_output,
-                                                              ValuesInputIterator  values_input,
-                                                              ValuesOutputIterator values_output,
-                                                              const OffsetT        input_size,
-                                                              const OffsetT  sorted_block_size,
-                                                              BinaryFunction compare_function)
+ROCPRIM_DEVICE ROCPRIM_INLINE
+void block_merge_oddeven_kernel(KeysInputIterator    keys_input,
+                                KeysOutputIterator   keys_output,
+                                ValuesInputIterator  values_input,
+                                ValuesOutputIterator values_output,
+                                const OffsetT        input_size,
+                                const OffsetT        sorted_block_size,
+                                BinaryFunction       compare_function)
 {
     using key_type             = typename std::iterator_traits<KeysInputIterator>::value_type;
     using value_type           = typename std::iterator_traits<ValuesInputIterator>::value_type;
     constexpr bool with_values = !std::is_same<value_type, ::rocprim::empty_type>::value;
 
-    constexpr unsigned int items_per_block     = BlockSize * ItemsPerThread;
-    const unsigned int     flat_id             = ::rocprim::detail::block_thread_id<0>();
-    const unsigned int     flat_block_id       = ::rocprim::detail::block_id<0>();
-    const bool             is_incomplete_block = flat_block_id == (input_size / items_per_block);
+    constexpr unsigned int items_per_block = BlockSize * ItemsPerThread;
+
+    const unsigned int flat_id             = ::rocprim::detail::block_thread_id<0>();
+    const unsigned int flat_block_id       = ::rocprim::detail::block_id<0>();
+    const bool         is_incomplete_block = flat_block_id == (input_size / items_per_block);
     // ^ bounds-checking: if input_size is not a multiple of items_per_block and
     // this is the last block: true, false otherwise
-    const OffsetT block_offset        = flat_block_id * items_per_block;
-    const OffsetT valid_in_last_block = input_size - block_offset;
+    const OffsetT      block_offset        = static_cast<OffsetT>(flat_block_id) * items_per_block;
+    const unsigned int valid_in_last_block = input_size - block_offset;
 
-    const OffsetT thread_offset = flat_id * ItemsPerThread;
-    if(thread_offset >= valid_in_last_block)
-    {
-        return;
-    }
+    const unsigned int thread_offset = flat_id * ItemsPerThread;
 
     key_type   keys[ItemsPerThread];
     value_type values[ItemsPerThread];
 
     if(is_incomplete_block)
     {
+        if(thread_offset >= valid_in_last_block)
+        {
+            return;
+        }
         block_load_direct_blocked(flat_id, keys_input + block_offset, keys, valid_in_last_block);
 
         if ROCPRIM_IF_CONSTEXPR(with_values)
@@ -780,8 +656,8 @@ ROCPRIM_DEVICE ROCPRIM_INLINE void block_merge_oddeven_kernel(KeysInputIterator
     const unsigned int mask                = merged_tiles_number - 1;
     // tilegroup_id is the id of the input sorted_block
     const unsigned int tilegroup_id = ~mask & flat_block_id;
-    const unsigned int block_is_odd = merged_tiles_number & tilegroup_id;
-    const OffsetT      block_start  = tilegroup_id * items_per_block;
+    const bool         block_is_odd = (merged_tiles_number & tilegroup_id) != 0;
+    const OffsetT      block_start  = static_cast<OffsetT>(tilegroup_id) * items_per_block;
     const OffsetT      next_block_start_
         = block_is_odd ? block_start - sorted_block_size : block_start + sorted_block_size;
     const OffsetT next_block_start = min(next_block_start_, input_size);
@@ -796,7 +672,7 @@ ROCPRIM_DEVICE ROCPRIM_INLINE void block_merge_oddeven_kernel(KeysInputIterator
             ROCPRIM_UNROLL
             for(unsigned int i = 0; i < ItemsPerThread; i++)
             {
-                const unsigned int id = block_offset + thread_offset + i;
+                const OffsetT id = block_offset + thread_offset + i;
                 if(id < input_size)
                 {
                     keys_output[id] = keys[i];
@@ -812,8 +688,8 @@ ROCPRIM_DEVICE ROCPRIM_INLINE void block_merge_oddeven_kernel(KeysInputIterator
             ROCPRIM_UNROLL
             for(unsigned int i = 0; i < ItemsPerThread; i++)
             {
-                const unsigned int id = block_offset + thread_offset + i;
-                keys_output[id]       = keys[i];
+                const OffsetT id = block_offset + thread_offset + i;
+                keys_output[id]  = keys[i];
                 if ROCPRIM_IF_CONSTEXPR(with_values)
                 {
                     values_output[id] = values[i];
