@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -34,20 +34,43 @@
 #include "../../block/block_scan.hpp"
 #include "../../block/block_discontinuity.hpp"
 
+#include "../config_types.hpp"
+#include "device_config_helper.hpp"
 #include "lookback_scan_state.hpp"
+#include "rocprim/type_traits.hpp"
+#include "rocprim/types/tuple.hpp"
 
 BEGIN_ROCPRIM_NAMESPACE
 
 namespace detail
 {
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+
 enum class select_method
 {
-    flag = 0,
-    predicate = 1,
-    unique = 2
+    flag            = 0,
+    predicate       = 1,
+    predicated_flag = 2,
+    unique          = 3
 };
 
+enum class partition_subalgo
+{
+    partition_two_way_predicate,
+    partition_two_way_flag,
+    partition_flag,
+    partition_predicate,
+    partition_three_way,
+    select_flag,
+    select_predicate,
+    select_predicated_flag,
+    select_unique,
+    select_unique_by_key
+};
+
+#endif // DOXYGEN_SHOULD_SKIP_THIS
+
 template<select_method SelectMethod,
          unsigned int  BlockSize,
          class BlockLoadFlagsType,
@@ -93,6 +116,60 @@ ROCPRIM_DEVICE ROCPRIM_INLINE auto
     ::rocprim::syncthreads(); // sync threads to reuse shared memory
 }
 
+template<select_method SelectMethod,
+         unsigned int  BlockSize,
+         class BlockLoadFlagsType,
+         class BlockDiscontinuityType,
+         class InputIterator,
+         class FlagIterator,
+         class ValueType,
+         unsigned int ItemsPerThread,
+         class UnaryPredicate,
+         class InequalityOp,
+         class StorageType>
+ROCPRIM_DEVICE ROCPRIM_INLINE
+auto partition_block_load_flags(InputIterator /* block_predecessor */,
+                                FlagIterator block_flags,
+                                ValueType (& /* values */)[ItemsPerThread],
+                                bool (&is_selected)[ItemsPerThread],
+                                UnaryPredicate predicate,
+                                InequalityOp /* inequality_op */,
+                                StorageType& storage,
+                                const bool /* is_first_block */,
+                                const unsigned int block_thread_id,
+                                const bool         is_global_last_block,
+                                const unsigned int valid_in_global_last_block) ->
+    typename std::enable_if<SelectMethod == select_method::predicated_flag>::type
+{
+    using flag_type = typename std::iterator_traits<FlagIterator>::value_type;
+    flag_type flags[ItemsPerThread];
+
+    if(is_global_last_block) // last block
+    {
+        BlockLoadFlagsType().load(block_flags,
+                                  flags,
+                                  valid_in_global_last_block,
+                                  false,
+                                  storage.load_flags);
+        const auto offset = block_thread_id * ItemsPerThread;
+        ROCPRIM_UNROLL
+        for(unsigned int i = 0; i < ItemsPerThread; i++)
+        {
+            is_selected[i] = ((offset + i) < valid_in_global_last_block) && predicate(flags[i]);
+        }
+    }
+    else
+    {
+        BlockLoadFlagsType().load(block_flags, flags, storage.load_flags);
+        ROCPRIM_UNROLL
+        for(unsigned int i = 0; i < ItemsPerThread; i++)
+        {
+            is_selected[i] = predicate(flags[i]);
+        }
+    }
+    ::rocprim::syncthreads(); // sync threads to reuse shared memory
+}
+
 template<select_method SelectMethod,
          unsigned int  BlockSize,
          class BlockLoadFlagsType,
@@ -302,18 +379,97 @@ ROCPRIM_DEVICE ROCPRIM_INLINE void
     }
 }
 
+// two-way partition into one iterator
 template<bool         OnlySelected,
          unsigned int BlockSize,
          class ValueType,
          unsigned int ItemsPerThread,
          class OffsetType,
-         class OutputIterator,
+         class SelectType,
+         class ScatterStorageType>
+ROCPRIM_DEVICE ROCPRIM_INLINE auto
+    partition_scatter(ValueType (&values)[ItemsPerThread],
+                      bool (&is_selected)[ItemsPerThread],
+                      OffsetType (&output_indices)[ItemsPerThread],
+                      tuple<SelectType, ::rocprim::empty_type*> output,
+                      const size_t                              total_size,
+                      const OffsetType                          selected_prefix,
+                      const OffsetType                          selected_in_block,
+                      ScatterStorageType&                       storage,
+                      const unsigned int                        flat_block_id,
+                      const unsigned int                        flat_block_thread_id,
+                      const bool                                is_global_last_block,
+                      const unsigned int                        valid_in_global_last_block,
+                      size_t (&prev_selected_count_values)[1],
+                      size_t prev_processed) -> typename std::enable_if<!OnlySelected>::type
+{
+    constexpr unsigned int items_per_block = BlockSize * ItemsPerThread;
+
+    // Scatter selected/rejected values to shared memory
+    auto scatter_storage = storage.get();
+    ROCPRIM_UNROLL
+    for(unsigned int i = 0; i < ItemsPerThread; i++)
+    {
+        unsigned int item_index          = (flat_block_thread_id * ItemsPerThread) + i;
+        unsigned int selected_item_index = output_indices[i] - selected_prefix;
+        unsigned int rejected_item_index = (item_index - selected_item_index) + selected_in_block;
+        // index of item in scatter_storage
+        unsigned int scatter_index     = is_selected[i] ? selected_item_index : rejected_item_index;
+        scatter_storage[scatter_index] = values[i];
+    }
+    ::rocprim::syncthreads(); // sync threads to reuse shared memory
+
+    ValueType reloaded_values[ItemsPerThread];
+    for(unsigned int i = 0; i < ItemsPerThread; i++)
+    {
+        const unsigned int item_index = i * BlockSize + flat_block_thread_id;
+        reloaded_values[i]            = scatter_storage[item_index];
+    }
+
+    const auto calculate_scatter_index = [=](const unsigned int item_index) -> size_t
+    {
+        const size_t selected_output_index = prev_selected_count_values[0] + selected_prefix;
+        const size_t rejected_output_index = total_size + selected_output_index - prev_processed
+                                             - flat_block_id * items_per_block + selected_in_block
+                                             - 1;
+        return item_index < selected_in_block ? selected_output_index + item_index
+                                              : rejected_output_index - item_index;
+    };
+    if(is_global_last_block)
+    {
+        for(unsigned int i = 0; i < ItemsPerThread; i++)
+        {
+            const unsigned int item_index = i * BlockSize + flat_block_thread_id;
+            if(item_index < valid_in_global_last_block)
+            {
+                get<0>(output)[calculate_scatter_index(item_index)] = reloaded_values[i];
+            }
+        }
+    }
+    else
+    {
+        for(unsigned int i = 0; i < ItemsPerThread; i++)
+        {
+            const unsigned int item_index = i * BlockSize + flat_block_thread_id;
+            get<0>(output)[calculate_scatter_index(item_index)] = reloaded_values[i];
+        }
+    }
+}
+
+// two-way partition into two iterators
+template<bool         OnlySelected,
+         unsigned int BlockSize,
+         class ValueType,
+         unsigned int ItemsPerThread,
+         class OffsetType,
+         class SelectType,
+         class RejectType,
          class ScatterStorageType>
 ROCPRIM_DEVICE ROCPRIM_INLINE auto partition_scatter(ValueType (&values)[ItemsPerThread],
                                                      bool (&is_selected)[ItemsPerThread],
                                                      OffsetType (&output_indices)[ItemsPerThread],
-                                                     OutputIterator      output,
-                                                     const size_t        total_size,
+                                                     tuple<SelectType, RejectType> output,
+                                                     const size_t /*total_size*/,
                                                      const OffsetType    selected_prefix,
                                                      const OffsetType    selected_in_block,
                                                      ScatterStorageType& storage,
@@ -332,11 +488,11 @@ ROCPRIM_DEVICE ROCPRIM_INLINE auto partition_scatter(ValueType (&values)[ItemsPe
     ROCPRIM_UNROLL
     for(unsigned int i = 0; i < ItemsPerThread; i++)
     {
-        unsigned int item_index = (flat_block_thread_id * ItemsPerThread) + i;
+        unsigned int item_index          = (flat_block_thread_id * ItemsPerThread) + i;
         unsigned int selected_item_index = output_indices[i] - selected_prefix;
         unsigned int rejected_item_index = (item_index - selected_item_index) + selected_in_block;
         // index of item in scatter_storage
-        unsigned int scatter_index = is_selected[i] ? selected_item_index : rejected_item_index;
+        unsigned int scatter_index     = is_selected[i] ? selected_item_index : rejected_item_index;
         scatter_storage[scatter_index] = values[i];
     }
     ::rocprim::syncthreads(); // sync threads to reuse shared memory
@@ -348,15 +504,22 @@ ROCPRIM_DEVICE ROCPRIM_INLINE auto partition_scatter(ValueType (&values)[ItemsPe
         reloaded_values[i]            = scatter_storage[item_index];
     }
 
-    const auto calculate_scatter_index = [=](const unsigned int item_index) -> size_t
+    auto save_to_output = [=](const unsigned int item_index, const unsigned int i)
     {
-        const size_t selected_output_index
-            = prev_selected_count_values[0] + selected_prefix + item_index;
-        const size_t rejected_output_index = total_size + selected_output_index - prev_processed
-                                             - flat_block_id * items_per_block - 2 * item_index
-                                             + selected_in_block - 1;
-        return item_index < selected_in_block ? selected_output_index : rejected_output_index;
+        const size_t selected_output_index = prev_selected_count_values[0] + selected_prefix;
+        const size_t rejected_output_index = prev_processed + flat_block_id * items_per_block
+                                             - selected_output_index - selected_in_block;
+
+        if(item_index < selected_in_block)
+        {
+            get<0>(output)[selected_output_index + item_index] = reloaded_values[i];
+        }
+        else
+        {
+            get<1>(output)[rejected_output_index + item_index] = reloaded_values[i];
+        }
     };
+
     if(is_global_last_block)
     {
         for(unsigned int i = 0; i < ItemsPerThread; i++)
@@ -364,7 +527,7 @@ ROCPRIM_DEVICE ROCPRIM_INLINE auto partition_scatter(ValueType (&values)[ItemsPe
             const unsigned int item_index = i * BlockSize + flat_block_thread_id;
             if(item_index < valid_in_global_last_block)
             {
-                output[calculate_scatter_index(item_index)] = reloaded_values[i];
+                save_to_output(item_index, i);
             }
         }
     }
@@ -372,24 +535,26 @@ ROCPRIM_DEVICE ROCPRIM_INLINE auto partition_scatter(ValueType (&values)[ItemsPe
     {
         for(unsigned int i = 0; i < ItemsPerThread; i++)
         {
-            const unsigned int item_index               = i * BlockSize + flat_block_thread_id;
-            output[calculate_scatter_index(item_index)] = reloaded_values[i];
+            const unsigned int item_index = i * BlockSize + flat_block_thread_id;
+            save_to_output(item_index, i);
         }
     }
 }
 
+// two-way partition, selection only
 template<bool         OnlySelected,
          unsigned int BlockSize,
          class ValueType,
          unsigned int ItemsPerThread,
          class OffsetType,
-         class OutputIterator,
+         class SelectType,
+         class RejectType,
          class ScatterStorageType>
 ROCPRIM_DEVICE ROCPRIM_INLINE auto
     partition_scatter(ValueType (&values)[ItemsPerThread],
                       bool (&is_selected)[ItemsPerThread],
                       OffsetType (&output_indices)[ItemsPerThread],
-                      OutputIterator output,
+                      tuple<SelectType, RejectType> output,
                       const size_t /*total_size*/,
                       const OffsetType    selected_prefix,
                       const OffsetType    selected_in_block,
@@ -419,7 +584,8 @@ ROCPRIM_DEVICE ROCPRIM_INLINE auto
         // Coalesced write from shared memory to global memory
         for(unsigned int i = flat_block_thread_id; i < selected_in_block; i += BlockSize)
         {
-            output[prev_selected_count_values[0] + selected_prefix + i] = scatter_storage[i];
+            get<0>(output)[prev_selected_count_values[0] + selected_prefix + i]
+                = scatter_storage[i];
         }
     }
     else
@@ -431,13 +597,14 @@ ROCPRIM_DEVICE ROCPRIM_INLINE auto
             {
                 if(is_selected[i])
                 {
-                    output[prev_selected_count_values[0] + output_indices[i]] = values[i];
+                    get<0>(output)[prev_selected_count_values[0] + output_indices[i]] = values[i];
                 }
             }
         }
     }
 }
 
+// three-way partition
 template<bool         OnlySelected,
          unsigned int BlockSize,
          class ValueType,
@@ -594,6 +761,153 @@ ROCPRIM_DEVICE void load_selected_count(const size_t* const prev_selected_count,
     }
 }
 
+template<unsigned int ItemsPerThread,
+         bool         OnlySelected,
+         unsigned int BlockSize,
+         typename BlockLoadValueType,
+         typename ValueIterator,
+         typename ValueType>
+class partition_values_helper
+{
+
+private:
+    ValueType values[ItemsPerThread];
+
+public:
+    ROCPRIM_DEVICE void load(ValueIterator                              values_input,
+                             unsigned int                               valid,
+                             unsigned int                               is_global_last_block,
+                             typename BlockLoadValueType::storage_type& load_values)
+    {
+        // Load values and sync threads
+        if(is_global_last_block)
+        {
+            BlockLoadValueType().load(values_input, values, valid, load_values);
+        }
+        else
+        {
+            BlockLoadValueType().load(values_input, values, load_values);
+        }
+        ::rocprim::syncthreads();
+    }
+
+    template<class OffsetType, class OutputType, class ScatterStorageType>
+    ROCPRIM_DEVICE void store(bool (&is_selected)[ItemsPerThread],
+                              OffsetType (&output_indices)[ItemsPerThread],
+                              OutputType          values_output,
+                              const size_t        total_size,
+                              const OffsetType    selected_prefix,
+                              const OffsetType    selected_in_block,
+                              ScatterStorageType& storage,
+                              const unsigned int  flat_block_id,
+                              const unsigned int  flat_block_thread_id,
+                              const bool          is_global_last_block,
+                              const unsigned int  valid_in_global_last_block,
+                              size_t (&prev_selected_count_values)[1],
+                              size_t prev_processed)
+    {
+        // Sync threads and store values
+        ::rocprim::syncthreads();
+        partition_scatter<OnlySelected, BlockSize>(values,
+                                                   is_selected,
+                                                   output_indices,
+                                                   values_output,
+                                                   total_size,
+                                                   selected_prefix,
+                                                   selected_in_block,
+                                                   storage,
+                                                   flat_block_id,
+                                                   flat_block_thread_id,
+                                                   is_global_last_block,
+                                                   valid_in_global_last_block,
+                                                   prev_selected_count_values,
+                                                   prev_processed);
+    }
+
+    template<class OffsetType, class OutputType, class ScatterStorageType>
+    ROCPRIM_DEVICE void store(bool (&is_selected)[2][ItemsPerThread],
+                              OffsetType (&output_indices)[ItemsPerThread],
+                              OutputType          values_output,
+                              const size_t        total_size,
+                              const OffsetType    selected_prefix,
+                              const OffsetType    selected_in_block,
+                              ScatterStorageType& storage,
+                              const unsigned int  flat_block_id,
+                              const unsigned int  flat_block_thread_id,
+                              const bool          is_global_last_block,
+                              const unsigned int  valid_in_global_last_block,
+                              size_t (&prev_selected_count_values)[2],
+                              size_t prev_processed)
+    {
+        // Sync threads and store values
+        ::rocprim::syncthreads();
+        partition_scatter<OnlySelected, BlockSize>(values,
+                                                   is_selected,
+                                                   output_indices,
+                                                   values_output,
+                                                   total_size,
+                                                   selected_prefix,
+                                                   selected_in_block,
+                                                   storage,
+                                                   flat_block_id,
+                                                   flat_block_thread_id,
+                                                   is_global_last_block,
+                                                   valid_in_global_last_block,
+                                                   prev_selected_count_values,
+                                                   prev_processed);
+    }
+};
+
+template<unsigned int ItemsPerThread,
+         bool         OnlySelected,
+         unsigned int BlockSize,
+         typename BlockLoadValueType,
+         typename ValueIterator>
+class partition_values_helper<ItemsPerThread,
+                              OnlySelected,
+                              BlockSize,
+                              BlockLoadValueType,
+                              ValueIterator,
+                              rocprim::empty_type>
+{
+public:
+    ROCPRIM_DEVICE void
+        load(ValueIterator, unsigned int, unsigned int, typename BlockLoadValueType::storage_type&)
+    {}
+
+    template<class OffsetType, class OutputType, class ScatterStorageType>
+    ROCPRIM_DEVICE void store(bool[ItemsPerThread],
+                              OffsetType[ItemsPerThread],
+                              OutputType,
+                              const size_t,
+                              const OffsetType,
+                              const OffsetType,
+                              ScatterStorageType&,
+                              const unsigned int,
+                              const unsigned int,
+                              const bool,
+                              const unsigned int,
+                              size_t[ItemsPerThread],
+                              size_t)
+    {}
+
+    template<class OffsetType, class OutputType, class ScatterStorageType>
+    ROCPRIM_DEVICE void store(bool[2][ItemsPerThread],
+                              OffsetType[ItemsPerThread],
+                              OutputType,
+                              const size_t,
+                              const OffsetType,
+                              const OffsetType,
+                              ScatterStorageType&,
+                              const unsigned int,
+                              const unsigned int,
+                              const bool,
+                              const unsigned int,
+                              size_t[ItemsPerThread],
+                              size_t)
+    {}
+};
+
 template<select_method SelectMethod,
          bool          OnlySelected,
          class Config,
@@ -605,7 +919,36 @@ template<select_method SelectMethod,
          class InequalityOp,
          class OffsetLookbackScanState,
          class... UnaryPredicates>
-ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
+ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE auto partition_kernel_impl(KeyIterator,
+                                                               ValueIterator,
+                                                               FlagIterator,
+                                                               OutputKeyIterator,
+                                                               OutputValueIterator,
+                                                               size_t*,
+                                                               size_t*,
+                                                               size_t,
+                                                               const size_t,
+                                                               InequalityOp,
+                                                               OffsetLookbackScanState,
+                                                               const unsigned int,
+                                                               UnaryPredicates...)
+    -> std::enable_if_t<!is_lookback_kernel_runnable<OffsetLookbackScanState>()>
+{
+    // No need to build the kernel with sleep on a device that does not require it
+}
+
+template<select_method SelectMethod,
+         bool          OnlySelected,
+         class Config,
+         class KeyIterator,
+         class ValueIterator, // Can be rocprim::empty_type* if key only
+         class FlagIterator,
+         class OutputKeyIterator,
+         class OutputValueIterator,
+         class InequalityOp,
+         class OffsetLookbackScanState,
+         class... UnaryPredicates>
+ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE auto
     partition_kernel_impl(KeyIterator             keys_input,
                           ValueIterator           values_input,
                           FlagIterator            flags,
@@ -619,32 +962,31 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
                           OffsetLookbackScanState offset_scan_state,
                           const unsigned int      number_of_blocks,
                           UnaryPredicates... predicates)
+        -> std::enable_if_t<is_lookback_kernel_runnable<OffsetLookbackScanState>()>
 {
-    constexpr auto block_size = Config::block_size;
-    constexpr auto items_per_thread = Config::items_per_thread;
-    constexpr unsigned int items_per_block = block_size * items_per_thread;
+    static constexpr partition_config_params params = device_params<Config>();
+
+    constexpr auto         block_size       = params.kernel_config.block_size;
+    constexpr auto         items_per_thread = params.kernel_config.items_per_thread;
+    constexpr unsigned int items_per_block  = block_size * items_per_thread;
 
     using offset_type = typename OffsetLookbackScanState::value_type;
     using key_type = typename std::iterator_traits<KeyIterator>::value_type;
     using value_type = typename std::iterator_traits<ValueIterator>::value_type;
+    using flag_type =
+        typename std::conditional<SelectMethod == select_method::predicated_flag,
+                                  typename std::iterator_traits<FlagIterator>::value_type,
+                                  bool>::type;
 
     // Block primitives
-    using block_load_key_type = ::rocprim::block_load<
-        key_type, block_size, items_per_thread,
-        Config::key_block_load_method
-    >;
-    using block_load_value_type = ::rocprim::block_load<
-        value_type, block_size, items_per_thread,
-        Config::value_block_load_method
-    >;
-    using block_load_flag_type = ::rocprim::block_load<
-        bool, block_size, items_per_thread,
-        Config::flag_block_load_method
-    >;
-    using block_scan_offset_type = ::rocprim::block_scan<
-        offset_type, block_size,
-        Config::block_scan_method
-    >;
+    using block_load_key_type = ::rocprim::
+        block_load<key_type, block_size, items_per_thread, params.key_block_load_method>;
+    using block_load_value_type = ::rocprim::
+        block_load<value_type, block_size, items_per_thread, params.value_block_load_method>;
+    using block_load_flag_type = ::rocprim::
+        block_load<flag_type, block_size, items_per_thread, params.flag_block_load_method>;
+    using block_scan_offset_type
+        = ::rocprim::block_scan<offset_type, block_size, params.block_scan_method>;
     using block_discontinuity_key_type = ::rocprim::block_discontinuity<key_type, block_size>;
 
     // Offset prefix operation type
@@ -653,10 +995,12 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
     >;
 
     // Memory required for 2-phase scatter
-    using exchange_keys_storage_type = key_type[items_per_block];
-    using raw_exchange_keys_storage_type = typename detail::raw_storage<exchange_keys_storage_type>;
+    using exchange_keys_storage_type   = key_type[items_per_block];
     using exchange_values_storage_type = value_type[items_per_block];
+    ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_WITH_PUSH
+    using raw_exchange_keys_storage_type = typename detail::raw_storage<exchange_keys_storage_type>;
     using raw_exchange_values_storage_type = typename detail::raw_storage<exchange_values_storage_type>;
+    ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_POP
 
     using is_selected_type = std::conditional_t<
         sizeof...(UnaryPredicates) == 1,
@@ -689,7 +1033,7 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
     is_selected_type is_selected;
     offset_type      output_indices[items_per_thread];
 
-    // Load input values into values
+    // Load input keys into keys
     if(is_global_last_block)
     {
         block_load_key_type().load(keys_input + block_offset,
@@ -708,6 +1052,19 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
     }
     ::rocprim::syncthreads(); // sync threads to reuse shared memory
 
+    // Load values before other blocks modify them.
+    partition_values_helper<items_per_thread,
+                            OnlySelected,
+                            block_size,
+                            block_load_value_type,
+                            ValueIterator,
+                            value_type>
+        values_helper;
+    values_helper.load(values_input + block_offset,
+                       valid_in_global_last_block,
+                       is_global_last_block,
+                       storage.load_values);
+
     // Load selection flags into is_selected, generate them using
     // input value and selection predicate, or generate them using
     // block_discontinuity primitive
@@ -791,45 +1148,19 @@ ROCPRIM_DEVICE ROCPRIM_FORCE_INLINE void
                                                 prev_selected_count_values,
                                                 prev_processed);
 
-    static constexpr bool with_values = !std::is_same<value_type, ::rocprim::empty_type>::value;
-
-    if ROCPRIM_IF_CONSTEXPR (with_values) {
-        value_type values[items_per_thread];
-
-        ::rocprim::syncthreads(); // sync threads to reuse shared memory
-        if(is_global_last_block)
-        {
-            block_load_value_type().load(values_input + block_offset,
-                                         values,
-                                         valid_in_global_last_block,
-                                         storage.load_values);
-        }
-        else
-        {
-            block_load_value_type()
-                .load(
-                    values_input + block_offset,
-                    values,
-                    storage.load_values
-                );
-        }
-        ::rocprim::syncthreads(); // sync threads to reuse shared memory
-
-        partition_scatter<OnlySelected, block_size>(values,
-                                                    is_selected,
-                                                    output_indices,
-                                                    values_output,
-                                                    total_size,
-                                                    selected_prefix,
-                                                    selected_in_block,
-                                                    storage.exchange_values,
-                                                    flat_block_id,
-                                                    flat_block_thread_id,
-                                                    is_global_last_block,
-                                                    valid_in_global_last_block,
-                                                    prev_selected_count_values,
-                                                    prev_processed);
-    }
+    values_helper.store(is_selected,
+                        output_indices,
+                        values_output,
+                        total_size,
+                        selected_prefix,
+                        selected_in_block,
+                        storage.exchange_values,
+                        flat_block_id,
+                        flat_block_thread_id,
+                        is_global_last_block,
+                        valid_in_global_last_block,
+                        prev_selected_count_values,
+                        prev_processed);
 
     // Last block in grid stores number of selected values
     const bool is_last_block = flat_block_id == (number_of_blocks - 1);
