@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,22 +21,22 @@
 #ifndef ROCPRIM_DEVICE_DETAIL_DEVICE_REDUCE_HPP_
 #define ROCPRIM_DEVICE_DETAIL_DEVICE_REDUCE_HPP_
 
-#include <type_traits>
-#include <iterator>
-
 #include "../../config.hpp"
 #include "../../detail/temp_storage.hpp"
 #include "../../detail/various.hpp"
 #include "../config_types.hpp"
 #include "../device_reduce_config.hpp"
 
-#include "../../intrinsics.hpp"
 #include "../../functional.hpp"
+#include "../../intrinsics.hpp"
 #include "../../types.hpp"
 
 #include "../../block/block_load.hpp"
 #include "../../block/block_reduce.hpp"
 
+#include <iterator>
+#include <type_traits>
+
 BEGIN_ROCPRIM_NAMESPACE
 
 namespace detail
@@ -44,38 +44,28 @@ namespace detail
 
 // Helper functions for reducing final value with
 // initial value.
-template<
-    bool WithInitialValue,
-    class T,
-    class BinaryFunction
->
+template<bool WithInitialValue, class T, class BinaryFunction>
 ROCPRIM_DEVICE ROCPRIM_INLINE
-auto reduce_with_initial(T output,
-                         T initial_value,
-                         BinaryFunction reduce_op)
-    -> typename std::enable_if<WithInitialValue, T>::type
+auto reduce_with_initial(T output, T initial_value, BinaryFunction reduce_op) ->
+    typename std::enable_if<WithInitialValue, T>::type
 {
     return reduce_op(initial_value, output);
 }
 
-template<
-    bool WithInitialValue,
-    class T,
-    class BinaryFunction
->
+template<bool WithInitialValue, class T, class BinaryFunction>
 ROCPRIM_DEVICE ROCPRIM_INLINE
-auto reduce_with_initial(T output,
-                         T initial_value,
-                         BinaryFunction reduce_op)
-    -> typename std::enable_if<!WithInitialValue, T>::type
+auto reduce_with_initial(T output, T initial_value, BinaryFunction reduce_op) ->
+    typename std::enable_if<!WithInitialValue, T>::type
 {
-    (void) initial_value;
-    (void) reduce_op;
+    (void)initial_value;
+    (void)reduce_op;
     return output;
 }
 
 template<
     bool WithInitialValue,
+    bool         FitLarger,
+    unsigned int FitItems,
     class Config,
     class ResultType,
     class InputIterator,
@@ -92,8 +82,10 @@ void block_reduce_kernel_impl(InputIterator input,
 {
     static constexpr reduce_config_params params = device_params<Config>();
 
-    constexpr unsigned int block_size       = params.reduce_config.block_size;
-    constexpr unsigned int items_per_thread = params.reduce_config.items_per_thread;
+    constexpr unsigned int block_size = params.reduce_config.block_size;
+    constexpr unsigned int items_per_thread
+        = FitLarger ? params.reduce_config.items_per_thread * FitItems
+                    : ceiling_div(params.reduce_config.items_per_thread, FitItems);
 
     using result_type = ResultType;
 
@@ -101,22 +93,20 @@ void block_reduce_kernel_impl(InputIterator input,
         = ::rocprim::block_reduce<result_type, block_size, params.block_reduce_method>;
     constexpr unsigned int items_per_block = block_size * items_per_thread;
 
-    const unsigned int flat_id = ::rocprim::detail::block_thread_id<0>();
-    const unsigned int flat_block_id = ::rocprim::detail::block_id<0>();
-    const unsigned int block_offset = flat_block_id * items_per_block;
-    const unsigned int number_of_blocks = ::rocprim::detail::grid_size<0>();
-    auto valid_in_last_block = input_size - items_per_block * (number_of_blocks - 1);
+    const unsigned int flat_id             = ::rocprim::detail::block_thread_id<0>();
+    const unsigned int flat_block_id       = ::rocprim::detail::block_id<0>();
+    const size_t       block_offset        = flat_block_id * items_per_block;
+    const unsigned int valid_in_last_block = input_size - block_offset;
 
     result_type values[items_per_thread];
     result_type output_value;
-    if(flat_block_id == (number_of_blocks - 1)) // last block
+    // last incomplete block
+    if(flat_block_id == (input_size / items_per_block))
     {
-        block_load_direct_striped<block_size>(
-            flat_id,
-            input + block_offset,
-            values,
-            valid_in_last_block
-        );
+        block_load_direct_striped<block_size>(flat_id,
+                                              input + block_offset,
+                                              values,
+                                              valid_in_last_block);
 
         output_value = values[0];
         ROCPRIM_UNROLL
@@ -129,44 +119,33 @@ void block_reduce_kernel_impl(InputIterator input,
             }
         }
 
-        block_reduce_type()
-            .reduce(
-                output_value, // input
-                output_value, // output
-                valid_in_last_block,
-                reduce_op
-            );
+        block_reduce_type().reduce(output_value, // input
+                                   output_value, // output
+                                   std::min(valid_in_last_block, block_size),
+                                   reduce_op);
     }
     else
     {
-        block_load_direct_striped<block_size>(
-            flat_id,
-            input + block_offset,
-            values
-        );
+        block_load_direct_striped<block_size>(flat_id, input + block_offset, values);
 
         // load input values into values
-        block_reduce_type()
-            .reduce(
-                values, // input
-                output_value, // output
-                reduce_op
-            );
+        block_reduce_type().reduce(values, // input
+                                   output_value, // output
+                                   reduce_op);
     }
 
     // Save value into output
     if(flat_id == 0)
     {
-        output[flat_block_id] = input_size == 0
-            ? static_cast<result_type>(initial_value)
-            : reduce_with_initial<WithInitialValue>(
-                output_value,
-                static_cast<result_type>(initial_value),
-                reduce_op
-            );
+        output[flat_block_id]
+            = input_size == 0
+                  ? static_cast<result_type>(initial_value)
+                  : reduce_with_initial<WithInitialValue>(output_value,
+                                                          static_cast<result_type>(initial_value),
+                                                          reduce_op);
     }
 }
-} // end of detail namespace
+} // namespace detail
 
 END_ROCPRIM_NAMESPACE
 
