@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -56,6 +56,8 @@ void segmented_reduce(InputIterator input,
                       BinaryFunction reduce_op,
                       ResultType initial_value)
 {
+    using offset_type = typename std::iterator_traits<OffsetIterator>::value_type;
+
     static constexpr reduce_config_params params = device_params<Config>();
 
     constexpr unsigned int block_size       = params.reduce_config.block_size;
@@ -69,8 +71,8 @@ void segmented_reduce(InputIterator input,
     const unsigned int flat_id = ::rocprim::detail::block_thread_id<0>();
     const unsigned int segment_id = ::rocprim::detail::block_id<0>();
 
-    const unsigned int begin_offset = begin_offsets[segment_id];
-    const unsigned int end_offset = end_offsets[segment_id];
+    const offset_type begin_offset = begin_offsets[segment_id];
+    const offset_type end_offset   = end_offsets[segment_id];
 
     // Empty segment
     if(end_offset <= begin_offset)
@@ -83,16 +85,18 @@ void segmented_reduce(InputIterator input,
     }
 
     ResultType result;
-    unsigned int block_offset = begin_offset;
-    if(block_offset + items_per_block > end_offset)
+    offset_type block_offset = begin_offset;
+    if(block_offset + static_cast<offset_type>(items_per_block) > end_offset)
     {
         // Segment is shorter than items_per_block
 
-        // Load the partial block and reduce the current thread's values
+        // Load the partial block and reduce the current thread's values.
+        // valid_count is shorter than items_per_block, so it doesn't need to be
+        // of type offset_type.
         const unsigned int valid_count = end_offset - block_offset;
         if(flat_id < valid_count)
         {
-            unsigned int offset = block_offset + flat_id;
+            offset_type offset = block_offset + flat_id;
             result = input[offset];
             offset += block_size;
             while(offset < end_offset)
@@ -129,7 +133,7 @@ void segmented_reduce(InputIterator input,
         block_offset += items_per_block;
 
         // Load next full blocks and continue reduction
-        while(block_offset + items_per_block < end_offset)
+        while(block_offset + static_cast<offset_type>(items_per_block) < end_offset)
         {
             block_load_direct_striped<block_size>(flat_id, input + block_offset, values);
             for(unsigned int i = 0; i < items_per_thread; i++)
@@ -140,6 +144,8 @@ void segmented_reduce(InputIterator input,
         }
 
         // Load the last (probably partial) block and continue reduction
+        // For this last block the valid_count is less or equal than items_per_block,
+        // so it doesn't need to be of type offset_type.
         const unsigned int valid_count = end_offset - block_offset;
         block_load_direct_striped<block_size>(flat_id, input + block_offset, values, valid_count);
         for(unsigned int i = 0; i < items_per_thread; i++)
