@@ -1,4 +1,4 @@
-// Copyright (c) 2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -29,6 +29,7 @@
 #include "device_transform.hpp"
 
 #include "../config.hpp"
+#include "../common.hpp"
 #include "../functional.hpp"
 
 #include "../detail/temp_storage.hpp"
@@ -41,6 +42,7 @@
 #include <chrono>
 #include <iostream>
 #include <iterator>
+#include <type_traits>
 
 #include <cstddef>
 
@@ -52,41 +54,30 @@ BEGIN_ROCPRIM_NAMESPACE
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS // Do not document
 
-#define ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR(name, size, start)                           \
-    {                                                                                            \
-        auto _error = hipGetLastError();                                                         \
-        if(_error != hipSuccess)                                                                 \
-            return _error;                                                                       \
-        if(debug_synchronous)                                                                    \
-        {                                                                                        \
-            std::cout << name << "(" << size << ")";                                             \
-            auto __error = hipStreamSynchronize(stream);                                         \
-            if(__error != hipSuccess)                                                            \
-                return __error;                                                                  \
-            auto _end = std::chrono::high_resolution_clock::now();                               \
-            auto _d   = std::chrono::duration_cast<std::chrono::duration<double>>(_end - start); \
-            std::cout << " " << _d.count() * 1000 << " ms" << '\n';                              \
-        }                                                                                        \
-    }
-
 namespace detail
 {
-template <typename Config,
-          bool InPlace,
-          bool Right,
-          typename InputIt,
-          typename OutputIt,
-          typename BinaryFunction>
-void ROCPRIM_KERNEL __launch_bounds__(Config::block_size) adjacent_difference_kernel(
-    const InputIt                                             input,
-    const OutputIt                                            output,
-    const std::size_t                                         size,
-    const BinaryFunction                                      op,
-    const typename std::iterator_traits<InputIt>::value_type* previous_values,
-    const std::size_t                                         starting_block)
+template<typename Config,
+         bool InPlace,
+         bool Right,
+         typename InputIt,
+         typename OutputIt,
+         typename BinaryFunction>
+void ROCPRIM_KERNEL
+    __launch_bounds__(device_params<Config>().adjacent_difference_kernel_config.block_size)
+        adjacent_difference_kernel(
+            const InputIt                                             input,
+            const OutputIt                                            output,
+            const std::size_t                                         size,
+            const BinaryFunction                                      op,
+            const typename std::iterator_traits<InputIt>::value_type* previous_values,
+            const std::size_t                                         starting_block)
 {
-    adjacent_difference_kernel_impl<Config, InPlace, Right>(
-        input, output, size, op, previous_values, starting_block);
+    adjacent_difference_kernel_impl<Config, InPlace, Right>(input,
+                                                            output,
+                                                            size,
+                                                            op,
+                                                            previous_values,
+                                                            starting_block);
 }
 
 template <typename Config,
@@ -104,18 +95,28 @@ hipError_t adjacent_difference_impl(void* const          temporary_storage,
                                     const hipStream_t    stream,
                                     const bool           debug_synchronous)
 {
-    using value_type = typename std::iterator_traits<InputIt>::value_type;
+    using value_type  = typename std::iterator_traits<InputIt>::value_type;
+    using output_type = rocprim::invoke_result_binary_op_t<value_type, BinaryFunction>;
+    using larger_type
+        = std::conditional_t<(sizeof(value_type) >= sizeof(output_type)), value_type, output_type>;
 
-    using config = detail::default_or_custom_config<
-        Config,
-        detail::default_adjacent_difference_config<ROCPRIM_TARGET_ARCH, value_type>>;
+    using config = wrapped_adjacent_difference_config<Config, InPlace, larger_type>;
 
-    static constexpr unsigned int block_size       = config::block_size;
-    static constexpr unsigned int items_per_thread = config::items_per_thread;
-    static constexpr unsigned int items_per_block  = block_size * items_per_thread;
+    detail::target_arch target_arch;
+    hipError_t          result = detail::host_target_arch(stream, target_arch);
+    if(result != hipSuccess)
+    {
+        return result;
+    }
 
-    const std::size_t num_blocks = ceiling_div(size, items_per_block);
-    const std::size_t num_previous_values = InPlace && num_blocks >= 2 ? num_blocks - 1 : 0;
+    const detail::adjacent_difference_config_params params
+        = detail::dispatch_target_arch<config>(target_arch);
+
+    const unsigned int block_size       = params.adjacent_difference_kernel_config.block_size;
+    const unsigned int items_per_thread = params.adjacent_difference_kernel_config.items_per_thread;
+    const unsigned int items_per_block  = block_size * items_per_thread;
+    const std::size_t  num_blocks       = ceiling_div(size, items_per_block);
+    const std::size_t  num_previous_values = InPlace && num_blocks >= 2 ? num_blocks - 1 : 0;
 
     value_type* previous_values;
 
@@ -139,11 +140,12 @@ hipError_t adjacent_difference_impl(void* const          temporary_storage,
     {
         // If doing left adjacent diff then the last item of each block is needed for the
         // next block, otherwise the first item is needed for the previous block
-        static constexpr auto offset = items_per_block - (Right ? 0 : 1);
+        const auto offset = items_per_block - (Right ? 0 : 1);
 
-        const auto block_starts_iter = make_transform_iterator(
-            rocprim::make_counting_iterator(std::size_t {0}),
-            [base = input + offset](std::size_t i) { return base[i * items_per_block]; });
+        const auto block_starts_iter
+            = make_transform_iterator(rocprim::make_counting_iterator(std::size_t{0}),
+                                      [=, base = input + offset](std::size_t i) -> value_type
+                                      { return base[i * items_per_block]; });
 
         const hipError_t error = ::rocprim::transform(block_starts_iter,
                                                       previous_values,
@@ -157,9 +159,9 @@ hipError_t adjacent_difference_impl(void* const          temporary_storage,
         }
     }
 
-    static constexpr unsigned int size_limit     = config::size_limit;
-    static constexpr auto number_of_blocks_limit = std::max(size_limit / items_per_block, 1u);
-    static constexpr auto aligned_size_limit     = number_of_blocks_limit * items_per_block;
+    const unsigned int size_limit             = params.adjacent_difference_kernel_config.size_limit;
+    const auto         number_of_blocks_limit = std::max(size_limit / items_per_block, 1u);
+    const auto         aligned_size_limit     = number_of_blocks_limit * items_per_block;
 
     // Launch number_of_blocks_limit blocks while there is still at least as many blocks
     // left as the limit
@@ -183,14 +185,14 @@ hipError_t adjacent_difference_impl(void* const          temporary_storage,
         const auto current_blocks = ceiling_div(current_size, items_per_block);
         const auto starting_block = i * number_of_blocks_limit;
 
-        std::chrono::time_point<std::chrono::high_resolution_clock> start;
+        std::chrono::time_point<std::chrono::steady_clock> start;
         if(debug_synchronous)
         {
             std::cout << "index:            " << i << '\n';
             std::cout << "current_size:     " << current_size << '\n';
             std::cout << "number of blocks: " << current_blocks << '\n';
 
-            start = std::chrono::high_resolution_clock::now();
+            start = std::chrono::steady_clock::now();
         }
         hipLaunchKernelGGL(HIP_KERNEL_NAME(adjacent_difference_kernel<config, InPlace, Right>),
                            dim3(current_blocks),
@@ -210,7 +212,7 @@ hipError_t adjacent_difference_impl(void* const          temporary_storage,
 }
 } // namespace detail
 
-#undef ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR
+
 
 #endif // DOXYGEN_SHOULD_SKIP_THIS
 
@@ -231,26 +233,25 @@ hipError_t adjacent_difference_impl(void* const          temporary_storage,
 /// }
 /// \endcode
 ///
-/// \tparam Config - [optional] configuration of the primitive. It can be
-/// `adjacent_difference_config` or a class with the same members.
-/// \tparam InputIt - [inferred] random-access iterator type of the input range. Must meet the
+/// \tparam Config [optional] configuration of the primitive, must be `default_config` or `adjacent_difference_config`.
+/// \tparam InputIt [inferred] random-access iterator type of the input range. Must meet the
 /// requirements of a C++ InputIterator concept. It can be a simple pointer type.
-/// \tparam OutputIt - [inferred] random-access iterator type of the output range. Must meet the
+/// \tparam OutputIt [inferred] random-access iterator type of the output range. Must meet the
 /// requirements of a C++ OutputIterator concept. It can be a simple pointer type.
-/// \tparam BinaryFunction - [inferred] binary operation function object that will be applied to
+/// \tparam BinaryFunction [inferred] binary operation function object that will be applied to
 /// consecutive items. The signature of the function should be equivalent to the following:
 /// `U f(const T1& a, const T2& b)`. The signature does not need to have
 /// `const &`, but function object must not modify the object passed to it
-/// \param temporary_storage - pointer to a device-accessible temporary storage. When
+/// \param temporary_storage pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
 /// `storage_size` and function returns without performing the scan operation
-/// \param storage_size - reference to a size (in bytes) of `temporary_storage`
-/// \param input - iterator to the input range
-/// \param output - iterator to the output range, must have any overlap with input
-/// \param size - number of items in the input
-/// \param op - [optional] the binary operation to apply
-/// \param stream - [optional] HIP stream object. Default is `0` (the default stream)
-/// \param debug_synchronous - [optional] If true, synchronization after every kernel
+/// \param storage_size reference to a size (in bytes) of `temporary_storage`
+/// \param input iterator to the input range
+/// \param output iterator to the output range, must not have any overlap with input.
+/// \param size number of items in the input
+/// \param op [optional] the binary operation to apply
+/// \param stream [optional] HIP stream object. Default is `0` (the default stream)
+/// \param debug_synchronous [optional] If true, synchronization after every kernel
 /// launch is forced in order to check for errors and extra debugging info is printed to the
 /// standard output. Default value is `false`
 ///
@@ -327,23 +328,22 @@ hipError_t adjacent_difference(void* const          temporary_storage,
 /// }
 /// \endcode
 ///
-/// \tparam Config - [optional] configuration of the primitive. It can be
-/// `adjacent_difference_config` or a class with the same members.
-/// \tparam InputIt - [inferred] random-access iterator type of the value range. Must meet the
+/// \tparam Config [optional] configuration of the primitive, must be `default_config` or `adjacent_difference_config`.
+/// \tparam InputIt [inferred] random-access iterator type of the value range. Must meet the
 /// requirements of a C++ InputIterator concept. It can be a simple pointer type.
-/// \tparam BinaryFunction - [inferred] binary operation function object that will be applied to
+/// \tparam BinaryFunction [inferred] binary operation function object that will be applied to
 /// consecutive items. The signature of the function should be equivalent to the following:
 /// `U f(const T1& a, const T2& b)`. The signature does not need to have
 /// `const &`, but function object must not modify the object passed to it
-/// \param temporary_storage - pointer to a device-accessible temporary storage. When
+/// \param temporary_storage pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
 /// `storage_size` and function returns without performing the scan operation
-/// \param storage_size - reference to a size (in bytes) of `temporary_storage`
-/// \param values - iterator to the range values, will be overwritten with the results
-/// \param size - number of items in the input
-/// \param op - [optional] the binary operation to apply
-/// \param stream - [optional] HIP stream object. Default is `0` (the default stream)
-/// \param debug_synchronous - [optional] If true, synchronization after every kernel
+/// \param storage_size reference to a size (in bytes) of `temporary_storage`
+/// \param values iterator to the range values, will be overwritten with the results
+/// \param size number of items in the input
+/// \param op [optional] the binary operation to apply
+/// \param stream [optional] HIP stream object. Default is `0` (the default stream)
+/// \param debug_synchronous [optional] If true, synchronization after every kernel
 /// launch is forced in order to check for errors and extra debugging info is printed to the
 /// standard output. Default value is `false`
 ///
@@ -366,6 +366,64 @@ hipError_t adjacent_difference_inplace(void* const          temporary_storage,
         temporary_storage, storage_size, values, values, size, op, stream, debug_synchronous);
 }
 
+/// \brief Parallel primitive for applying a binary operation across pairs of consecutive elements
+/// in device accessible memory. Writes the output to the position of the left item.
+///
+/// \tparam Config [optional] configuration of the primitive, must be `default_config` or `adjacent_difference_config`.
+/// \tparam InputIt [inferred] random-access iterator type of the value range. Must meet the
+/// requirements of a C++ InputIterator concept. It can be a simple pointer type.
+/// \tparam OutputIt [inferred] random-access iterator type of the output range. Must meet the
+/// requirements of a C++ OutputIterator concept. It can be a simple pointer type.
+/// \tparam BinaryFunction [inferred] binary operation function object that will be applied to
+/// consecutive items. The signature of the function should be equivalent to the following:
+/// `U f(const T1& a, const T2& b)`. The signature does not need to have
+/// `const &`, but function object must not modify the object passed to it
+///
+/// \param temporary_storage pointer to a device-accessible temporary storage. When
+/// a null pointer is passed, the required allocation size (in bytes) is written to
+/// `storage_size` and function returns without performing the scan operation
+/// \param storage_size reference to a size (in bytes) of `temporary_storage`
+/// \param input iterator to the range values
+/// \param output iterator to the output range. Allowed to point to the same elements as `input`.
+///   Only complete overlap or no overlap at all is allowed between `input` and `output`. In other words
+///   writing to `output[i]` is only allowed to overwrite `input[i]`, any other element must not be changed.
+/// \param size number of items in the input
+/// \param op [optional] the binary operation to apply
+/// \param stream [optional] HIP stream object. Default is `0` (the default stream)
+/// \param debug_synchronous [optional] If true, synchronization after every kernel
+/// launch is forced in order to check for errors and extra debugging info is printed to the
+/// standard output. Default value is `false`
+///
+/// \return `hipSuccess` (0) on success, otherwise the HIP runtime error of
+/// type `hipError_t`
+///
+/// \note This function has to perform an extra copy due to (potentially) writing its values in-place. If it is known that `input` and `output`
+/// don't overlap then adjacent_difference should be preferred as it avoids this extra copy.
+template<typename Config = default_config,
+         typename InputIt,
+         typename OutputIt,
+         typename BinaryFunction = ::rocprim::minus<>>
+hipError_t adjacent_difference_inplace(void* const          temporary_storage,
+                                       std::size_t&         storage_size,
+                                       const InputIt        input,
+                                       const OutputIt       output,
+                                       const std::size_t    size,
+                                       const BinaryFunction op                = BinaryFunction{},
+                                       const hipStream_t    stream            = 0,
+                                       const bool           debug_synchronous = false)
+{
+    static constexpr bool in_place = true;
+    static constexpr bool right    = false;
+    return detail::adjacent_difference_impl<Config, in_place, right>(temporary_storage,
+                                                                     storage_size,
+                                                                     input,
+                                                                     output,
+                                                                     size,
+                                                                     op,
+                                                                     stream,
+                                                                     debug_synchronous);
+}
+
 /// \brief Parallel primitive for applying a binary operation across pairs of consecutive elements
 /// in device accessible memory. Writes the output to the position of the right item.
 ///
@@ -380,26 +438,25 @@ hipError_t adjacent_difference_inplace(void* const          temporary_storage,
 /// }
 /// \endcode
 ///
-/// \tparam Config - [optional] configuration of the primitive. It can be
-/// `adjacent_difference_config` or a class with the same members.
-/// \tparam InputIt - [inferred] random-access iterator type of the input range. Must meet the
+/// \tparam Config [optional] configuration of the primitive, must be `default_config` or `adjacent_difference_config`.
+/// \tparam InputIt [inferred] random-access iterator type of the input range. Must meet the
 /// requirements of a C++ InputIterator concept. It can be a simple pointer type.
-/// \tparam OutputIt - [inferred] random-access iterator type of the output range. Must meet the
+/// \tparam OutputIt [inferred] random-access iterator type of the output range. Must meet the
 /// requirements of a C++ OutputIterator concept. It can be a simple pointer type.
-/// \tparam BinaryFunction - [inferred] binary operation function object that will be applied to
+/// \tparam BinaryFunction [inferred] binary operation function object that will be applied to
 /// consecutive items. The signature of the function should be equivalent to the following:
 /// `U f(const T1& a, const T2& b)`. The signature does not need to have
 /// `const &`, but function object must not modify the object passed to it
-/// \param temporary_storage - pointer to a device-accessible temporary storage. When
+/// \param temporary_storage pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
 /// `storage_size` and function returns without performing the scan operation
-/// \param storage_size - reference to a size (in bytes) of `temporary_storage`
-/// \param input - iterator to the input range
-/// \param output - iterator to the output range, must have any overlap with input
-/// \param size - number of items in the input
-/// \param op - [optional] the binary operation to apply
-/// \param stream - [optional] HIP stream object. Default is `0` (the default stream)
-/// \param debug_synchronous - [optional] If true, synchronization after every kernel
+/// \param storage_size reference to a size (in bytes) of `temporary_storage`
+/// \param input iterator to the input range
+/// \param output iterator to the output range, must not have any overlap with input.
+/// \param size number of items in the input
+/// \param op [optional] the binary operation to apply
+/// \param stream [optional] HIP stream object. Default is `0` (the default stream)
+/// \param debug_synchronous [optional] If true, synchronization after every kernel
 /// launch is forced in order to check for errors and extra debugging info is printed to the
 /// standard output. Default value is `false`
 ///
@@ -476,23 +533,22 @@ hipError_t adjacent_difference_right(void* const          temporary_storage,
 /// }
 /// \endcode
 ///
-/// \tparam Config - [optional] configuration of the primitive. It can be
-/// `adjacent_difference_config` or a class with the same members.
-/// \tparam InputIt - [inferred] random-access iterator type of the value range. Must meet the
+/// \tparam Config [optional] configuration of the primitive, must be `default_config` or `adjacent_difference_config`.
+/// \tparam InputIt [inferred] random-access iterator type of the value range. Must meet the
 /// requirements of a C++ InputIterator concept. It can be a simple pointer type.
-/// \tparam BinaryFunction - [inferred] binary operation function object that will be applied to
+/// \tparam BinaryFunction [inferred] binary operation function object that will be applied to
 /// consecutive items. The signature of the function should be equivalent to the following:
 /// `U f(const T1& a, const T2& b)`. The signature does not need to have
 /// `const &`, but function object must not modify the object passed to it
-/// \param temporary_storage - pointer to a device-accessible temporary storage. When
+/// \param temporary_storage pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
 /// `storage_size` and function returns without performing the scan operation
-/// \param storage_size - reference to a size (in bytes) of `temporary_storage`
-/// \param values - iterator to the range values, will be overwritten with the results
-/// \param size - number of items in the input
-/// \param op - [optional] the binary operation to apply
-/// \param stream - [optional] HIP stream object. Default is `0` (the default stream)
-/// \param debug_synchronous - [optional] If true, synchronization after every kernel
+/// \param storage_size reference to a size (in bytes) of `temporary_storage`
+/// \param values iterator to the range values, will be overwritten with the results
+/// \param size number of items in the input
+/// \param op [optional] the binary operation to apply
+/// \param stream [optional] HIP stream object. Default is `0` (the default stream)
+/// \param debug_synchronous [optional] If true, synchronization after every kernel
 /// launch is forced in order to check for errors and extra debugging info is printed to the
 /// standard output. Default value is `false`
 ///
@@ -515,6 +571,63 @@ hipError_t adjacent_difference_right_inplace(void* const          temporary_stor
         temporary_storage, storage_size, values, values, size, op, stream, debug_synchronous);
 }
 
+/// \brief Parallel primitive for applying a binary operation across pairs of consecutive elements
+/// in device accessible memory. Writes the output to the position of the right item.
+///
+/// \tparam Config [optional] configuration of the primitive, must be `default_config` or `adjacent_difference_config`.
+/// \tparam InputIt [inferred] random-access iterator type of the value range. Must meet the
+/// requirements of a C++ InputIterator concept. It can be a simple pointer type.
+/// \tparam OutputIt [inferred] random-access iterator type of the output range. Must meet the
+/// requirements of a C++ OutputIterator concept. It can be a simple pointer type.
+/// \tparam BinaryFunction [inferred] binary operation function object that will be applied to
+/// consecutive items. The signature of the function should be equivalent to the following:
+/// `U f(const T1& a, const T2& b)`. The signature does not need to have
+/// `const &`, but function object must not modify the object passed to it
+
+/// \param temporary_storage pointer to a device-accessible temporary storage. When
+/// a null pointer is passed, the required allocation size (in bytes) is written to
+/// `storage_size` and function returns without performing the scan operation
+/// \param storage_size reference to a size (in bytes) of `temporary_storage`
+/// \param input iterator to the range values, will be overwritten with the results
+/// \param output iterator to the output range. Allowed to point to the same elements as `input`.
+///   Only complete overlap or no overlap at all is allowed between `input` and `output`. In other words
+///   writing to `output[i]` is only allowed to overwrite `input[i]`, any other element must not be changed.
+/// \param size number of items in the input
+/// \param op [optional] the binary operation to apply
+/// \param stream [optional] HIP stream object. Default is `0` (the default stream)
+/// \param debug_synchronous [optional] If true, synchronization after every kernel
+/// launch is forced in order to check for errors and extra debugging info is printed to the
+/// standard output. Default value is `false`
+///
+/// \return `hipSuccess` (0) on success, otherwise the HIP runtime error of
+/// type `hipError_t`
+/// \note This function has to perform an extra copy due to (potentially) writing its values in-place. If it is known that `input` and `output`
+/// don't overlap then adjacent_difference_right should be preferred as it avoids this extra copy.
+template<typename Config = default_config,
+         typename InputIt,
+         typename OutputIt,
+         typename BinaryFunction = ::rocprim::minus<>>
+hipError_t adjacent_difference_right_inplace(void* const          temporary_storage,
+                                             std::size_t&         storage_size,
+                                             const InputIt        input,
+                                             const OutputIt       output,
+                                             const std::size_t    size,
+                                             const BinaryFunction op     = BinaryFunction{},
+                                             const hipStream_t    stream = 0,
+                                             const bool           debug_synchronous = false)
+{
+    static constexpr bool in_place = true;
+    static constexpr bool right    = true;
+    return detail::adjacent_difference_impl<Config, in_place, right>(temporary_storage,
+                                                                     storage_size,
+                                                                     input,
+                                                                     output,
+                                                                     size,
+                                                                     op,
+                                                                     stream,
+                                                                     debug_synchronous);
+}
+
 /// @}
 // end of group devicemodule
 
