@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -22,6 +22,7 @@
 #define ROCPRIM_DEVICE_DEVICE_SCAN_BY_KEY_HPP_
 
 #include "../config.hpp"
+#include "../common.hpp"
 #include "../detail/temp_storage.hpp"
 #include "../detail/various.hpp"
 #include "../functional.hpp"
@@ -45,7 +46,8 @@ BEGIN_ROCPRIM_NAMESPACE
 namespace detail
 {
 
-template<bool Exclusive,
+template<lookback_scan_determinism Determinism,
+         bool                      Exclusive,
          typename Config,
          typename KeyInputIterator,
          typename InputIterator,
@@ -54,59 +56,45 @@ template<bool Exclusive,
          typename CompareFunction,
          typename BinaryFunction,
          typename LookbackScanState,
-         typename ResultType>
+         typename AccType>
 void __global__ __launch_bounds__(device_params<Config>().kernel_config.block_size)
-    device_scan_by_key_kernel(const KeyInputIterator                          keys,
-                              const InputIterator                             values,
-                              const OutputIterator                            output,
-                              const InitialValueType                          initial_value,
-                              const CompareFunction                           compare,
-                              const BinaryFunction                            scan_op,
-                              const LookbackScanState                         scan_state,
-                              const size_t                                    size,
-                              const size_t                                    starting_block,
-                              const size_t                                    number_of_blocks,
-                              const ::rocprim::tuple<ResultType, bool>* const previous_last_value)
+    device_scan_by_key_kernel(const KeyInputIterator                       keys,
+                              const InputIterator                          values,
+                              const OutputIterator                         output,
+                              const InitialValueType                       initial_value,
+                              const CompareFunction                        compare,
+                              const BinaryFunction                         scan_op,
+                              const LookbackScanState                      scan_state,
+                              const size_t                                 size,
+                              const size_t                                 starting_block,
+                              const size_t                                 number_of_blocks,
+                              const ::rocprim::tuple<AccType, bool>* const previous_last_value)
 {
-    device_scan_by_key_kernel_impl<Exclusive, Config>(keys,
-                                                      values,
-                                                      output,
-                                                      get_input_value(initial_value),
-                                                      compare,
-                                                      scan_op,
-                                                      scan_state,
-                                                      size,
-                                                      starting_block,
-                                                      number_of_blocks,
-                                                      previous_last_value);
+    device_scan_by_key_kernel_impl<Determinism, Exclusive, Config>(
+        keys,
+        values,
+        output,
+        static_cast<AccType>(get_input_value(initial_value)),
+        compare,
+        scan_op,
+        scan_state,
+        size,
+        starting_block,
+        number_of_blocks,
+        previous_last_value);
 }
 
-#define ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR(name, size, start)                           \
-    do                                                                                           \
-    {                                                                                            \
-        auto _error = hipGetLastError();                                                         \
-        if(_error != hipSuccess)                                                                 \
-            return _error;                                                                       \
-        if(debug_synchronous)                                                                    \
-        {                                                                                        \
-            std::cout << name << "(" << size << ")";                                             \
-            auto __error = hipStreamSynchronize(stream);                                         \
-            if(__error != hipSuccess)                                                            \
-                return __error;                                                                  \
-            auto _end = std::chrono::high_resolution_clock::now();                               \
-            auto _d   = std::chrono::duration_cast<std::chrono::duration<double>>(_end - start); \
-            std::cout << " " << _d.count() * 1000 << " ms" << '\n';                              \
-        }                                                                                        \
-    } while(false)
 
-template<bool Exclusive,
+template<lookback_scan_determinism Determinism,
+         bool                      Exclusive,
          typename Config,
          typename KeysInputIterator,
          typename InputIterator,
          typename OutputIterator,
          typename InitValueType,
          typename BinaryFunction,
-         typename CompareFunction>
+         typename CompareFunction,
+         typename AccType>
 inline hipError_t scan_by_key_impl(void* const           temporary_storage,
                                    size_t&               storage_size,
                                    KeysInputIterator     keys,
@@ -119,10 +107,9 @@ inline hipError_t scan_by_key_impl(void* const           temporary_storage,
                                    const hipStream_t     stream,
                                    const bool            debug_synchronous)
 {
-    using key_type             = typename std::iterator_traits<KeysInputIterator>::value_type;
-    using real_init_value_type = input_type_t<InitValueType>;
+    using key_type = typename std::iterator_traits<KeysInputIterator>::value_type;
 
-    using config = wrapped_scan_by_key_config<Config, key_type, real_init_value_type>;
+    using config = wrapped_scan_by_key_config<Config, key_type, AccType>;
 
     detail::target_arch target_arch;
     hipError_t          result = host_target_arch(stream, target_arch);
@@ -132,7 +119,7 @@ inline hipError_t scan_by_key_impl(void* const           temporary_storage,
     }
     const scan_by_key_config_params params = dispatch_target_arch<config>(target_arch);
 
-    using wrapped_type = ::rocprim::tuple<real_init_value_type, bool>;
+    using wrapped_type = ::rocprim::tuple<AccType, bool>;
 
     using scan_state_type            = detail::lookback_scan_state<wrapped_type>;
     using scan_state_with_sleep_type = detail::lookback_scan_state<wrapped_type, true>;
@@ -155,14 +142,20 @@ inline hipError_t scan_by_key_impl(void* const           temporary_storage,
     void*         scan_state_storage;
     wrapped_type* previous_last_value;
 
+    detail::temp_storage::layout layout{};
+    const hipError_t             layout_result
+        = scan_state_type::get_temp_storage_layout(number_of_blocks, stream, layout);
+    if(layout_result != hipSuccess)
+    {
+        return layout_result;
+    }
+
     const hipError_t partition_result = detail::temp_storage::partition(
         temporary_storage,
         storage_size,
         detail::temp_storage::make_linear_partition(
             // This is valid even with offset_scan_state_with_sleep_type
-            detail::temp_storage::make_partition(
-                &scan_state_storage,
-                scan_state_type::get_temp_storage_layout(number_of_blocks)),
+            detail::temp_storage::make_partition(&scan_state_storage, layout),
             detail::temp_storage::ptr_aligned_array(&previous_last_value,
                                                     use_limited_size ? 1 : 0)));
     if(partition_result != hipSuccess || temporary_storage == nullptr)
@@ -176,19 +169,28 @@ inline hipError_t scan_by_key_impl(void* const           temporary_storage,
     }
 
     bool use_sleep;
-    if(const hipError_t error = is_sleep_scan_state_used(use_sleep))
+    if(const hipError_t error = is_sleep_scan_state_used(stream, use_sleep))
     {
         return error;
     }
 
+    scan_state_type scan_state{};
+    hipError_t      scan_state_result
+        = scan_state_type::create(scan_state, scan_state_storage, number_of_blocks, stream);
+    scan_state_with_sleep_type scan_state_with_sleep{};
+    scan_state_result = scan_state_with_sleep_type::create(scan_state_with_sleep,
+                                                           scan_state_storage,
+                                                           number_of_blocks,
+                                                           stream);
+    if(scan_state_result != hipSuccess)
+    {
+        return scan_state_result;
+    }
+
     // Call the provided function with either scan_state or scan_state_with_sleep based on
-    // the value of use_sleep_scan_state
+    // the value of use_sleep
     auto with_scan_state
-        = [use_sleep,
-           scan_state = scan_state_type::create(scan_state_storage, number_of_blocks),
-           scan_state_with_sleep
-           = scan_state_with_sleep_type::create(scan_state_storage, number_of_blocks)](
-              auto&& func) mutable -> decltype(auto)
+        = [use_sleep, scan_state, scan_state_with_sleep](auto&& func) mutable -> decltype(auto)
     {
         if(use_sleep)
         {
@@ -227,14 +229,14 @@ inline hipError_t scan_by_key_impl(void* const           temporary_storage,
         const unsigned int init_grid_size = ceiling_div(scan_blocks, block_size);
 
         // Start point for time measurements
-        std::chrono::high_resolution_clock::time_point start;
+        std::chrono::steady_clock::time_point start;
         if(debug_synchronous)
         {
             std::cout << "index:            " << i << '\n';
             std::cout << "current_size:     " << current_size << '\n';
             std::cout << "number of blocks: " << scan_blocks << '\n';
 
-            start = std::chrono::high_resolution_clock::now();
+            start = std::chrono::steady_clock::now();
         }
 
         with_scan_state(
@@ -256,27 +258,28 @@ inline hipError_t scan_by_key_impl(void* const           temporary_storage,
 
         if(debug_synchronous)
         {
-            start = std::chrono::high_resolution_clock::now();
+            start = std::chrono::steady_clock::now();
         }
         with_scan_state(
             [&](auto& scan_state)
             {
-                hipLaunchKernelGGL(HIP_KERNEL_NAME(device_scan_by_key_kernel<Exclusive, config>),
-                                   dim3(scan_blocks),
-                                   dim3(block_size),
-                                   0,
-                                   stream,
-                                   keys + offset,
-                                   input + offset,
-                                   output + offset,
-                                   static_cast<real_init_value_type>(initial_value),
-                                   compare,
-                                   scan_op,
-                                   scan_state,
-                                   size,
-                                   i * number_of_blocks,
-                                   total_number_of_blocks,
-                                   i > 0 ? as_const_ptr(previous_last_value) : nullptr);
+                hipLaunchKernelGGL(
+                    HIP_KERNEL_NAME(device_scan_by_key_kernel<Determinism, Exclusive, config>),
+                    dim3(scan_blocks),
+                    dim3(block_size),
+                    0,
+                    stream,
+                    keys + offset,
+                    input + offset,
+                    output + offset,
+                    initial_value,
+                    compare,
+                    scan_op,
+                    scan_state,
+                    size,
+                    i * number_of_blocks,
+                    total_number_of_blocks,
+                    i > 0 ? as_const_ptr(previous_last_value) : nullptr);
             });
         ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR("device_scan_by_key_kernel",
                                                     current_size,
@@ -285,7 +288,7 @@ inline hipError_t scan_by_key_impl(void* const           temporary_storage,
     return hipSuccess;
 }
 
-#undef ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR
+
 }
 
 /// \addtogroup devicemodule
@@ -298,14 +301,19 @@ inline hipError_t scan_by_key_impl(void* const           temporary_storage,
 ///
 /// \par Overview
 /// * Supports non-commutative scan operators. However, a scan operator should be
-/// associative. When used with non-associative functions the results may be non-deterministic
-/// and/or vary in precision.
+/// associative.
+/// * When used with non-associative functions (e.g. floating point arithmetic operations):
+///    - the results may be non-deterministic and/or vary in precision,
+///    - and bit-wise reproducibility is not guaranteed, that is, results from multiple runs
+///      using the same input values on the same device may not be bit-wise identical.
+///   If deterministic behavior is required, Use \link deterministic_inclusive_scan_by_key()
+///   rocprim::deterministic_inclusive_scan_by_key \endlink instead.
 /// * Returns the required size of \p temporary_storage in \p storage_size
 /// if \p temporary_storage in a null pointer.
 /// * Ranges specified by \p keys_input, \p values_input, and \p values_output must have
 /// at least \p size elements.
 ///
-/// \tparam Config - [optional] configuration of the primitive, should be \p scan_by_key_config_v2.
+/// \tparam Config - [optional] Configuration of the primitive, must be `default_config` or `scan_by_key_config`.
 /// \tparam KeysInputIterator - random-access iterator type of the input range. It can be
 /// a simple pointer type.
 /// \tparam ValuesInputIterator - random-access iterator type of the input range. It can be
@@ -316,6 +324,8 @@ inline hipError_t scan_by_key_impl(void* const           temporary_storage,
 /// is \p rocprim::plus<T>, where \p T is a \p value_type of \p InputIterator.
 /// \tparam KeyCompareFunction - type of binary function used to determine keys equality. Default type
 /// is \p rocprim::equal_to<T>, where \p T is a \p value_type of \p KeysInputIterator.
+/// \tparam AccType - accumulator type used to propagate the scanned values. Default type
+/// is value type of the input iterator.
 ///
 /// \param [in] temporary_storage - pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
@@ -387,7 +397,8 @@ template<typename Config = default_config,
          typename BinaryFunction
          = ::rocprim::plus<typename std::iterator_traits<ValuesInputIterator>::value_type>,
          typename KeyCompareFunction
-         = ::rocprim::equal_to<typename std::iterator_traits<KeysInputIterator>::value_type>>
+         = ::rocprim::equal_to<typename std::iterator_traits<KeysInputIterator>::value_type>,
+         typename AccType = typename std::iterator_traits<ValuesInputIterator>::value_type>
 inline hipError_t inclusive_scan_by_key(void* const                temporary_storage,
                                         size_t&                    storage_size,
                                         const KeysInputIterator    keys_input,
@@ -401,17 +412,78 @@ inline hipError_t inclusive_scan_by_key(void* const                temporary_sto
                                         const bool        debug_synchronous = false)
 {
     using value_type = typename std::iterator_traits<ValuesInputIterator>::value_type;
-    return detail::scan_by_key_impl<false, Config>(temporary_storage,
-                                                   storage_size,
-                                                   keys_input,
-                                                   values_input,
-                                                   values_output,
-                                                   value_type(),
-                                                   size,
-                                                   scan_op,
-                                                   key_compare_op,
-                                                   stream,
-                                                   debug_synchronous);
+    return detail::scan_by_key_impl<detail::lookback_scan_determinism::default_determinism,
+                                    false,
+                                    Config,
+                                    KeysInputIterator,
+                                    ValuesInputIterator,
+                                    ValuesOutputIterator,
+                                    value_type,
+                                    BinaryFunction,
+                                    KeyCompareFunction,
+                                    AccType>(temporary_storage,
+                                             storage_size,
+                                             keys_input,
+                                             values_input,
+                                             values_output,
+                                             value_type(),
+                                             size,
+                                             scan_op,
+                                             key_compare_op,
+                                             stream,
+                                             debug_synchronous);
+}
+
+/// \brief Bitwise-reproducible parallel inclusive scan-by-key primitive for device level.
+///
+/// This function behaves the same as <tt>inclusive_scan_by_key()</tt>, except that unlike
+/// <tt>inclusive_scan_by_key()</tt>, it provides run-to-run deterministic behavior for
+/// non-associative scan operators like floating point arithmetic operations.
+/// Refer to the documentation for \link inclusive_scan_by_key() rocprim::inclusive_scan_by_key \endlink
+/// for a detailed description of this function.
+template<typename Config = default_config,
+         typename KeysInputIterator,
+         typename ValuesInputIterator,
+         typename ValuesOutputIterator,
+         typename BinaryFunction
+         = ::rocprim::plus<typename std::iterator_traits<ValuesInputIterator>::value_type>,
+         typename KeyCompareFunction
+         = ::rocprim::equal_to<typename std::iterator_traits<KeysInputIterator>::value_type>,
+         typename AccType = typename std::iterator_traits<ValuesInputIterator>::value_type>
+inline hipError_t deterministic_inclusive_scan_by_key(void* const                temporary_storage,
+                                                      size_t&                    storage_size,
+                                                      const KeysInputIterator    keys_input,
+                                                      const ValuesInputIterator  values_input,
+                                                      const ValuesOutputIterator values_output,
+                                                      const size_t               size,
+                                                      const BinaryFunction       scan_op
+                                                      = BinaryFunction(),
+                                                      const KeyCompareFunction key_compare_op
+                                                      = KeyCompareFunction(),
+                                                      const hipStream_t stream            = 0,
+                                                      const bool        debug_synchronous = false)
+{
+    using value_type = typename std::iterator_traits<ValuesInputIterator>::value_type;
+    return detail::scan_by_key_impl<detail::lookback_scan_determinism::deterministic,
+                                    false,
+                                    Config,
+                                    KeysInputIterator,
+                                    ValuesInputIterator,
+                                    ValuesOutputIterator,
+                                    value_type,
+                                    BinaryFunction,
+                                    KeyCompareFunction,
+                                    AccType>(temporary_storage,
+                                             storage_size,
+                                             keys_input,
+                                             values_input,
+                                             values_output,
+                                             value_type(),
+                                             size,
+                                             scan_op,
+                                             key_compare_op,
+                                             stream,
+                                             debug_synchronous);
 }
 
 /// \brief Parallel exclusive scan-by-key primitive for device level.
@@ -421,14 +493,19 @@ inline hipError_t inclusive_scan_by_key(void* const                temporary_sto
 ///
 /// \par Overview
 /// * Supports non-commutative scan operators. However, a scan operator should be
-/// associative. When used with non-associative functions the results may be non-deterministic
-/// and/or vary in precision.
+/// associative.
+/// * When used with non-associative functions (e.g. floating point arithmetic operations):
+///    - the results may be non-deterministic and/or vary in precision,
+///    - and bit-wise reproducibility is not guaranteed, that is, results from multiple runs
+///      using the same input values on the same device may not be bit-wise identical.
+///   If deterministic behavior is required, Use \link deterministic_exclusive_scan_by_key()
+///   rocprim::deterministic_exclusive_scan_by_key \endlink instead.
 /// * Returns the required size of \p temporary_storage in \p storage_size
 /// if \p temporary_storage in a null pointer.
 /// * Ranges specified by \p keys_input, \p values_input, and \p values_output must have
 /// at least \p size elements.
 ///
-/// \tparam Config - [optional] configuration of the primitive, should be \p scan_by_key_config_v2.
+/// \tparam Config - [optional] Configuration of the primitive, must be `default_config` or `scan_by_key_config`.
 /// \tparam KeysInputIterator - random-access iterator type of the input range. It can be
 /// a simple pointer type.
 /// \tparam ValuesInputIterator - random-access iterator type of the input range. It can be
@@ -440,6 +517,8 @@ inline hipError_t inclusive_scan_by_key(void* const                temporary_sto
 /// is \p rocprim::plus<T>, where \p T is a \p value_type of \p InputIterator.
 /// \tparam KeyCompareFunction - type of binary function used to determine keys equality. Default type
 /// is \p rocprim::equal_to<T>, where \p T is a \p value_type of \p KeysInputIterator.
+/// \tparam AccType - accumulator type used to propagate the scanned values. Default type
+/// is 'InitValueType', unless it's 'rocprim::future_value'. Then it will be the wrapped input type.
 ///
 /// \param [in] temporary_storage - pointer to a device-accessible temporary storage. When
 /// a null pointer is passed, the required allocation size (in bytes) is written to
@@ -515,7 +594,8 @@ template<typename Config = default_config,
          typename BinaryFunction
          = ::rocprim::plus<typename std::iterator_traits<ValuesInputIterator>::value_type>,
          typename KeyCompareFunction
-         = ::rocprim::equal_to<typename std::iterator_traits<KeysInputIterator>::value_type>>
+         = ::rocprim::equal_to<typename std::iterator_traits<KeysInputIterator>::value_type>,
+         typename AccType = detail::input_type_t<InitialValueType>>
 inline hipError_t exclusive_scan_by_key(void* const                temporary_storage,
                                         size_t&                    storage_size,
                                         const KeysInputIterator    keys_input,
@@ -529,17 +609,79 @@ inline hipError_t exclusive_scan_by_key(void* const                temporary_sto
                                         const hipStream_t stream            = 0,
                                         const bool        debug_synchronous = false)
 {
-    return detail::scan_by_key_impl<true, Config>(temporary_storage,
-                                                  storage_size,
-                                                  keys_input,
-                                                  values_input,
-                                                  values_output,
-                                                  initial_value,
-                                                  size,
-                                                  scan_op,
-                                                  key_compare_op,
-                                                  stream,
-                                                  debug_synchronous);
+    return detail::scan_by_key_impl<detail::lookback_scan_determinism::default_determinism,
+                                    true,
+                                    Config,
+                                    KeysInputIterator,
+                                    ValuesInputIterator,
+                                    ValuesOutputIterator,
+                                    InitialValueType,
+                                    BinaryFunction,
+                                    KeyCompareFunction,
+                                    AccType>(temporary_storage,
+                                             storage_size,
+                                             keys_input,
+                                             values_input,
+                                             values_output,
+                                             initial_value,
+                                             size,
+                                             scan_op,
+                                             key_compare_op,
+                                             stream,
+                                             debug_synchronous);
+}
+
+/// \brief Bitwise-reproducible parallel exclusive scan-by-key primitive for device level.
+///
+/// This function behaves the same as <tt>exclusive_scan_by_key()</tt>, except that unlike
+/// <tt>exclusive_scan_by_key()</tt>, it provides run-to-run deterministic behavior for
+/// non-associative scan operators like floating point arithmetic operations.
+/// Refer to the documentation for \link exclusive_scan_by_key() rocprim::exclusive_scan_by_key \endlink
+/// for a detailed description of this function.
+template<typename Config = default_config,
+         typename KeysInputIterator,
+         typename ValuesInputIterator,
+         typename ValuesOutputIterator,
+         typename InitialValueType,
+         typename BinaryFunction
+         = ::rocprim::plus<typename std::iterator_traits<ValuesInputIterator>::value_type>,
+         typename KeyCompareFunction
+         = ::rocprim::equal_to<typename std::iterator_traits<KeysInputIterator>::value_type>,
+         typename AccType = detail::input_type_t<InitialValueType>>
+inline hipError_t deterministic_exclusive_scan_by_key(void* const                temporary_storage,
+                                                      size_t&                    storage_size,
+                                                      const KeysInputIterator    keys_input,
+                                                      const ValuesInputIterator  values_input,
+                                                      const ValuesOutputIterator values_output,
+                                                      const InitialValueType     initial_value,
+                                                      const size_t               size,
+                                                      const BinaryFunction       scan_op
+                                                      = BinaryFunction(),
+                                                      const KeyCompareFunction key_compare_op
+                                                      = KeyCompareFunction(),
+                                                      const hipStream_t stream            = 0,
+                                                      const bool        debug_synchronous = false)
+{
+    return detail::scan_by_key_impl<detail::lookback_scan_determinism::deterministic,
+                                    true,
+                                    Config,
+                                    KeysInputIterator,
+                                    ValuesInputIterator,
+                                    ValuesOutputIterator,
+                                    InitialValueType,
+                                    BinaryFunction,
+                                    KeyCompareFunction,
+                                    AccType>(temporary_storage,
+                                             storage_size,
+                                             keys_input,
+                                             values_input,
+                                             values_output,
+                                             initial_value,
+                                             size,
+                                             scan_op,
+                                             key_compare_op,
+                                             stream,
+                                             debug_synchronous);
 }
 
 /// @}
