@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -27,57 +27,36 @@
 #include <type_traits>
 
 #include "../config.hpp"
+#include "../common.hpp"
 #include "../detail/various.hpp"
-#include "../detail/match_result_type.hpp"
-#include "../types/tuple.hpp"
 #include "../iterator/zip_iterator.hpp"
+#include "../types/tuple.hpp"
 
 #include "device_transform_config.hpp"
 #include "detail/device_transform.hpp"
 
-BEGIN_ROCPRIM_NAMESPACE
-
 /// \addtogroup devicemodule
 /// @{
 
+BEGIN_ROCPRIM_NAMESPACE
+
 namespace detail
 {
 
-template<
-    unsigned int BlockSize,
-    unsigned int ItemsPerThread,
-    class ResultType,
-    class InputIterator,
-    class OutputIterator,
-    class UnaryFunction
->
+template<class Config,
+         class ResultType,
+         class InputIterator,
+         class OutputIterator,
+         class UnaryFunction>
 ROCPRIM_KERNEL
-__launch_bounds__(BlockSize)
-void transform_kernel(InputIterator input,
-                      const size_t size,
-                      OutputIterator output,
-                      UnaryFunction transform_op)
+    __launch_bounds__(device_params<Config>().kernel_config.block_size) void transform_kernel(
+        InputIterator input, const size_t size, OutputIterator output, UnaryFunction transform_op)
 {
-    transform_kernel_impl<BlockSize, ItemsPerThread, ResultType>(
-        input, size, output, transform_op
-    );
+    transform_kernel_impl<device_params<Config>().kernel_config.block_size,
+                          device_params<Config>().kernel_config.items_per_thread,
+                          ResultType>(input, size, output, transform_op);
 }
 
-#define ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR(name, size, start) \
-    { \
-        auto _error = hipGetLastError(); \
-        if(_error != hipSuccess) return _error; \
-        if(debug_synchronous) \
-        { \
-            std::cout << name << "(" << size << ")"; \
-            _error = hipStreamSynchronize(stream); \
-            if(_error != hipSuccess) return _error; \
-            auto _end = std::chrono::high_resolution_clock::now(); \
-            auto _d = std::chrono::duration_cast<std::chrono::duration<double>>(_end - start); \
-            std::cout << " " << _d.count() * 1000 << " ms" << '\n'; \
-        } \
-    }
-
 } // end of detail namespace
 
 /// \brief Parallel transform primitive for device level.
@@ -88,8 +67,7 @@ void transform_kernel(InputIterator input,
 /// \par Overview
 /// * Ranges specified by \p input and \p output must have at least \p size elements.
 ///
-/// \tparam Config - [optional] configuration of the primitive. It can be \p transform_config or
-/// a custom class with the same members.
+/// \tparam Config - [optional] configuration of the primitive. It has to be \p transform_config or a class derived from it.
 /// \tparam InputIterator - random-access iterator type of the input range. Must meet the
 /// requirements of a C++ InputIterator concept. It can be a simple pointer type.
 /// \tparam OutputIterator - random-access iterator type of the output range. Must meet the
@@ -134,42 +112,43 @@ void transform_kernel(InputIterator input,
 /// // output: [6, 7, 8, 9, 10, 11, 12, 13]
 /// \endcode
 /// \endparblock
-template<
-    class Config = default_config,
-    class InputIterator,
-    class OutputIterator,
-    class UnaryFunction
->
-inline
-hipError_t transform(InputIterator input,
-                     OutputIterator output,
-                     const size_t size,
-                     UnaryFunction transform_op,
-                     const hipStream_t stream = 0,
-                     bool debug_synchronous = false)
+template<class Config = default_config,
+         class InputIterator,
+         class OutputIterator,
+         class UnaryFunction>
+inline hipError_t transform(InputIterator     input,
+                            OutputIterator    output,
+                            const size_t      size,
+                            UnaryFunction     transform_op,
+                            const hipStream_t stream            = 0,
+                            bool              debug_synchronous = false)
 {
     if( size == size_t(0) )
         return hipSuccess;
 
     using input_type = typename std::iterator_traits<InputIterator>::value_type;
-    using result_type = typename ::rocprim::detail::invoke_result<UnaryFunction, input_type>::type;
+    using result_type = typename ::rocprim::invoke_result<UnaryFunction, input_type>::type;
 
-    // Get default config if Config is default_config
-    using config = detail::default_or_custom_config<
-        Config,
-        detail::default_transform_config<ROCPRIM_TARGET_ARCH, result_type>
-    >;
+    using config = detail::wrapped_transform_config<Config, result_type>;
 
-    static constexpr unsigned int block_size = config::block_size;
-    static constexpr unsigned int items_per_thread = config::items_per_thread;
-    static constexpr auto items_per_block = block_size * items_per_thread;
+    detail::target_arch target_arch;
+    hipError_t          result = detail::host_target_arch(stream, target_arch);
+    if(result != hipSuccess)
+    {
+        return result;
+    }
+    const detail::transform_config_params params
+        = detail::dispatch_target_arch<config>(target_arch);
+
+    const unsigned int block_size       = params.kernel_config.block_size;
+    const unsigned int items_per_thread = params.kernel_config.items_per_thread;
+    const auto         items_per_block  = block_size * items_per_thread;
 
     // Start point for time measurements
-    std::chrono::high_resolution_clock::time_point start;
+    std::chrono::steady_clock::time_point start;
 
-    static constexpr auto size_limit = config::size_limit;
-    static constexpr auto number_of_blocks_limit
-        = ::rocprim::max<size_t>(size_limit / items_per_block, 1);
+    const auto size_limit             = params.kernel_config.size_limit;
+    const auto number_of_blocks_limit = ::rocprim::max<size_t>(size_limit / items_per_block, 1);
 
     auto number_of_blocks = (size + items_per_block - 1)/items_per_block;
     if(debug_synchronous)
@@ -180,7 +159,7 @@ hipError_t transform(InputIterator input,
         std::cout << "items_per_block " << items_per_block << '\n';
     }
 
-    static constexpr auto aligned_size_limit = number_of_blocks_limit * items_per_block;
+    const auto aligned_size_limit = number_of_blocks_limit * items_per_block;
 
     // Launch number_of_blocks_limit blocks while there is still at least as many blocks left as the limit
     const auto number_of_launch = (size + aligned_size_limit - 1) / aligned_size_limit;
@@ -188,15 +167,17 @@ hipError_t transform(InputIterator input,
         const auto current_size = std::min(size - offset, aligned_size_limit);
         const auto current_blocks = (current_size + items_per_block - 1) / items_per_block;
 
-        if(debug_synchronous) start = std::chrono::high_resolution_clock::now();
-        hipLaunchKernelGGL(
-            HIP_KERNEL_NAME(detail::transform_kernel<
-                block_size, items_per_thread, result_type,
-                InputIterator, OutputIterator, UnaryFunction
-            >),
-            dim3(current_blocks), dim3(block_size), 0, stream,
-            input + offset, current_size, output + offset, transform_op
-        );
+        if(debug_synchronous)
+            start = std::chrono::steady_clock::now();
+        hipLaunchKernelGGL(HIP_KERNEL_NAME(detail::transform_kernel<config, result_type>),
+                           dim3(current_blocks),
+                           dim3(block_size),
+                           0,
+                           stream,
+                           input + offset,
+                           current_size,
+                           output + offset,
+                           transform_op);
         ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR("transform_kernel", current_size, start);
     }
 
@@ -211,8 +192,7 @@ hipError_t transform(InputIterator input,
 /// \par Overview
 /// * Ranges specified by \p input1, \p input2, and \p output must have at least \p size elements.
 ///
-/// \tparam Config - [optional] configuration of the primitive. It can be \p transform_config or
-/// a custom class with the same members.
+/// \tparam Config - [optional] Configuration of the primitive, must be `default_config` or `transform_config`.
 /// \tparam InputIterator1 - random-access iterator type of the input range. Must meet the
 /// requirements of a C++ InputIterator concept. It can be a simple pointer type.
 /// \tparam InputIterator2 - random-access iterator type of the input range. Must meet the
@@ -286,11 +266,11 @@ hipError_t transform(InputIterator1 input1,
     );
 }
 
-#undef ROCPRIM_DETAIL_HIP_SYNC_AND_RETURN_ON_ERROR
+
+
+END_ROCPRIM_NAMESPACE
 
 /// @}
 // end of group devicemodule
 
-END_ROCPRIM_NAMESPACE
-
 #endif // ROCPRIM_DEVICE_DEVICE_TRANSFORM_HPP_
