@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2023 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -24,6 +24,7 @@
 #include <type_traits>
 
 #include "../config.hpp"
+#include "../detail/various.hpp"
 #include "thread.hpp"
 
 /// \addtogroup warpmodule
@@ -34,24 +35,6 @@ BEGIN_ROCPRIM_NAMESPACE
 namespace detail
 {
 
-#ifdef __HIP_CPU_RT__
-// TODO: consider adding macro checks relaying to std::bit_cast when compiled
-//       using C++20.
-template <class To, class From>
-typename std::enable_if_t<
-    sizeof(To) == sizeof(From) &&
-    std::is_trivially_copyable_v<From> &&
-    std::is_trivially_copyable_v<To>,
-    To>
-// constexpr support needs compiler magic
-bit_cast(const From& src) noexcept
-{
-    To dst;
-    std::memcpy(&dst, &src, sizeof(To));
-    return dst;
-}
-#endif
-
 template<class T, class ShuffleOp>
 ROCPRIM_DEVICE ROCPRIM_INLINE
 typename std::enable_if<std::is_trivially_copyable<T>::value && (sizeof(T) % sizeof(int) == 0), T>::type
@@ -60,11 +43,8 @@ warp_shuffle_op(const T& input, ShuffleOp&& op)
     constexpr int words_no = (sizeof(T) + sizeof(int) - 1) / sizeof(int);
 
     struct V { int words[words_no]; };
-#ifdef __HIP_CPU_RT__
-    V a = bit_cast<V>(input);
-#else
-    V a = __builtin_bit_cast(V, input);
-#endif
+
+    auto a = ::rocprim::detail::bit_cast<V>(input);
 
     ROCPRIM_UNROLL
     for(int i = 0; i < words_no; i++)
@@ -72,11 +52,7 @@ warp_shuffle_op(const T& input, ShuffleOp&& op)
         a.words[i] = op(a.words[i]);
     }
 
-#ifdef __HIP_CPU_RT__
-    return bit_cast<T>(a);
-#else
-    return __builtin_bit_cast(T, a);
-#endif
+    return ::rocprim::detail::bit_cast<T>(a);
 }
 
 template<class T, class ShuffleOp>
@@ -91,18 +67,10 @@ warp_shuffle_op(const T& input, ShuffleOp&& op)
     for(int i = 0; i < words_no; i++)
     {
         const size_t s = std::min(sizeof(int), sizeof(T) - i * sizeof(int));
-        int word;
-#ifdef __HIP_CPU_RT__
-        std::memcpy(&word, reinterpret_cast<const char*>(&input) + i * sizeof(int), s);
-#else
+        int          word;
         __builtin_memcpy(&word, reinterpret_cast<const char*>(&input) + i * sizeof(int), s);
-#endif
         word = op(word);
-#ifdef __HIP_CPU_RT__
-        std::memcpy(reinterpret_cast<char*>(&output) + i * sizeof(int), &word, s);
-#else
         __builtin_memcpy(reinterpret_cast<char*>(&output) + i * sizeof(int), &word, s);
-#endif
     }
 
     return output;
@@ -123,13 +91,8 @@ T warp_move_dpp(const T& input)
             //       __builtin_amdgcn_update_dpp, hence fail to parse the template altogether. (Except MSVC
             //       because even using /permissive- they somehow still do delayed parsing of the body of
             //       function templates, even though they pinky-swear they don't.)
-#if !defined(__HIP_CPU_RT__)
             return ::__builtin_amdgcn_mov_dpp(v, dpp_ctrl, row_mask, bank_mask, bound_ctrl);
-#else
-            return v;
-#endif
-        }
-    );
+        });
 }
 
 /// \brief Swizzle for any data type.
@@ -156,20 +119,20 @@ T warp_swizzle(const T& input)
 /// \brief Shuffle for any data type.
 ///
 /// Each thread in warp obtains \p input from <tt>src_lane</tt>-th thread
-/// in warp. If \p width is less than device_warp_size() then each subsection of the
+/// in warp. If \p width is less than arch::wavefront::min_size() then each subsection of the
 /// warp behaves as a separate entity with a starting logical lane id of 0.
 /// If \p src_lane is not in [0; \p width) range, the returned value is
 /// equal to \p input passed by the <tt>src_lane modulo width</tt> thread.
 ///
 /// Note: The optional \p width parameter must be a power of 2; results are
-/// undefined if it is not a power of 2, or it is greater than device_warp_size().
+/// undefined if it is not a power of 2, or it is greater than arch::wavefront::min_size().
 ///
 /// \param input - input to pass to other threads
 /// \param src_lane - warp if of a thread whose \p input should be returned
 /// \param width - logical warp width
 template<class T>
 ROCPRIM_DEVICE ROCPRIM_INLINE
-T warp_shuffle(const T& input, const int src_lane, const int width = device_warp_size())
+T warp_shuffle(const T& input, const int src_lane, const int width = arch::wavefront::min_size())
 {
     return detail::warp_shuffle_op(
         input,
@@ -187,14 +150,14 @@ T warp_shuffle(const T& input, const int src_lane, const int width = device_warp
 /// thread's own \p input is returned.
 ///
 /// Note: The optional \p width parameter must be a power of 2; results are
-/// undefined if it is not a power of 2, or it is greater than device_warp_size().
+/// undefined if it is not a power of 2, or it is greater than arch::wavefront::min_size().
 ///
 /// \param input - input to pass to other threads
 /// \param delta - offset for calculating source lane id
 /// \param width - logical warp width
 template<class T>
 ROCPRIM_DEVICE ROCPRIM_INLINE
-T warp_shuffle_up(const T& input, const unsigned int delta, const int width = device_warp_size())
+T warp_shuffle_up(const T& input, const unsigned int delta, const int width = arch::wavefront::min_size())
 {
     return detail::warp_shuffle_op(
         input,
@@ -212,14 +175,14 @@ T warp_shuffle_up(const T& input, const unsigned int delta, const int width = de
 /// thread's own \p input is returned.
 ///
 /// Note: The optional \p width parameter must be a power of 2; results are
-/// undefined if it is not a power of 2, or it is greater than device_warp_size().
+/// undefined if it is not a power of 2, or it is greater than arch::wavefront::min_size().
 ///
 /// \param input - input to pass to other threads
 /// \param delta - offset for calculating source lane id
 /// \param width - logical warp width
 template<class T>
 ROCPRIM_DEVICE ROCPRIM_INLINE
-T warp_shuffle_down(const T& input, const unsigned int delta, const int width = device_warp_size())
+T warp_shuffle_down(const T& input, const unsigned int delta, const int width = arch::wavefront::min_size())
 {
     return detail::warp_shuffle_op(
         input,
@@ -236,14 +199,14 @@ T warp_shuffle_down(const T& input, const unsigned int delta, const int width =
 /// thread in warp.
 ///
 /// Note: The optional \p width parameter must be a power of 2; results are
-/// undefined if it is not a power of 2, or it is greater than device_warp_size().
+/// undefined if it is not a power of 2, or it is greater than arch::wavefront::min_size().
 ///
 /// \param input - input to pass to other threads
 /// \param lane_mask - mask used for calculating source lane id
 /// \param width - logical warp width
 template<class T>
 ROCPRIM_DEVICE ROCPRIM_INLINE
-T warp_shuffle_xor(const T& input, const int lane_mask, const int width = device_warp_size())
+T warp_shuffle_xor(const T& input, const int lane_mask, const int width = arch::wavefront::min_size())
 {
     return detail::warp_shuffle_op(
         input,
@@ -254,6 +217,39 @@ T warp_shuffle_xor(const T& input, const int lane_mask, const int width = device
     );
 }
 
+namespace detail
+{
+
+/// \brief Shuffle XOR for any data type using warp_swizzle.
+///
+/// <tt>i</tt>-th thread in warp obtains \p input from <tt>i^lane_mask</tt>-th
+/// thread in warp. Makes use of of the swizzle instruction for powers of 2 till 16.
+/// Defaults to warp_shuffle_xor.
+///
+/// Note: The optional \p width parameter must be a power of 2; results are
+/// undefined if it is not a power of 2, or it is greater than arch::wavefront::min_size().
+///
+/// \param v - input to pass to other threads
+/// \param mask - mask used for calculating source lane id
+/// \param width - logical warp width
+template<class V>
+ROCPRIM_DEVICE ROCPRIM_INLINE V warp_swizzle_shuffle(V&        v,
+                                                     const int mask,
+                                                     const int width = arch::wavefront::min_size())
+{
+    switch(mask)
+    {
+        case 1: return warp_swizzle<V, 0x041F>(v);
+        case 2: return warp_swizzle<V, 0x081F>(v);
+        case 4: return warp_swizzle<V, 0x101F>(v);
+        case 8: return warp_swizzle<V, 0x201F>(v);
+        case 16: return warp_swizzle<V, 0x401F>(v);
+        default: return warp_shuffle_xor(v, mask, width);
+    }
+}
+
+} // namespace detail
+
 /// \brief Permute items across the threads in a warp.
 ///
 /// The value from this thread in the warp is permuted to the <tt>dst_lane</tt>-th
@@ -264,7 +260,7 @@ T warp_shuffle_xor(const T& input, const int lane_mask, const int width = device
 /// than the logical warp size will wrap around.
 ///
 /// Note: The optional \p width parameter must be a power of 2; results are
-/// undefined if it is not a power of 2, or it is greater than device_warp_size().
+/// undefined if it is not a power of 2, or it is greater than arch::wavefront::min_size().
 ///
 /// \param input - input to pass to other threads
 /// \param dst_lane - the destination lane to which the value from this thread is written.
@@ -272,18 +268,67 @@ T warp_shuffle_xor(const T& input, const int lane_mask, const int width = device
 template<typename T>
 ROCPRIM_DEVICE ROCPRIM_INLINE T warp_permute(const T&  input,
                                              const int dst_lane,
-                                             const int width = device_warp_size())
+                                             const int width = arch::wavefront::min_size())
 {
+    // The amdgcn intrinsic does not support virtual warp sizes, so in order to support those, manually
+    // wrap around the dst_lane within groups of log2(width) bits.
     const int self  = lane_id();
-    const int index = (dst_lane + (self & ~(width - 1))) << 2;
+    // Construct a mask of bits which make up a virtual warp. If the warp size is `width` (a power of 2),
+    // then the lower `width - 1` bits indicate the position within a virtual warp, and the remainder
+    // indicate the position of the virtual warp within the real warp.
+    const unsigned int mask = width - 1;
+    // Wrap the `dst_lane` around the virtual warp size by only considering the part within the
+    // virtual warp size, defined by the bits in the mask. The hardware warp index is given by adding the
+    // virtual warp's destination lane to the virtual warp's base offset, which is given by rounding down the
+    // current lane's position by the virtual warp size.
+    // Note that the extra right shift by 2 is required for the amdgcn intrinsic.
+    const int index = ((dst_lane & mask) + (self & ~mask)) << 2;
+    return detail::warp_shuffle_op(
+        input,
+        [=](int v) -> int
+        // __builtin_amdgcn_ds_permute maps to the `ds_permute_b32` instruction.
+        // See AMD ISA reference at https://gpuopen.com/amd-isa-documentation/.
+        { return __builtin_amdgcn_ds_permute(index, v); });
+}
+
+/// \brief Broadcast the first lane to all threads.
+///
+/// Each thread in the warp obtains \p input from the first active thread in a warp.
+/// This function always operates on all <tt>arch::wavefront::min_size()</tt> threads in the warp.
+///
+/// \remark This operation is significantly faster than \p warp_shuffle.
+///
+/// \param input - the value to broadcast
+template<typename T>
+ROCPRIM_DEVICE ROCPRIM_INLINE T warp_readfirstlane(const T& input)
+{
+    return detail::warp_shuffle_op(input,
+                                   [](int v) -> int { return __builtin_amdgcn_readfirstlane(v); });
+}
+
+/// \brief Broadcast a particular lane to all threads.
+///
+/// Each thread in the warp obtains \p input from the <tt>src_lane</tt>-th thread
+/// in the warp. \p src_lane must be the same value for all threads in the warp.
+/// This function does not distinguish between active threads and non-active
+/// threads: all threads must participate in the broadcast. This function also
+/// always operates on all <tt>arch::wavefront::min_size()</tt> threads in the warp.
+///
+/// \remark This operation is significantly faster than \p warp_shuffle.
+///
+/// \param input - the value to broadcast
+/// \param src_lane - the lane whose value to broadcast to other threads in the warp
+template<typename T>
+ROCPRIM_DEVICE ROCPRIM_INLINE T warp_readlane(const T& input, const int src_lane)
+{
     return detail::warp_shuffle_op(input,
                                    [=](int v) -> int
-                                   { return __builtin_amdgcn_ds_permute(index, v); });
+                                   { return __builtin_amdgcn_readlane(v, src_lane); });
 }
 
 END_ROCPRIM_NAMESPACE
 
-#endif // ROCPRIM_INTRINSICS_WARP_SHUFFLE_HPP_
-
 /// @}
 // end of group warpmodule
+
+#endif // ROCPRIM_INTRINSICS_WARP_SHUFFLE_HPP_
