@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -158,7 +158,7 @@ public:
     {
     }
 
-    //! \skip_doxy_start
+    #ifndef DOXYGEN_SHOULD_SKIP_THIS
     ROCPRIM_HOST_DEVICE inline
     zip_iterator& operator++()
     {
@@ -283,16 +283,17 @@ public:
         return !(*this < other);
     }
 
-    friend std::ostream& operator<<(std::ostream& os, const zip_iterator& /* iter */)
+    [[deprecated]] friend std::ostream& operator<<(std::ostream& os, const zip_iterator& /* iter */)
     {
         return os;
     }
-    //! \skip_doxy_end
+    #endif // DOXYGEN_SHOULD_SKIP_THIS
 
 private:
     IteratorTuple iterator_tuple_;
 };
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
 template<class IteratorTuple>
 ROCPRIM_HOST_DEVICE inline
 zip_iterator<IteratorTuple>
@@ -301,6 +302,7 @@ operator+(typename zip_iterator<IteratorTuple>::difference_type distance,
 {
     return iterator + distance;
 }
+#endif // DOXYGEN_SHOULD_SKIP_THIS
 
 /// make_zip_iterator creates a zip_iterator using \p iterator_tuple as
 /// the underlying tuple of iterator.
@@ -317,9 +319,9 @@ make_zip_iterator(IteratorTuple iterator_tuple)
     return zip_iterator<IteratorTuple>(iterator_tuple);
 }
 
+END_ROCPRIM_NAMESPACE
+
 /// @}
 // end of group iteratormodule
 
-END_ROCPRIM_NAMESPACE
-
 #endif // ROCPRIM_ITERATOR_ZIP_ITERATOR_HPP_
