@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -45,7 +45,9 @@ class warp_scan_shared_mem
         T threads[WarpSize];
     };
 public:
+    ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_WITH_PUSH
     using storage_type = detail::raw_storage<storage_type_>;
+    ROCPRIM_DETAIL_SUPPRESS_DEPRECATION_POP
 
     template<class BinaryFunction>
     ROCPRIM_DEVICE ROCPRIM_INLINE
@@ -104,6 +106,15 @@ public:
         to_exclusive(output, storage);
     }
 
+    template<class BinaryFunction>
+    ROCPRIM_DEVICE ROCPRIM_INLINE void exclusive_scan(
+        T input, T& output, storage_type& storage, T& reduction, BinaryFunction scan_op)
+    {
+        inclusive_scan(input, output, storage, scan_op);
+        reduction = storage.get().threads[WarpSize - 1];
+        to_exclusive(output, storage);
+    }
+
     template<class BinaryFunction>
     ROCPRIM_DEVICE ROCPRIM_INLINE
     void exclusive_scan(T input, T& output, T init, T& reduction,
@@ -158,8 +169,8 @@ public:
     }
 
 protected:
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    void to_exclusive(T inclusive_input, T& exclusive_output, storage_type& storage)
+    [[deprecated]] ROCPRIM_DEVICE ROCPRIM_INLINE void
+        to_exclusive(T inclusive_input, T& exclusive_output, storage_type& storage)
     {
         (void) inclusive_input;
         return to_exclusive(exclusive_output, storage);
