@@ -1,4 +1,4 @@
-// Copyright (c) 2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,13 +21,17 @@
 #ifndef ROCPRIM_WARP_WARP_EXCHANGE_HPP_
 #define ROCPRIM_WARP_WARP_EXCHANGE_HPP_
 
+#include <utility>
+
 #include "../config.hpp"
 #include "../detail/various.hpp"
 
+#include "../functional.hpp"
 #include "../intrinsics.hpp"
 #include "../intrinsics/warp_shuffle.hpp"
-#include "../functional.hpp"
 #include "../types.hpp"
+#include <rocprim/functional.hpp>
+#include <rocprim/intrinsics/thread.hpp>
 
 /// \addtogroup warpmodule
 /// @{
@@ -75,36 +79,173 @@ BEGIN_ROCPRIM_NAMESPACE
 template<
     class T,
     unsigned int ItemsPerThread,
-    unsigned int WarpSize = ::rocprim::device_warp_size()
+    unsigned int WarpSize = ::rocprim::arch::wavefront::min_size()
 >
 class warp_exchange
 {
     static_assert(::rocprim::detail::is_power_of_two(WarpSize),
                   "Logical warp size must be a power of two.");
-    static_assert(WarpSize <= ::rocprim::device_warp_size(),
-                  "Logical warp size cannot be larger than physical warp size.");
+    ROCPRIM_DETAIL_DEVICE_STATIC_ASSERT(
+        WarpSize <= ::rocprim::arch::wavefront::min_size(),
+        "Logical warp size cannot be larger than physical warp size.");
 
-    // Struct used for creating a raw_storage object for this primitive's temporary storage.
     struct storage_type_
     {
-        T buffer[WarpSize * ItemsPerThread];
+        uninitialized_array<T, WarpSize * ItemsPerThread> buffer;
     };
 
-public:
+    template<int NumEntries, int IdX, class U>
+    ROCPRIM_DEVICE ROCPRIM_INLINE void Foreach(const T (&input)[ItemsPerThread],
+                                               U (&output)[ItemsPerThread],
+                                               const int xor_bit_set)
+    {
+        // To prevent double work for IdX and IdX + NumEntries
+        if(NumEntries != 0 && (IdX / NumEntries) % 2 == 0)
+        {
+            const T send_val = (xor_bit_set ? input[IdX] : input[IdX + NumEntries]);
+            const T recv_val
+                = ::rocprim::detail::warp_swizzle_shuffle(send_val, NumEntries, WarpSize);
+            (xor_bit_set ? output[IdX] : output[IdX + NumEntries]) = recv_val;
+        }
+    }
 
+    template<int NumEntries, class U, int... Ids>
+    ROCPRIM_DEVICE ROCPRIM_INLINE void Foreach(const T (&input)[ItemsPerThread],
+                                               U (&output)[ItemsPerThread],
+                                               const std::integer_sequence<int, Ids...>,
+                                               const bool xor_bit_set)
+    {
+        // To create a static inner loop that executes the code with
+        // the values [0, 1, ..., ItemsPerThread-1, ItemsPerThread] as IdX
+        int ignored[] = {((Foreach<NumEntries, Ids>(input, output, xor_bit_set)), 0)...};
+        (void)ignored;
+    }
+
+    template<unsigned int MaxIter, class U, int... Iter>
+    ROCPRIM_DEVICE ROCPRIM_INLINE void TransposeImpl(const T (&input)[ItemsPerThread],
+                                                     U (&output)[ItemsPerThread],
+                                                     const unsigned int lane_id,
+                                                     const std::integer_sequence<int, Iter...>)
+    {
+        // To create a static outer loop that executes the code with
+        // the values [ItemsPerThread/2, ItemsPerThread/4, ..., 1, 0] as NumEntries
+        int ignored[]
+            = {(Foreach<(1 << (MaxIter - Iter))>(input,
+                                                 output,
+                                                 std::make_integer_sequence<int, ItemsPerThread>{},
+                                                 lane_id & (1 << (MaxIter - Iter))),
+                0)...};
+        (void)ignored;
+    }
+
+    template<class U>
+    ROCPRIM_DEVICE ROCPRIM_INLINE void Transpose(const T (&input)[ItemsPerThread],
+                                                 U (&output)[ItemsPerThread],
+                                                 const unsigned int lane_id)
+    {
+        constexpr unsigned int n_iter = rocprim::Log2<ItemsPerThread>::VALUE;
+        TransposeImpl<n_iter - 1>(input,
+                                  output,
+                                  lane_id,
+                                  std::make_integer_sequence<int, n_iter>{});
+    }
+
+    template<class U>
+    ROCPRIM_DEVICE ROCPRIM_INLINE void
+        blocked_striped_shuffle_efficient_impl(const T (&input)[ItemsPerThread],
+                                               U (&output)[ItemsPerThread])
+    {
+        static constexpr bool IS_ARCH_WARP = WarpSize == ::rocprim::arch::wavefront::min_size();
+        const unsigned int    flat_lane_id = ::rocprim::detail::logical_lane_id<WarpSize>();
+        const unsigned int    lane_id = IS_ARCH_WARP ? flat_lane_id : (flat_lane_id % WarpSize);
+        T                     temp[ItemsPerThread];
+        ROCPRIM_UNROLL
+        for(unsigned int i = 0; i < ItemsPerThread; i++)
+        {
+            temp[i] = input[i];
+        }
+        Transpose(temp, temp, lane_id);
+        ROCPRIM_UNROLL
+        for(unsigned int i = 0; i < ItemsPerThread; i++)
+        {
+            output[i] = temp[i];
+        }
+    }
+
+    template<class U>
+    ROCPRIM_DEVICE ROCPRIM_INLINE void
+        blocked_to_striped_shuffle_impl(const T (&input)[ItemsPerThread],
+                                        U (&output)[ItemsPerThread])
+    {
+        const unsigned int flat_id = ::rocprim::detail::logical_lane_id<WarpSize>();
+        U                  work_array[ItemsPerThread];
+
+        ROCPRIM_UNROLL
+        for(unsigned int dst_idx = 0; dst_idx < ItemsPerThread; dst_idx++)
+        {
+            ROCPRIM_UNROLL
+            for(unsigned int src_idx = 0; src_idx < ItemsPerThread; src_idx++)
+            {
+                const auto value = ::rocprim::warp_shuffle(
+                    input[src_idx],
+                    flat_id / ItemsPerThread + dst_idx * (WarpSize / ItemsPerThread),
+                    WarpSize);
+                if(src_idx == flat_id % ItemsPerThread)
+                {
+                    work_array[dst_idx] = value;
+                }
+            }
+        }
+
+        ROCPRIM_UNROLL
+        for(unsigned int i = 0; i < ItemsPerThread; i++)
+        {
+            output[i] = work_array[i];
+        }
+    }
+
+    template<class U>
+    ROCPRIM_DEVICE ROCPRIM_INLINE void
+        striped_to_blocked_shuffle_impl(const T (&input)[ItemsPerThread],
+                                        U (&output)[ItemsPerThread])
+    {
+        const unsigned int flat_id = ::rocprim::detail::logical_lane_id<WarpSize>();
+        U                  work_array[ItemsPerThread];
+
+        ROCPRIM_UNROLL
+        for(unsigned int dst_idx = 0; dst_idx < ItemsPerThread; dst_idx++)
+        {
+            ROCPRIM_UNROLL
+            for(unsigned int src_idx = 0; src_idx < ItemsPerThread; src_idx++)
+            {
+                const auto value
+                    = ::rocprim::warp_shuffle(input[src_idx],
+                                              (ItemsPerThread * flat_id + dst_idx) % WarpSize,
+                                              WarpSize);
+                if(flat_id / (WarpSize / ItemsPerThread) == src_idx)
+                {
+                    work_array[dst_idx] = value;
+                }
+            }
+        }
+
+        ROCPRIM_UNROLL
+        for(unsigned int i = 0; i < ItemsPerThread; i++)
+        {
+            output[i] = work_array[i];
+        }
+    }
+
+public:
     /// \brief Struct used to allocate a temporary memory that is required for thread
     /// communication during operations provided by the related parallel primitive.
     ///
-    /// Depending on the implemention the operations exposed by parallel primitive may
+    /// Depending on the implementation the operations exposed by parallel primitive may
     /// require a temporary storage for thread communication. The storage should be allocated
     /// using keywords <tt>__shared__</tt>. It can be aliased to
     /// an externally allocated memory, or be a part of a union type with other storage types
     /// to increase shared memory reusability.
-    #ifndef DOXYGEN_SHOULD_SKIP_THIS // hides storage_type implementation for Doxygen
-    using storage_type = detail::raw_storage<storage_type_>;
-    #else
     using storage_type = storage_type_; // only for Doxygen
-    #endif
 
     /// \brief Transposes a blocked arrangement of items to a striped arrangement
     /// across the warp, using temporary storage.
@@ -147,24 +288,24 @@ public:
                             storage_type& storage)
     {
         const unsigned int flat_id = ::rocprim::detail::logical_lane_id<WarpSize>();
-        storage_type_& storage_ = storage.get();
-
         ROCPRIM_UNROLL
         for(unsigned int i = 0; i < ItemsPerThread; i++)
         {
-            storage_.buffer[flat_id * ItemsPerThread + i] = input[i];
+            storage.buffer.emplace(flat_id * ItemsPerThread + i, input[i]);
         }
         ::rocprim::wave_barrier();
+        const auto& storage_buffer = storage.buffer.get_unsafe_array();
 
         ROCPRIM_UNROLL
         for(unsigned int i = 0; i < ItemsPerThread; i++)
         {
-            output[i] = storage_.buffer[i * WarpSize + flat_id];
+            output[i] = storage_buffer[i * WarpSize + flat_id];
         }
     }
 
     /// \brief Transposes a blocked arrangement of items to a striped arrangement
     /// across the warp, using warp shuffle operations.
+    /// Uses an optimized implementation for when WarpSize is equal to ItemsPerThread.
     /// Caution: this API is experimental. Performance might not be consistent.
     /// ItemsPerThread must be a divisor of WarpSize.
     ///
@@ -193,36 +334,19 @@ public:
     /// }
     /// \endcode
     template<class U>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    void blocked_to_striped_shuffle(const T (&input)[ItemsPerThread],
-                                    U (&output)[ItemsPerThread])
+    ROCPRIM_DEVICE ROCPRIM_INLINE void blocked_to_striped_shuffle(const T (&input)[ItemsPerThread],
+                                                                  U (&output)[ItemsPerThread])
     {
-        static_assert(WarpSize % ItemsPerThread == 0,
-                      "ItemsPerThread must be a divisor of WarpSize to use blocked_to_striped_shuffle");
-        const unsigned int flat_id = ::rocprim::detail::logical_lane_id<WarpSize>();
-        U work_array[ItemsPerThread];
-
-        ROCPRIM_UNROLL
-        for(unsigned int dst_idx = 0; dst_idx < ItemsPerThread; dst_idx++)
+        static_assert(
+            WarpSize % ItemsPerThread == 0,
+            "ItemsPerThread must be a divisor of WarpSize to use blocked_to_striped_shuffle");
+        if(WarpSize == ItemsPerThread)
         {
-            ROCPRIM_UNROLL
-            for(unsigned int src_idx = 0; src_idx < ItemsPerThread; src_idx++)
-            {
-                const auto value = ::rocprim::warp_shuffle(
-                    input[src_idx],
-                    flat_id / ItemsPerThread + dst_idx * (WarpSize / ItemsPerThread)
-                );
-                if(src_idx == flat_id % ItemsPerThread)
-                {
-                    work_array[dst_idx] = value;
-                }
-            }
+            blocked_striped_shuffle_efficient_impl(input, output);
         }
-
-        ROCPRIM_UNROLL
-        for(unsigned int i = 0; i < ItemsPerThread; i++)
+        else
         {
-            output[i] = work_array[i];
+            blocked_to_striped_shuffle_impl(input, output);
         }
     }
 
@@ -267,24 +391,25 @@ public:
                             storage_type& storage)
     {
         const unsigned int flat_id = ::rocprim::detail::logical_lane_id<WarpSize>();
-        storage_type_& storage_ = storage.get();
 
         ROCPRIM_UNROLL
         for(unsigned int i = 0; i < ItemsPerThread; i++)
         {
-            storage_.buffer[i * WarpSize + flat_id] = input[i];
+            storage.buffer.emplace(i * WarpSize + flat_id, input[i]);
         }
         ::rocprim::wave_barrier();
+        const auto& storage_buffer = storage.buffer.get_unsafe_array();
 
         ROCPRIM_UNROLL
         for(unsigned int i = 0; i < ItemsPerThread; i++)
         {
-            output[i] = storage_.buffer[flat_id * ItemsPerThread + i];
+            output[i] = storage_buffer[flat_id * ItemsPerThread + i];
         }
     }
 
     /// \brief Transposes a striped arrangement of items to a blocked arrangement
     /// across the warp, using warp shuffle operations.
+    /// Uses an optimized implementation for when WarpSize is equal to ItemsPerThread.
     /// Caution: this API is experimental. Performance might not be consistent.
     /// ItemsPerThread must be a divisor of WarpSize.
     ///
@@ -313,36 +438,20 @@ public:
     /// }
     /// \endcode
     template<class U>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    void striped_to_blocked_shuffle(const T (&input)[ItemsPerThread],
-                                    U (&output)[ItemsPerThread])
+    ROCPRIM_DEVICE ROCPRIM_INLINE void striped_to_blocked_shuffle(const T (&input)[ItemsPerThread],
+                                                                  U (&output)[ItemsPerThread])
     {
-        static_assert(WarpSize % ItemsPerThread == 0,
-                      "ItemsPerThread must be a divisor of WarpSize to use striped_to_blocked_shuffle");
-        const unsigned int flat_id = ::rocprim::detail::logical_lane_id<WarpSize>();
-        U work_array[ItemsPerThread];
+        static_assert(
+            WarpSize % ItemsPerThread == 0,
+            "ItemsPerThread must be a divisor of WarpSize to use striped_to_blocked_shuffle");
 
-        ROCPRIM_UNROLL
-        for(unsigned int dst_idx = 0; dst_idx < ItemsPerThread; dst_idx++)
+        if(WarpSize == ItemsPerThread)
         {
-            ROCPRIM_UNROLL
-            for(unsigned int src_idx = 0; src_idx < ItemsPerThread; src_idx++)
-            {
-                const auto value = ::rocprim::warp_shuffle(
-                    input[src_idx],
-                    (ItemsPerThread * flat_id + dst_idx) % WarpSize
-                );
-                if(flat_id / (WarpSize / ItemsPerThread) == src_idx)
-                {
-                    work_array[dst_idx] = value;
-                }
-            }
+            blocked_striped_shuffle_efficient_impl(input, output);
         }
-
-        ROCPRIM_UNROLL
-        for(unsigned int i = 0; i < ItemsPerThread; i++)
+        else
         {
-            output[i] = work_array[i];
+            striped_to_blocked_shuffle_impl(input, output);
         }
     }
 
@@ -394,20 +503,20 @@ public:
             storage_type& storage)
     {
         const unsigned int flat_id = ::rocprim::detail::logical_lane_id<WarpSize>();
-        storage_type_& storage_ = storage.get();
 
         ROCPRIM_UNROLL
         for (unsigned int i = 0; i < ItemsPerThread; i++)
         {
-            storage_.buffer[ranks[i]] = input[i];
+            storage.buffer.emplace(ranks[i], input[i]);
         }
         ::rocprim::wave_barrier();
+        const auto& storage_buffer = storage.buffer.get_unsafe_array();
 
         ROCPRIM_UNROLL
         for (unsigned int i = 0; i < ItemsPerThread; i++)
         {
             unsigned int item_offset = (i * WarpSize) + flat_id;
-            output[i] = storage_.buffer[item_offset];
+            output[i]                = storage_buffer[item_offset];
         }
     }
 };
