@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2021 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -61,12 +61,12 @@ struct select_warp_scan_impl
 ///
 /// \tparam T - the input/output type.
 /// \tparam WarpSize - the size of logical warp size, which can be equal to or less than
-/// the size of hardware warp (see rocprim::device_warp_size()). Scan operations are performed
+/// the size of hardware warp (see rocprim::arch::wavefront::min_size()). Scan operations are performed
 /// separately within groups determined by WarpSize.
 ///
 /// \par Overview
 /// * \p WarpSize must be equal to or less than the size of hardware warp (see
-/// rocprim::device_warp_size()). If it is less, scan is performed separately within groups
+/// rocprim::arch::wavefront::min_size()). If it is less, scan is performed separately within groups
 /// determined by WarpSize. \n
 /// For example, if \p WarpSize is 4, hardware warp is 64, scan will be performed in logical
 /// warps grouped like this: `{ {0, 1, 2, 3}, {4, 5, 6, 7 }, ..., {60, 61, 62, 63} }`
@@ -106,7 +106,7 @@ struct select_warp_scan_impl
 /// \endparblock
 template<
     class T,
-    unsigned int WarpSize = device_warp_size()
+    unsigned int WarpSize = arch::wavefront::min_size()
 >
 class warp_scan
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
@@ -178,12 +178,11 @@ public:
     /// <tt>{33, -34, -34, -36, ..., -64}</tt> etc.
     /// \endparblock
     template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto inclusive_scan(T input,
-                        T& output,
-                        storage_type& storage,
-                        BinaryFunction scan_op = BinaryFunction())
-        -> typename std::enable_if<(FunctionWarpSize <= __AMDGCN_WAVEFRONT_SIZE), void>::type
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto inclusive_scan(T              input,
+                                                      T&             output,
+                                                      storage_type&  storage,
+                                                      BinaryFunction scan_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::min_size()), void>::type
     {
         base_type::inclusive_scan(input, output, storage, scan_op);
     }
@@ -191,12 +190,9 @@ public:
     /// \brief Performs inclusive scan across threads in a logical warp.
     /// Invalid Warp Size
     template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto inclusive_scan(T ,
-                        T& ,
-                        storage_type& ,
-                        BinaryFunction scan_op = BinaryFunction())
-        -> typename std::enable_if<(FunctionWarpSize > __AMDGCN_WAVEFRONT_SIZE), void>::type
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto
+        inclusive_scan(T, T&, storage_type&, BinaryFunction scan_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize > arch::wavefront::min_size()), void>::type
     {
         (void) scan_op;
         ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp size . Aborting warp sort.");
@@ -253,13 +249,12 @@ public:
     /// The \p reduction will be equal \p 64.
     /// \endparblock
     template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto inclusive_scan(T input,
-                        T& output,
-                        T& reduction,
-                        storage_type& storage,
-                        BinaryFunction scan_op = BinaryFunction())
-        -> typename std::enable_if<(FunctionWarpSize <= __AMDGCN_WAVEFRONT_SIZE), void>::type
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto inclusive_scan(T              input,
+                                                      T&             output,
+                                                      T&             reduction,
+                                                      storage_type&  storage,
+                                                      BinaryFunction scan_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::min_size()), void>::type
     {
         base_type::inclusive_scan(input, output, reduction, storage, scan_op);
     }
@@ -267,13 +262,9 @@ public:
     /// \brief Performs inclusive scan and reduction across threads in a logical warp.
     /// Invalid Warp Size
     template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto inclusive_scan(T ,
-                        T& ,
-                        T& ,
-                        storage_type& ,
-                        BinaryFunction scan_op = BinaryFunction())
-        -> typename std::enable_if<(FunctionWarpSize > __AMDGCN_WAVEFRONT_SIZE), void>::type
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto
+        inclusive_scan(T, T&, T&, storage_type&, BinaryFunction scan_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize > arch::wavefront::min_size()), void>::type
     {
         (void) scan_op;
         ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp size . Aborting warp sort.");
@@ -333,13 +324,12 @@ public:
     /// <tt>{100, 33, -34, -34, -36, ..., -62}</tt> etc.
     /// \endparblock
     template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto exclusive_scan(T input,
-                        T& output,
-                        T init,
-                        storage_type& storage,
-                        BinaryFunction scan_op = BinaryFunction())
-        -> typename std::enable_if<(FunctionWarpSize <= __AMDGCN_WAVEFRONT_SIZE), void>::type
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto exclusive_scan(T              input,
+                                                      T&             output,
+                                                      T              init,
+                                                      storage_type&  storage,
+                                                      BinaryFunction scan_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::min_size()), void>::type
     {
         base_type::exclusive_scan(input, output, init, storage, scan_op);
     }
@@ -347,13 +337,9 @@ public:
     /// \brief Performs exclusive scan across threads in a logical warp.
     /// Invalid Warp Size
     template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto exclusive_scan(T ,
-                        T& ,
-                        T ,
-                        storage_type& ,
-                        BinaryFunction scan_op = BinaryFunction())
-        -> typename std::enable_if<(FunctionWarpSize > __AMDGCN_WAVEFRONT_SIZE), void>::type
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto
+        exclusive_scan(T, T&, T, storage_type&, BinaryFunction scan_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize > arch::wavefront::min_size()), void>::type
     {
         (void) scan_op;
         ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp size . Aborting warp sort.");
@@ -414,14 +400,13 @@ public:
     /// <tt>{10, 11, 12, 13, ..., 73}</tt>. The \p reduction will be 64.
     /// \endparblock
     template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto exclusive_scan(T input,
-                        T& output,
-                        T init,
-                        T& reduction,
-                        storage_type& storage,
-                        BinaryFunction scan_op = BinaryFunction())
-        -> typename std::enable_if<(FunctionWarpSize <= __AMDGCN_WAVEFRONT_SIZE), void>::type
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto exclusive_scan(T              input,
+                                                      T&             output,
+                                                      T              init,
+                                                      T&             reduction,
+                                                      storage_type&  storage,
+                                                      BinaryFunction scan_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::min_size()), void>::type
     {
         base_type::exclusive_scan(input, output, init, reduction, storage, scan_op);
     }
@@ -429,20 +414,89 @@ public:
     /// \brief Performs exclusive scan and reduction across threads in a logical warp.
     /// Invalid Warp Size
     template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto exclusive_scan(T ,
-                        T& ,
-                        T ,
-                        T& ,
-                        storage_type& ,
-                        BinaryFunction scan_op = BinaryFunction())
-        -> typename std::enable_if<(FunctionWarpSize > __AMDGCN_WAVEFRONT_SIZE), void>::type
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto
+        exclusive_scan(T, T&, T, T&, storage_type&, BinaryFunction scan_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize > arch::wavefront::min_size()), void>::type
     {
         (void) scan_op;
         ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp size . Aborting warp sort.");
         return;
     }
 
+    /// \brief Performs exclusive scan without an initial value across threads in a logical warp
+    /// \tparam BinaryFunction binary function used for scan
+    /// \param input Thread input value
+    /// \param[out] output Reference to thread output value. Each threads value for the scan will
+    /// be written to it. May be aliased with `input`. The value written is unspecified for the first
+    /// thread of each logical warp.
+    /// \param [in] storage Reference to a temporary storage object of type storage_type.
+    /// \param scan_op The function object used to combine elements used for the scan
+    template<class BinaryFunction = ::rocprim::plus<>, unsigned int FunctionWarpSize = WarpSize>
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto exclusive_scan(T              input,
+                                                      T&             output,
+                                                      storage_type&  storage,
+                                                      BinaryFunction scan_op = BinaryFunction())
+#ifndef DOXYGEN_DOCUMENTATION_BUILD
+        -> std::enable_if_t<FunctionWarpSize <= arch::wavefront::min_size()>
+#else
+        -> void
+#endif
+    {
+        base_type::exclusive_scan(input, output, storage, scan_op);
+    }
+
+    /// \cond
+    template<class BinaryFunction = ::rocprim::plus<>, unsigned int FunctionWarpSize = WarpSize>
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto exclusive_scan(T /*input*/,
+                                                      T& /*output*/,
+                                                      storage_type& /*storage*/,
+                                                      BinaryFunction /*scan_op*/ = BinaryFunction())
+        -> std::enable_if_t<(FunctionWarpSize > arch::wavefront::min_size())>
+    {
+        ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp size."
+                                 " Aborting warp scan.");
+    }
+    /// \endcond
+
+    /// \brief Performs exclusive scan and reduction without an initial value across threads in
+    /// a logical warp
+    /// \tparam BinaryFunction binary function used for scan
+    /// \param input Thread input value
+    /// \param[out] output Reference to thread output value. Each threads value for the scan will
+    /// be written to it. May be aliased with `input`. The value written is unspecified for the first
+    /// thread of each logical warp.
+    /// \param[out] reduction Result of reducing of all `input` values in the logical warp.
+    /// \param [in] storage Reference to a temporary storage object of type storage_type.
+    /// \param scan_op The function object used to combine elements used for the scan
+    template<class BinaryFunction = ::rocprim::plus<>, unsigned int FunctionWarpSize = WarpSize>
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto exclusive_scan(T              input,
+                                                      T&             output,
+                                                      storage_type&  storage,
+                                                      T&             reduction,
+                                                      BinaryFunction scan_op = BinaryFunction())
+#ifndef DOXYGEN_DOCUMENTATION_BUILD
+        -> std::enable_if_t<FunctionWarpSize <= arch::wavefront::min_size()>
+#else
+        -> void
+#endif
+    {
+        base_type::exclusive_scan(input, output, storage, reduction, scan_op);
+    }
+
+    /// \cond
+    template<class BinaryFunction = ::rocprim::plus<>, unsigned int FunctionWarpSize = WarpSize>
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto exclusive_scan(T /*input*/,
+                                                      T& /*output*/,
+                                                      storage_type& /*storage*/,
+                                                      T& /*reduction*/,
+                                                      BinaryFunction /*scan_op*/ = BinaryFunction())
+        -> std::enable_if_t<(FunctionWarpSize > arch::wavefront::min_size())>
+    {
+        ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp size."
+                                 " Aborting warp scan.");
+    }
+    /// \endcond
+
     /// \brief Performs inclusive and exclusive scan operations across threads
     /// in a logical warp.
     ///
@@ -502,14 +556,13 @@ public:
     /// <tt>{100, 33, -34, -34, -36, ..., -62}</tt> etc.
     /// \endparblock
     template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto scan(T input,
-              T& inclusive_output,
-              T& exclusive_output,
-              T init,
-              storage_type& storage,
-              BinaryFunction scan_op = BinaryFunction())
-        -> typename std::enable_if<(FunctionWarpSize <= __AMDGCN_WAVEFRONT_SIZE), void>::type
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto scan(T              input,
+                                            T&             inclusive_output,
+                                            T&             exclusive_output,
+                                            T              init,
+                                            storage_type&  storage,
+                                            BinaryFunction scan_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::min_size()), void>::type
     {
         base_type::scan(input, inclusive_output, exclusive_output, init, storage, scan_op);
     }
@@ -517,14 +570,9 @@ public:
     /// \brief Performs inclusive and exclusive scan operations across threads
     /// Invalid Warp Size
     template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto scan(T ,
-              T& ,
-              T& ,
-              T ,
-              storage_type& ,
-              BinaryFunction scan_op = BinaryFunction())
-        -> typename std::enable_if<(FunctionWarpSize > __AMDGCN_WAVEFRONT_SIZE), void>::type
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto
+        scan(T, T&, T&, T, storage_type&, BinaryFunction scan_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize > arch::wavefront::min_size()), void>::type
     {
         (void) scan_op;
         ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp size . Aborting warp sort.");
@@ -590,15 +638,14 @@ public:
     /// be <tt>{10, 11, 12, 13, ..., 73}</tt>. The \p reduction will be 64.
     /// \endparblock
     template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto scan(T input,
-              T& inclusive_output,
-              T& exclusive_output,
-              T init,
-              T& reduction,
-              storage_type& storage,
-              BinaryFunction scan_op = BinaryFunction())
-        -> typename std::enable_if<(FunctionWarpSize <= __AMDGCN_WAVEFRONT_SIZE), void>::type
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto scan(T              input,
+                                            T&             inclusive_output,
+                                            T&             exclusive_output,
+                                            T              init,
+                                            T&             reduction,
+                                            storage_type&  storage,
+                                            BinaryFunction scan_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::min_size()), void>::type
     {
         base_type::scan(
             input, inclusive_output, exclusive_output, init, reduction,
@@ -609,15 +656,9 @@ public:
     /// \brief Performs inclusive and exclusive scan operations across threads
     /// Invalid Warp Size
     template<class BinaryFunction = ::rocprim::plus<T>, unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto scan(T ,
-              T& ,
-              T& ,
-              T ,
-              T& ,
-              storage_type& ,
-              BinaryFunction scan_op = BinaryFunction())
-        -> typename std::enable_if<(FunctionWarpSize > __AMDGCN_WAVEFRONT_SIZE), void>::type
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto
+        scan(T, T&, T&, T, T&, storage_type&, BinaryFunction scan_op = BinaryFunction()) ->
+        typename std::enable_if<(FunctionWarpSize > arch::wavefront::min_size()), void>::type
     {
         (void) scan_op;
         ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp size . Aborting warp sort.");
@@ -634,11 +675,9 @@ public:
     /// Synchronization barrier should be placed before \p storage is reused
     /// or repurposed: \p __syncthreads() or \p rocprim::syncthreads().
     template<unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto broadcast(T input,
-                   const unsigned int src_lane,
-                   storage_type& storage)
-        -> typename std::enable_if<(FunctionWarpSize <= __AMDGCN_WAVEFRONT_SIZE), T>::type
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto
+        broadcast(T input, const unsigned int src_lane, storage_type& storage) ->
+        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::min_size()), T>::type
     {
         return base_type::broadcast(input, src_lane, storage);
     }
@@ -646,11 +685,8 @@ public:
     /// \brief Broadcasts value from one thread to all threads in logical warp.
     /// Invalid Warp Size
     template<unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto broadcast(T ,
-                   const unsigned int ,
-                   storage_type& )
-        -> typename std::enable_if<(FunctionWarpSize > __AMDGCN_WAVEFRONT_SIZE), T>::type
+    ROCPRIM_DEVICE ROCPRIM_INLINE auto broadcast(T, const unsigned int, storage_type&) ->
+        typename std::enable_if<(FunctionWarpSize > arch::wavefront::min_size()), T>::type
     {
         ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp size. Aborting warp sort.");
         return T();
@@ -658,19 +694,18 @@ public:
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 protected:
-
+    // These undocumented functions are used by hipCUB prior to version 3.1
     template<unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto to_exclusive(T inclusive_input, T& exclusive_output, storage_type& storage)
-        -> typename std::enable_if<(FunctionWarpSize <= __AMDGCN_WAVEFRONT_SIZE), void>::type
+    [[deprecated]] ROCPRIM_DEVICE ROCPRIM_INLINE auto
+        to_exclusive(T inclusive_input, T& exclusive_output, storage_type& storage) ->
+        typename std::enable_if<(FunctionWarpSize <= arch::wavefront::min_size()), void>::type
     {
         return base_type::to_exclusive(inclusive_input, exclusive_output, storage);
     }
 
     template<unsigned int FunctionWarpSize = WarpSize>
-    ROCPRIM_DEVICE ROCPRIM_INLINE
-    auto to_exclusive(T , T& , storage_type&)
-        -> typename std::enable_if<(FunctionWarpSize > __AMDGCN_WAVEFRONT_SIZE), void>::type
+    [[deprecated]] ROCPRIM_DEVICE ROCPRIM_INLINE auto to_exclusive(T, T&, storage_type&) ->
+        typename std::enable_if<(FunctionWarpSize > arch::wavefront::min_size()), void>::type
     {
         ROCPRIM_PRINT_ERROR_ONCE("Specified warp size exceeds current hardware supported warp size. Aborting warp sort.");
         return;
