@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2020-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2020-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -24,18 +24,17 @@
 #define ROCPRIM_COMMON_TEST_HEADER
 
 #include <algorithm>
+#include <cmath>
+#include <cstddef>
+#include <cstring>
 #include <functional>
 #include <iostream>
-#include <type_traits>
-#include <tuple>
-#include <vector>
-#include <utility>
-#include <random>
-#include <cmath>
-
-#ifdef WIN32
 #include <numeric>
-#endif
+#include <random>
+#include <tuple>
+#include <type_traits>
+#include <utility>
+#include <vector>
 
 // Google Test
 #include <gtest/gtest.h>
@@ -43,9 +42,6 @@
 // HIP API
 #include <hip/hip_runtime.h>
 #include <hip/hip_vector_types.h>
-#ifndef __HIP_CPU_RT__
-#include <hip/hip_ext.h>
-#endif
 
 // GoogleTest-compatible HIP_CHECK macro. FAIL is called to log the Google Test trace.
 // The lambda is invoked immediately as assertions that generate a fatal failure can
@@ -63,6 +59,28 @@
         }
 #endif
 
+#define HIP_CHECK_MEMORY(condition)                                                         \
+    {                                                                                       \
+        hipError_t error = condition;                                                       \
+        if(error == hipErrorOutOfMemory)                                                    \
+        {                                                                                   \
+            std::cout << "Out of memory. Skipping size = " << size << std::endl;            \
+            break;                                                                          \
+        }                                                                                   \
+        if(error != hipSuccess)                                                             \
+        {                                                                                   \
+            std::cout << "HIP error: " << hipGetErrorString(error) << " line: " << __LINE__ \
+                      << std::endl;                                                         \
+            exit(error);                                                                    \
+        }                                                                                   \
+    }
+
+#if(defined(__GNUC__) || defined(__clang__)) && (defined(__GLIBCXX__) || defined(_LIBCPP_VERSION))
+    #define ROCPRIM_HAS_INT128_SUPPORT 1
+#else
+    #define ROCPRIM_HAS_INT128_SUPPORT 0
+#endif
+
 #define INSTANTIATE_TYPED_TEST_EXPANDED_1(line, test_suite_name, ...)         \
     namespace Id##line                                                        \
     {                                                                         \
@@ -79,6 +97,9 @@
 #define INSTANTIATE_TYPED_TEST(test_suite_name, ...) \
     INSTANTIATE_TYPED_TEST_EXPANDED(__LINE__, test_suite_name, __VA_ARGS__)
 
+// C++17 or newer
+#define CPP17 __cplusplus >= 201703L
+
 #include <cstdlib>
 #include <string>
 #include <cctype>
@@ -86,45 +107,64 @@
 namespace test_common_utils
 {
 
-inline int obtain_device_from_ctest()
+inline char* __get_env(const char* name)
+{
+    char* env;
+#ifdef _MSC_VER
+    size_t  len;
+    errno_t err = _dupenv_s(&env, &len, name);
+    if(err)
+    {
+        return nullptr;
+    }
+#else
+    env = std::getenv(name);
+#endif
+    return env;
+}
+
+inline void clean_env(char* name)
 {
 #ifdef _MSC_VER
-#pragma warning( push )
-#pragma warning( disable : 4996 ) // This function or variable may be unsafe. Consider using _dupenv_s instead.
-#endif
-    static const std::string rg0 = "CTEST_RESOURCE_GROUP_0";
-    if (std::getenv(rg0.c_str()) != nullptr)
+    if(name != nullptr)
     {
-        std::string amdgpu_target = std::getenv(rg0.c_str());
+        free(name);
+    }
+#endif
+    (void)name;
+}
+
+inline int obtain_device_from_ctest()
+{
+    static const std::string rg0    = "CTEST_RESOURCE_GROUP_0";
+    char*                    env    = __get_env(rg0.c_str());
+    int                      device = 0;
+    if(env != nullptr)
+    {
+        std::string amdgpu_target(env);
         std::transform(
             amdgpu_target.cbegin(),
             amdgpu_target.cend(),
             amdgpu_target.begin(),
             // Feeding std::toupper plainly results in implicitly truncating conversions between int and char triggering warnings.
-            [](unsigned char c){ return static_cast<char>(std::toupper(c)); }
-        );
-        std::string reqs = std::getenv((rg0 + "_" + amdgpu_target).c_str());
-        return std::atoi(reqs.substr(reqs.find(':') + 1, reqs.find(',') - (reqs.find(':') + 1)).c_str());
+            [](unsigned char c) { return static_cast<char>(std::toupper(c)); });
+        char*       env_reqs = __get_env((rg0 + "_" + amdgpu_target).c_str());
+        std::string reqs(env_reqs);
+        device = std::atoi(
+            reqs.substr(reqs.find(':') + 1, reqs.find(',') - (reqs.find(':') + 1)).c_str());
+        clean_env(env_reqs);
     }
-    else
-        return 0;
-#ifdef _MSC_VER
-#pragma warning( pop )
-#endif
+    clean_env(env);
+    return device;
 }
 
 inline bool use_hmm()
 {
-    if (getenv("ROCPRIM_USE_HMM") == nullptr)
-    {
-        return false;
-    }
 
-    if (strcmp(getenv("ROCPRIM_USE_HMM"), "1") == 0)
-    {
-        return true;
-    }
-    return false;
+    char*      env = __get_env("ROCPRIM_USE_HMM");
+    const bool hmm = (env != nullptr) && (strcmp(env, "1") == 0);
+    clean_env(env);
+    return hmm;
 }
 
 // Helper for HMM allocations: HMM is requested through ROCPRIM_USE_HMM=1 environment variable
