@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -66,7 +66,8 @@ void flag_heads_kernel(Type* device_input, long long* device_heads)
     Type input[ItemsPerThread];
     rocprim::block_load_direct_blocked(lid, device_input + block_offset, input);
 
-    rocprim::block_adjacent_difference<Type, BlockSize> bdiscontinuity;
+    rocprim::block_adjacent_difference<Type, BlockSize>             adjacent_difference;
+    __shared__ typename decltype(adjacent_difference)::storage_type storage;
 
     FlagType head_flags[ItemsPerThread];
 
@@ -75,11 +76,15 @@ void flag_heads_kernel(Type* device_input, long long* device_heads)
     if(blockIdx.x % 2 == 1)
     {
         const Type tile_predecessor_item = device_input[block_offset - 1];
-        bdiscontinuity.flag_heads(head_flags, tile_predecessor_item, input, FlagOpType());
+        adjacent_difference.flag_heads(head_flags,
+                                       tile_predecessor_item,
+                                       input,
+                                       FlagOpType(),
+                                       storage);
     }
     else
     {
-        bdiscontinuity.flag_heads(head_flags, input, FlagOpType());
+        adjacent_difference.flag_heads(head_flags, input, FlagOpType(), storage);
     }
     ROCPRIM_CLANG_SUPPRESS_WARNING_POP
 
@@ -255,7 +260,8 @@ void flag_tails_kernel(Type* device_input, long long* device_tails)
     Type input[ItemsPerThread];
     rocprim::block_load_direct_blocked(lid, device_input + block_offset, input);
 
-    rocprim::block_adjacent_difference<Type, BlockSize> bdiscontinuity;
+    rocprim::block_adjacent_difference<Type, BlockSize>             adjacent_difference;
+    __shared__ typename decltype(adjacent_difference)::storage_type storage;
 
     FlagType tail_flags[ItemsPerThread];
 
@@ -264,11 +270,15 @@ void flag_tails_kernel(Type* device_input, long long* device_tails)
     if(blockIdx.x % 2 == 0)
     {
         const Type tile_successor_item = device_input[block_offset + items_per_block];
-        bdiscontinuity.flag_tails(tail_flags, tile_successor_item, input, FlagOpType());
+        adjacent_difference.flag_tails(tail_flags,
+                                       tile_successor_item,
+                                       input,
+                                       FlagOpType(),
+                                       storage);
     }
     else
     {
-        bdiscontinuity.flag_tails(tail_flags, input, FlagOpType());
+        adjacent_difference.flag_tails(tail_flags, input, FlagOpType(), storage);
     }
     ROCPRIM_CLANG_SUPPRESS_WARNING_POP
 
@@ -293,7 +303,8 @@ void flag_heads_and_tails_kernel(Type* device_input, long long* device_heads, lo
     Type input[ItemsPerThread];
     rocprim::block_load_direct_blocked(lid, device_input + block_offset, input);
 
-    rocprim::block_adjacent_difference<Type, BlockSize> bdiscontinuity;
+    rocprim::block_adjacent_difference<Type, BlockSize>             adjacent_difference;
+    __shared__ typename decltype(adjacent_difference)::storage_type storage;
 
     FlagType head_flags[ItemsPerThread];
     FlagType tail_flags[ItemsPerThread];
@@ -303,22 +314,42 @@ void flag_heads_and_tails_kernel(Type* device_input, long long* device_heads, lo
     if(blockIdx.x % 4 == 0)
     {
         const Type tile_successor_item = device_input[block_offset + items_per_block];
-        bdiscontinuity.flag_heads_and_tails(head_flags, tail_flags, tile_successor_item, input, FlagOpType());
+        adjacent_difference.flag_heads_and_tails(head_flags,
+                                                 tail_flags,
+                                                 tile_successor_item,
+                                                 input,
+                                                 FlagOpType(),
+                                                 storage);
     }
     else if(blockIdx.x % 4 == 1)
     {
         const Type tile_predecessor_item = device_input[block_offset - 1];
-        const Type tile_successor_item = device_input[block_offset + items_per_block];
-        bdiscontinuity.flag_heads_and_tails(head_flags, tile_predecessor_item, tail_flags, tile_successor_item, input, FlagOpType());
+        const Type tile_successor_item   = device_input[block_offset + items_per_block];
+        adjacent_difference.flag_heads_and_tails(head_flags,
+                                                 tile_predecessor_item,
+                                                 tail_flags,
+                                                 tile_successor_item,
+                                                 input,
+                                                 FlagOpType(),
+                                                 storage);
     }
     else if(blockIdx.x % 4 == 2)
     {
         const Type tile_predecessor_item = device_input[block_offset - 1];
-        bdiscontinuity.flag_heads_and_tails(head_flags, tile_predecessor_item, tail_flags, input, FlagOpType());
+        adjacent_difference.flag_heads_and_tails(head_flags,
+                                                 tile_predecessor_item,
+                                                 tail_flags,
+                                                 input,
+                                                 FlagOpType(),
+                                                 storage);
     }
     else if(blockIdx.x % 4 == 3)
     {
-        bdiscontinuity.flag_heads_and_tails(head_flags, tail_flags, input, FlagOpType());
+        adjacent_difference.flag_heads_and_tails(head_flags,
+                                                 tail_flags,
+                                                 input,
+                                                 FlagOpType(),
+                                                 storage);
     }
     ROCPRIM_CLANG_SUPPRESS_WARNING_POP
 
@@ -339,11 +370,15 @@ auto test_block_adjacent_difference()
 {
     using type = Type;
     // std::vector<bool> is a special case that will cause an error in hipMemcpy
+    // rocprim::half/rocprim::bfloat16 are special cases that cannot be compared '=='
+    // in ASSERT_EQ
     using stored_flag_type = typename std::conditional<
-                               std::is_same<bool, FlagType>::value,
-                               int,
-                               FlagType
-                           >::type;
+        std::is_same<bool, FlagType>::value,
+        int,
+        typename std::conditional<std::is_same<rocprim::half, FlagType>::value
+                                      || std::is_same<rocprim::bfloat16, FlagType>::value,
+                                  float,
+                                  FlagType>::type>::type;
     using flag_type = FlagType;
     using flag_op_type = FlagOpType;
     static constexpr size_t block_size = BlockSize;
@@ -352,6 +387,10 @@ auto test_block_adjacent_difference()
     static constexpr size_t size = items_per_block * 20;
     static constexpr size_t grid_size = size / items_per_block;
 
+    SCOPED_TRACE(testing::Message() << "items_per_block = " << items_per_block);
+    SCOPED_TRACE(testing::Message() << "size = " << size);
+    SCOPED_TRACE(testing::Message() << "grid_size = " << grid_size);
+
     // Given block size not supported
     if(block_size > test_utils::get_max_block_size())
     {
@@ -378,8 +417,11 @@ auto test_block_adjacent_difference()
                 if(ii == 0)
                 {
                     expected_heads[i] = bi % 2 == 1
-                        ? apply<Type, FlagType, FlagOpType>(flag_op, input[i - 1], input[i], ii)
-                        : flag_type(true);
+                                            ? apply<Type, FlagType, FlagOpType>(flag_op,
+                                                                                input[i - 1],
+                                                                                input[i],
+                                                                                ii)
+                                            : stored_flag_type(true);
                 }
                 else
                 {
@@ -450,11 +492,15 @@ auto test_block_adjacent_difference()
 {
     using type = Type;
     // std::vector<bool> is a special case that will cause an error in hipMemcpy
+    // rocprim::half/rocprim::bfloat16 are special cases that cannot be compared '=='
+    // in ASSERT_EQ
     using stored_flag_type = typename std::conditional<
-                               std::is_same<bool, FlagType>::value,
-                               int,
-                               FlagType
-                           >::type;
+        std::is_same<bool, FlagType>::value,
+        int,
+        typename std::conditional<std::is_same<rocprim::half, FlagType>::value
+                                      || std::is_same<rocprim::bfloat16, FlagType>::value,
+                                  float,
+                                  FlagType>::type>::type;
     using flag_type = FlagType;
     using flag_op_type = FlagOpType;
     static constexpr size_t block_size = BlockSize;
@@ -463,6 +509,10 @@ auto test_block_adjacent_difference()
     static constexpr size_t size = items_per_block * 20;
     static constexpr size_t grid_size = size / items_per_block;
 
+    SCOPED_TRACE(testing::Message() << "items_per_block = " << items_per_block);
+    SCOPED_TRACE(testing::Message() << "size = " << size);
+    SCOPED_TRACE(testing::Message() << "grid_size = " << grid_size);
+
     // Given block size not supported
     if(block_size > test_utils::get_max_block_size())
     {
@@ -489,8 +539,11 @@ auto test_block_adjacent_difference()
                 if(ii == items_per_block - 1)
                 {
                     expected_tails[i] = bi % 2 == 0
-                        ? apply<Type, FlagType, FlagOpType>(flag_op, input[i], input[i + 1], ii + 1)
-                        : flag_type(true);
+                                            ? apply<Type, FlagType, FlagOpType>(flag_op,
+                                                                                input[i],
+                                                                                input[i + 1],
+                                                                                ii + 1)
+                                            : stored_flag_type(true);
                 }
                 else
                 {
@@ -561,11 +614,15 @@ auto test_block_adjacent_difference()
 {
     using type = Type;
     // std::vector<bool> is a special case that will cause an error in hipMemcpy
+    // rocprim::half/rocprim::bfloat16 are special cases that cannot be compared '=='
+    // in ASSERT_EQ
     using stored_flag_type = typename std::conditional<
-                               std::is_same<bool, FlagType>::value,
-                               int,
-                               FlagType
-                           >::type;
+        std::is_same<bool, FlagType>::value,
+        int,
+        typename std::conditional<std::is_same<rocprim::half, FlagType>::value
+                                      || std::is_same<rocprim::bfloat16, FlagType>::value,
+                                  float,
+                                  FlagType>::type>::type;
     using flag_type = FlagType;
     using flag_op_type = FlagOpType;
     static constexpr size_t block_size = BlockSize;
@@ -574,6 +631,10 @@ auto test_block_adjacent_difference()
     static constexpr size_t size = items_per_block * 20;
     static constexpr size_t grid_size = size / items_per_block;
 
+    SCOPED_TRACE(testing::Message() << "items_per_block = " << items_per_block);
+    SCOPED_TRACE(testing::Message() << "size = " << size);
+    SCOPED_TRACE(testing::Message() << "grid_size = " << grid_size);
+
     // Given block size not supported
     if(block_size > test_utils::get_max_block_size())
     {
@@ -602,8 +663,11 @@ auto test_block_adjacent_difference()
                 if(ii == 0)
                 {
                     expected_heads[i] = (bi % 4 == 1 || bi % 4 == 2)
-                        ? apply<Type, FlagType, FlagOpType>(flag_op, input[i - 1], input[i], ii)
-                        : flag_type(true);
+                                            ? apply<Type, FlagType, FlagOpType>(flag_op,
+                                                                                input[i - 1],
+                                                                                input[i],
+                                                                                ii)
+                                            : stored_flag_type(true);
                 }
                 else
                 {
@@ -612,8 +676,11 @@ auto test_block_adjacent_difference()
                 if(ii == items_per_block - 1)
                 {
                     expected_tails[i] = (bi % 4 == 0 || bi % 4 == 1)
-                        ? apply<Type, FlagType, FlagOpType>(flag_op, input[i], input[i + 1], ii + 1)
-                        : flag_type(true);
+                                            ? apply<Type, FlagType, FlagOpType>(flag_op,
+                                                                                input[i],
+                                                                                input[i + 1],
+                                                                                ii + 1)
+                                            : stored_flag_type(true);
                 }
                 else
                 {
@@ -703,6 +770,10 @@ auto test_block_adjacent_difference() -> typename std::enable_if<Method == 3>::t
     SCOPED_TRACE(testing::Message() << "with block_size = " << block_size << ", items_per_thread = "
                                     << items_per_thread << ", size = " << size);
 
+    SCOPED_TRACE(testing::Message() << "items_per_block = " << items_per_block);
+    SCOPED_TRACE(testing::Message() << "size = " << size);
+    SCOPED_TRACE(testing::Message() << "grid_size = " << grid_size);
+
     // Given block size not supported
     if(block_size > test_utils::get_max_block_size())
     {
@@ -794,6 +865,10 @@ auto test_block_adjacent_difference() -> typename std::enable_if<Method == 4>::t
     SCOPED_TRACE(testing::Message() << "with block_size = " << block_size << ", items_per_thread = "
                                     << items_per_thread << ", size = " << size);
 
+    SCOPED_TRACE(testing::Message() << "items_per_block = " << items_per_block);
+    SCOPED_TRACE(testing::Message() << "size = " << size);
+    SCOPED_TRACE(testing::Message() << "grid_size = " << grid_size);
+
     // Given block size not supported
     if(block_size > test_utils::get_max_block_size())
     {
@@ -885,6 +960,10 @@ auto test_block_adjacent_difference() -> typename std::enable_if<Method == 5>::t
     SCOPED_TRACE(testing::Message() << "with block_size = " << block_size << ", items_per_thread = "
                                     << items_per_thread << ", size = " << size);
 
+    SCOPED_TRACE(testing::Message() << "items_per_block = " << items_per_block);
+    SCOPED_TRACE(testing::Message() << "size = " << size);
+    SCOPED_TRACE(testing::Message() << "grid_size = " << grid_size);
+
     // Given block size not supported
     if(block_size > test_utils::get_max_block_size())
     {
@@ -994,6 +1073,10 @@ auto test_block_adjacent_difference() -> typename std::enable_if<Method == 6>::t
     SCOPED_TRACE(testing::Message() << "with block_size = " << block_size << ", items_per_thread = "
                                     << items_per_thread << ", size = " << size);
 
+    SCOPED_TRACE(testing::Message() << "items_per_block = " << items_per_block);
+    SCOPED_TRACE(testing::Message() << "size = " << size);
+    SCOPED_TRACE(testing::Message() << "grid_size = " << grid_size);
+
     // Given block size not supported
     if(block_size > test_utils::get_max_block_size())
     {
@@ -1090,41 +1173,43 @@ auto test_block_adjacent_difference() -> typename std::enable_if<Method == 6>::t
 }
 
 // Static for-loop
-template <
-    unsigned int First,
-    unsigned int Last,
-    class Type,
-    class FlagType,
-    class FlagOpType,
-    unsigned int Method,
-    unsigned int BlockSize = 256U
->
+template<unsigned int First,
+         unsigned int Last,
+         class Type,
+         class FlagType,
+         class FlagOpType,
+         unsigned int Method,
+         unsigned int BlockSize = 256U>
 struct static_for
 {
     static void run()
     {
-        int device_id = test_common_utils::obtain_device_from_ctest();
-        SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
-        HIP_CHECK(hipSetDevice(device_id));
+        {
+            SCOPED_TRACE(testing::Message() << "TestID = " << First);
+            int device_id = test_common_utils::obtain_device_from_ctest();
+            SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
+            HIP_CHECK(hipSetDevice(device_id));
 
-        test_block_adjacent_difference<Type, FlagType, FlagOpType, Method, BlockSize, items[First]>();
+            test_block_adjacent_difference<Type,
+                                           FlagType,
+                                           FlagOpType,
+                                           Method,
+                                           BlockSize,
+                                           items[First]>();
+        }
         static_for<First + 1, Last, Type, FlagType, FlagOpType, Method, BlockSize>::run();
     }
 };
 
-template <
-    unsigned int N,
-    class Type,
-    class FlagType,
-    class FlagOpType,
-    unsigned int Method,
-    unsigned int BlockSize
->
+template<unsigned int N,
+         class Type,
+         class FlagType,
+         class FlagOpType,
+         unsigned int Method,
+         unsigned int BlockSize>
 struct static_for<N, N, Type, FlagType, FlagOpType, Method, BlockSize>
 {
-    static void run()
-    {
-    }
+    static void run() {}
 };
 
 #endif // TEST_BLOCK_ADJACENT_DIFFERENCE_KERNELS_HPP_
