@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -38,7 +38,6 @@ typed_test_def(suite_name_single, name_suffix, Reduce)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
-    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     constexpr size_t block_size = TestFixture::block_size;
 
@@ -70,7 +69,7 @@ typed_test_def(suite_name_single, name_suffix, Reduce)
                 auto idx = i * block_size + j;
                 value = binary_op_host(value, output[idx]);
             }
-            expected_reductions[i] = static_cast<cast_type>(value);
+            expected_reductions[i] = static_cast<T>(value);
         }
 
         // Preparing device
@@ -95,7 +94,6 @@ typed_test_def(suite_name_single, name_suffix, Reduce)
         HIP_CHECK(hipFree(device_output));
         HIP_CHECK(hipFree(device_output_reductions));
     }
-
 }
 
 typed_test_def(suite_name_single, name_suffix, ReduceMultiplies)
@@ -107,8 +105,7 @@ typed_test_def(suite_name_single, name_suffix, ReduceMultiplies)
     using T                     = typename TestFixture::input_type;
     using binary_op_type        = rocprim::multiplies<T>;
     constexpr size_t block_size = TestFixture::block_size;
-    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
-    
+
     // Given block size not supported
     if(block_size > test_utils::get_max_block_size())
     {
@@ -138,7 +135,7 @@ typed_test_def(suite_name_single, name_suffix, ReduceMultiplies)
                 auto idx = i * block_size + j;
                 value *= static_cast<double>(output[idx]);
             }
-            expected_reductions[i] = static_cast<cast_type>(value);
+            expected_reductions[i] = static_cast<T>(value);
         }
 
         // Preparing device
@@ -269,7 +266,6 @@ typed_test_def(suite_name_single, name_suffix, ReduceValid)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
-    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     constexpr size_t block_size = TestFixture::block_size;
 
@@ -303,7 +299,7 @@ typed_test_def(suite_name_single, name_suffix, ReduceValid)
                 auto idx = i * block_size + j;
                 value = binary_op_host(value, output[idx]);
             }
-            expected_reductions[i] = static_cast<cast_type>(value);
+            expected_reductions[i] = static_cast<T>(value);
         }
 
         // Preparing device
@@ -328,7 +324,6 @@ typed_test_def(suite_name_single, name_suffix, ReduceValid)
         HIP_CHECK(hipFree(device_output));
         HIP_CHECK(hipFree(device_output_reductions));
     }
-
 }
 
 typed_test_def(suite_name_array, name_suffix, Reduce)
@@ -340,8 +335,9 @@ typed_test_def(suite_name_array, name_suffix, Reduce)
     using T = typename TestFixture::input_type;
     constexpr size_t block_size = TestFixture::block_size;
 
-    static_for_input_array<0, 2, T, block_size, rocprim::block_reduce_algorithm::using_warp_reduce>::run();
-    static_for_input_array<0, 2, T, block_size, rocprim::block_reduce_algorithm::raking_reduce>::run();
-    static_for_input_array<0, 2, T, block_size, rocprim::block_reduce_algorithm::raking_reduce_commutative_only>::run();
+    using bra = rocprim::block_reduce_algorithm;
 
+    static_for_input_array<0, 6, T, block_size, bra::using_warp_reduce>::run();
+    static_for_input_array<0, 6, T, block_size, bra::raking_reduce>::run();
+    static_for_input_array<0, 6, T, block_size, bra::raking_reduce_commutative_only>::run();
 }
