@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -117,7 +117,7 @@ TYPED_TEST(RocprimConstantIteratorTests, Transform)
         // Validating results
         test_utils::assert_near(output, expected, test_utils::precision<T>);
 
-        hipFree(d_output);
+        HIP_CHECK(hipFree(d_output));
     }
 
 }
