@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2021 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -126,7 +126,7 @@ TYPED_TEST(RocprimCountingIteratorTests, Transform)
             ASSERT_EQ(output[i], expected[i]) << "where index = " << i;
         }
 
-        hipFree(d_output);
+        HIP_CHECK(hipFree(d_output));
     }
 
 }
