@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2019-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2019-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -29,18 +29,20 @@
 // required test headers
 #include "test_utils_types.hpp"
 
-template<
-    class Haystack,
-    class Needle,
-    class Output = size_t,
-    class CompareFunction = rocprim::less<>
->
+template<class Haystack,
+         class Needle,
+         class Output          = size_t,
+         class CompareFunction = rocprim::less<>,
+         class Config          = rocprim::default_config,
+         bool  UseGraphs       = false>
 struct params
 {
     using haystack_type = Haystack;
     using needle_type = Needle;
     using output_type = Output;
     using compare_op_type = CompareFunction;
+    using config          = Config;
+    static constexpr bool use_graphs = UseGraphs;
 };
 
 template<class Params>
@@ -52,6 +54,9 @@ public:
 using custom_int2 = test_utils::custom_test_type<int>;
 using custom_double2 = test_utils::custom_test_type<double>;
 
+struct use_custom_config
+{};
+
 typedef ::testing::Types<
     params<int, int>,
     params<unsigned long long, unsigned long long, size_t, rocprim::greater<unsigned long long>>,
@@ -60,9 +65,14 @@ typedef ::testing::Types<
     params<int8_t, int8_t>,
     params<uint8_t, uint8_t>,
     params<rocprim::half, rocprim::half, size_t, rocprim::less<rocprim::half>>,
-    params<rocprim::bfloat16, rocprim::bfloat16, size_t, rocprim::less<rocprim::bfloat16>>,
+    params<rocprim::bfloat16,
+           rocprim::bfloat16,
+           size_t,
+           rocprim::less<rocprim::bfloat16>,
+           use_custom_config>,
     params<custom_int2, custom_int2>,
-    params<custom_double2, custom_double2, unsigned int, rocprim::greater<custom_double2>>>
+    params<custom_double2, custom_double2, unsigned int, rocprim::greater<custom_double2>>,
+    params<int, int, size_t, rocprim::less<>, rocprim::default_config, true>>
     Params;
 
 TYPED_TEST_SUITE(RocprimDeviceBinarySearch, Params);
@@ -77,8 +87,17 @@ TYPED_TEST(RocprimDeviceBinarySearch, LowerBound)
     using needle_type = typename TestFixture::params::needle_type;
     using output_type = typename TestFixture::params::output_type;
     using compare_op_type = typename TestFixture::params::compare_op_type;
+    using config          = std::conditional_t<
+        std::is_same<typename TestFixture::params::config, use_custom_config>::value,
+        rocprim::lower_bound_config<64, 2>,
+        typename TestFixture::params::config>;
 
     hipStream_t stream = 0;
+    if (TestFixture::params::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
     const bool debug_synchronous = false;
 
@@ -140,29 +159,42 @@ TYPED_TEST(RocprimDeviceBinarySearch, LowerBound)
 
             void * d_temporary_storage = nullptr;
             size_t temporary_storage_bytes;
-            HIP_CHECK(
-                rocprim::lower_bound(
-                    d_temporary_storage, temporary_storage_bytes,
-                    d_haystack, d_needles, d_output,
-                    haystack_size, needles_size,
-                    compare_op,
-                    stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(rocprim::lower_bound<config>(d_temporary_storage,
+                                                   temporary_storage_bytes,
+                                                   d_haystack,
+                                                   d_needles,
+                                                   d_output,
+                                                   haystack_size,
+                                                   needles_size,
+                                                   compare_op,
+                                                   stream,
+                                                   debug_synchronous));
 
             ASSERT_GT(temporary_storage_bytes, 0);
 
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
-            HIP_CHECK(
-                rocprim::lower_bound(
-                    d_temporary_storage, temporary_storage_bytes,
-                    d_haystack, d_needles, d_output,
-                    haystack_size, needles_size,
-                    compare_op,
-                    stream, debug_synchronous
-                )
-            );
+            test_utils::GraphHelper gHelper;;
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.startStreamCapture(stream);
+            }
+
+            HIP_CHECK(rocprim::lower_bound<config>(d_temporary_storage,
+                                                   temporary_storage_bytes,
+                                                   d_haystack,
+                                                   d_needles,
+                                                   d_output,
+                                                   haystack_size,
+                                                   needles_size,
+                                                   compare_op,
+                                                   stream,
+                                                   debug_synchronous));
+
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.createAndLaunchGraph(stream);
+            }
 
             std::vector<output_type> output(needles_size);
             HIP_CHECK(
@@ -178,11 +210,17 @@ TYPED_TEST(RocprimDeviceBinarySearch, LowerBound)
             HIP_CHECK(hipFree(d_needles));
             HIP_CHECK(hipFree(d_output));
 
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.cleanupGraphHelper();
+            }
+
             ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
         }
     }
 
-
+    if (TestFixture::params::use_graphs)
+        HIP_CHECK(hipStreamDestroy(stream));
 }
 
 TYPED_TEST(RocprimDeviceBinarySearch, UpperBound)
@@ -195,8 +233,17 @@ TYPED_TEST(RocprimDeviceBinarySearch, UpperBound)
     using needle_type = typename TestFixture::params::needle_type;
     using output_type = typename TestFixture::params::output_type;
     using compare_op_type = typename TestFixture::params::compare_op_type;
+    using config          = std::conditional_t<
+        std::is_same<typename TestFixture::params::config, use_custom_config>::value,
+        rocprim::upper_bound_config<64, 2>,
+        typename TestFixture::params::config>;
 
     hipStream_t stream = 0;
+    if (TestFixture::params::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
     const bool debug_synchronous = false;
 
@@ -257,29 +304,42 @@ TYPED_TEST(RocprimDeviceBinarySearch, UpperBound)
 
             void * d_temporary_storage = nullptr;
             size_t temporary_storage_bytes;
-            HIP_CHECK(
-                rocprim::upper_bound(
-                    d_temporary_storage, temporary_storage_bytes,
-                    d_haystack, d_needles, d_output,
-                    haystack_size, needles_size,
-                    compare_op,
-                    stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(rocprim::upper_bound<config>(d_temporary_storage,
+                                                   temporary_storage_bytes,
+                                                   d_haystack,
+                                                   d_needles,
+                                                   d_output,
+                                                   haystack_size,
+                                                   needles_size,
+                                                   compare_op,
+                                                   stream,
+                                                   debug_synchronous));
 
             ASSERT_GT(temporary_storage_bytes, 0);
 
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
-            HIP_CHECK(
-                rocprim::upper_bound(
-                    d_temporary_storage, temporary_storage_bytes,
-                    d_haystack, d_needles, d_output,
-                    haystack_size, needles_size,
-                    compare_op,
-                    stream, debug_synchronous
-                )
-            );
+            test_utils::GraphHelper gHelper;;
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.startStreamCapture(stream);
+            }
+
+            HIP_CHECK(rocprim::upper_bound<config>(d_temporary_storage,
+                                                   temporary_storage_bytes,
+                                                   d_haystack,
+                                                   d_needles,
+                                                   d_output,
+                                                   haystack_size,
+                                                   needles_size,
+                                                   compare_op,
+                                                   stream,
+                                                   debug_synchronous));
+
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.createAndLaunchGraph(stream);
+            }
 
             std::vector<output_type> output(needles_size);
             HIP_CHECK(
@@ -295,11 +355,19 @@ TYPED_TEST(RocprimDeviceBinarySearch, UpperBound)
             HIP_CHECK(hipFree(d_needles));
             HIP_CHECK(hipFree(d_output));
 
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.cleanupGraphHelper();
+            }
+
             ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
         }
     }
 
-
+    if(TestFixture::params::use_graphs)
+    {
+        HIP_CHECK(hipStreamDestroy(stream));
+    }
 }
 
 TYPED_TEST(RocprimDeviceBinarySearch, BinarySearch)
@@ -312,8 +380,17 @@ TYPED_TEST(RocprimDeviceBinarySearch, BinarySearch)
     using needle_type = typename TestFixture::params::needle_type;
     using output_type = typename TestFixture::params::output_type;
     using compare_op_type = typename TestFixture::params::compare_op_type;
+    using config          = std::conditional_t<
+        std::is_same<typename TestFixture::params::config, use_custom_config>::value,
+        rocprim::binary_search_config<64, 2>,
+        typename TestFixture::params::config>;
 
     hipStream_t stream = 0;
+    if (TestFixture::params::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
     const bool debug_synchronous = false;
 
@@ -373,29 +450,42 @@ TYPED_TEST(RocprimDeviceBinarySearch, BinarySearch)
 
             void * d_temporary_storage = nullptr;
             size_t temporary_storage_bytes;
-            HIP_CHECK(
-                rocprim::binary_search(
-                    d_temporary_storage, temporary_storage_bytes,
-                    d_haystack, d_needles, d_output,
-                    haystack_size, needles_size,
-                    compare_op,
-                    stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(rocprim::binary_search<config>(d_temporary_storage,
+                                                     temporary_storage_bytes,
+                                                     d_haystack,
+                                                     d_needles,
+                                                     d_output,
+                                                     haystack_size,
+                                                     needles_size,
+                                                     compare_op,
+                                                     stream,
+                                                     debug_synchronous));
 
             ASSERT_GT(temporary_storage_bytes, 0);
 
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
-            HIP_CHECK(
-                rocprim::binary_search(
-                    d_temporary_storage, temporary_storage_bytes,
-                    d_haystack, d_needles, d_output,
-                    haystack_size, needles_size,
-                    compare_op,
-                    stream, debug_synchronous
-                )
-            );
+            test_utils::GraphHelper gHelper;;
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.startStreamCapture(stream);
+            }
+
+            HIP_CHECK(rocprim::binary_search<config>(d_temporary_storage,
+                                                     temporary_storage_bytes,
+                                                     d_haystack,
+                                                     d_needles,
+                                                     d_output,
+                                                     haystack_size,
+                                                     needles_size,
+                                                     compare_op,
+                                                     stream,
+                                                     debug_synchronous));
+
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.createAndLaunchGraph(stream);
+            }
 
             std::vector<output_type> output(needles_size);
             HIP_CHECK(
@@ -411,7 +501,17 @@ TYPED_TEST(RocprimDeviceBinarySearch, BinarySearch)
             HIP_CHECK(hipFree(d_needles));
             HIP_CHECK(hipFree(d_output));
 
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.cleanupGraphHelper();
+            }
+
             ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(output, expected));
         }
     }
+
+    if(TestFixture::params::use_graphs)
+    {
+        HIP_CHECK(hipStreamDestroy(stream));
+    }
 }
