@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2025 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -24,6 +24,9 @@
 #include "../common_test_header.hpp"
 #include "test_utils_types.hpp"
 
+#include "../../common/utils.hpp"
+#include "../../common/utils_device_ptr.hpp"
+
 // required rocprim headers
 #include <rocprim/device/device_merge.hpp>
 #include <rocprim/functional.hpp>
@@ -36,19 +39,22 @@
 
 #include <algorithm>
 #include <numeric>
-#include <vector>
+
+using DefaultConfig = rocprim::default_config;
 
 // Params for tests
-template<
-    class KeyType,
-    class ValueType,
-    class CompareOp = ::rocprim::less<KeyType>
->
+template<class KeyType,
+         class ValueType,
+         class CompareOp = ::rocprim::less<KeyType>,
+         bool UseGraphs  = false,
+         typename Config = rocprim::default_config>
 struct DeviceMergeParams
 {
     using key_type = KeyType;
     using value_type = ValueType;
     using compare_op_type = CompareOp;
+    static constexpr bool use_graphs = UseGraphs;
+    using config                     = Config;
 };
 
 template<class Params>
@@ -58,7 +64,9 @@ public:
     using key_type = typename Params::key_type;
     using value_type = typename Params::value_type;
     using compare_op_type = typename Params::compare_op_type;
+    using config                            = typename Params::config;
     const bool debug_synchronous = false;
+    static constexpr bool use_graphs = Params::use_graphs;
 };
 
 using custom_int2 = test_utils::custom_test_type<int>;
@@ -69,12 +77,14 @@ typedef ::testing::Types<
     DeviceMergeParams<unsigned long, unsigned int, rocprim::greater<unsigned long>>,
     DeviceMergeParams<float, custom_double2>,
     DeviceMergeParams<int, float>,
+    DeviceMergeParams<double, double>,
     DeviceMergeParams<int8_t, int8_t>,
     DeviceMergeParams<uint8_t, uint8_t>,
     DeviceMergeParams<rocprim::half, rocprim::half, rocprim::less<rocprim::half>>,
     DeviceMergeParams<rocprim::bfloat16, rocprim::bfloat16, rocprim::less<rocprim::bfloat16>>,
     DeviceMergeParams<custom_double2, custom_int2, rocprim::greater<custom_double2>>,
-    DeviceMergeParams<custom_int2, char>>
+    DeviceMergeParams<custom_int2, char>,
+    DeviceMergeParams<int, int, ::rocprim::less<int>, true>>
     RocprimDeviceMergeTestsParams;
 
 // size1, size2
@@ -102,6 +112,7 @@ TYPED_TEST_SUITE(RocprimDeviceMergeTests, RocprimDeviceMergeTestsParams);
 
 TYPED_TEST(RocprimDeviceMergeTests, MergeKey)
 {
+    using config  = typename TestFixture::config;
     int device_id = test_common_utils::obtain_device_from_ctest();
     SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
@@ -111,14 +122,27 @@ TYPED_TEST(RocprimDeviceMergeTests, MergeKey)
     const bool debug_synchronous = TestFixture::debug_synchronous;
 
     hipStream_t stream = 0; // default
+    if (TestFixture::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
     for(auto sizes : get_sizes())
     {
-        if ((std::get<0>(sizes) == 0 || std::get<1>(sizes) == 0) && test_common_utils::use_hmm())
+        if((std::get<0>(sizes) == 0 || std::get<1>(sizes) == 0) && test_common_utils::use_hmm())
         {
             // hipMallocManaged() currently doesnt support zero byte allocation
             continue;
         }
+
+        if((std::get<0>(sizes) + std::get<1>(sizes) >= 100000
+            && sizeof(key_type) > sizeof(size_t) * 16))
+        {
+            // Huge types are slow
+            continue;
+        }
+
         SCOPED_TRACE(
             testing::Message() << "with sizes = {" <<
             std::get<0>(sizes) << ", " << std::get<1>(sizes) << "}"
@@ -130,7 +154,7 @@ TYPED_TEST(RocprimDeviceMergeTests, MergeKey)
         // compare function
         compare_op_type compare_op;
 
-        for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+        for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
         {
             unsigned int seed_value = seed_index < random_seeds_count  ? rand() : seeds[seed_index - random_seeds_count];
             SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
@@ -155,92 +179,80 @@ TYPED_TEST(RocprimDeviceMergeTests, MergeKey)
 
             test_utils::out_of_bounds_flag out_of_bounds;
 
-            key_type * d_keys_input1;
-            key_type * d_keys_input2;
-            key_type * d_keys_output;
-            HIP_CHECK(test_common_utils::hipMallocHelper(&d_keys_input1, keys_input1.size() * sizeof(key_type)));
-            HIP_CHECK(test_common_utils::hipMallocHelper(&d_keys_input2, keys_input2.size() * sizeof(key_type)));
-            HIP_CHECK(test_common_utils::hipMallocHelper(&d_keys_output, keys_output.size() * sizeof(key_type)));
-            HIP_CHECK(
-                hipMemcpy(
-                    d_keys_input1, keys_input1.data(),
-                    keys_input1.size() * sizeof(key_type),
-                    hipMemcpyHostToDevice
-                )
-            );
-            HIP_CHECK(
-                hipMemcpy(
-                    d_keys_input2, keys_input2.data(),
-                    keys_input2.size() * sizeof(key_type),
-                    hipMemcpyHostToDevice
-                )
-            );
+            common::device_ptr<key_type> d_keys_input1(keys_input1);
+            common::device_ptr<key_type> d_keys_input2(keys_input2);
+            common::device_ptr<key_type> d_keys_output(keys_output.size());
 
             test_utils::bounds_checking_iterator<key_type> d_keys_checking_output(
-                d_keys_output,
+                d_keys_output.get(),
                 out_of_bounds.device_pointer(),
-                size1 + size2
-            );
+                size1 + size2);
 
-            // temp storage
-            size_t temp_storage_size_bytes;
-            void * d_temp_storage = nullptr;
             // Get size of d_temp_storage
-            HIP_CHECK(
-                rocprim::merge(
-                    d_temp_storage, temp_storage_size_bytes,
-                    d_keys_input1, d_keys_input2,
-                    d_keys_checking_output,
-                    keys_input1.size(), keys_input2.size(),
-                    compare_op, stream, debug_synchronous
-                )
-            );
+            size_t temp_storage_size_bytes;
+            HIP_CHECK(rocprim::merge<config>(nullptr,
+                                             temp_storage_size_bytes,
+                                             d_keys_input1.get(),
+                                             d_keys_input2.get(),
+                                             d_keys_checking_output,
+                                             keys_input1.size(),
+                                             keys_input2.size(),
+                                             compare_op,
+                                             stream,
+                                             debug_synchronous));
 
             // temp_storage_size_bytes must be >0
             ASSERT_GT(temp_storage_size_bytes, 0);
 
             // allocate temporary storage
-            HIP_CHECK(test_common_utils::hipMallocHelper(&d_temp_storage, temp_storage_size_bytes));
+            common::device_ptr<void> d_temp_storage(temp_storage_size_bytes);
+
+            test_utils::GraphHelper gHelper;
+            if(TestFixture::use_graphs)
+            {
+                gHelper.startStreamCapture(stream);
+            }
 
             // Run
-            HIP_CHECK(
-                rocprim::merge(
-                    d_temp_storage, temp_storage_size_bytes,
-                    d_keys_input1, d_keys_input2,
-                    d_keys_checking_output,
-                    keys_input1.size(), keys_input2.size(),
-                    compare_op, stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(rocprim::merge<config>(d_temp_storage.get(),
+                                             temp_storage_size_bytes,
+                                             d_keys_input1.get(),
+                                             d_keys_input2.get(),
+                                             d_keys_checking_output,
+                                             keys_input1.size(),
+                                             keys_input2.size(),
+                                             compare_op,
+                                             stream,
+                                             debug_synchronous));
+
+            if(TestFixture::use_graphs)
+            {
+                gHelper.createAndLaunchGraph(stream);
+            }
+
             HIP_CHECK(hipGetLastError());
             HIP_CHECK(hipDeviceSynchronize());
 
             ASSERT_FALSE(out_of_bounds.get());
 
             // Copy keys_output to host
-            HIP_CHECK(
-                hipMemcpy(
-                    keys_output.data(), d_keys_output,
-                    keys_output.size() * sizeof(key_type),
-                    hipMemcpyDeviceToHost
-                )
-            );
+            keys_output = d_keys_output.load();
 
             // Check if keys_output values are as expected
             ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(keys_output, expected));
 
-            hipFree(d_keys_input1);
-            hipFree(d_keys_input2);
-            hipFree(d_keys_output);
-            hipFree(d_temp_storage);
+            if (TestFixture::use_graphs)
+                gHelper.cleanupGraphHelper();
         }
-
-
     }
+
+    if (TestFixture::use_graphs)
+        HIP_CHECK(hipStreamDestroy(stream));
 }
 
 TYPED_TEST(RocprimDeviceMergeTests, MergeKeyValue)
 {
+    using config  = typename TestFixture::config;
     int device_id = test_common_utils::obtain_device_from_ctest();
     SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
     HIP_CHECK(hipSetDevice(device_id));
@@ -248,19 +260,24 @@ TYPED_TEST(RocprimDeviceMergeTests, MergeKeyValue)
     using key_type = typename TestFixture::key_type;
     using value_type = typename TestFixture::value_type;
     using compare_op_type = typename TestFixture::compare_op_type;
-    const bool debug_synchronous = TestFixture::debug_synchronous;
 
     using key_value = std::pair<key_type, value_type>;
 
     hipStream_t stream = 0; // default
+    if (TestFixture::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
     for(auto sizes : get_sizes())
     {
-        if ((std::get<0>(sizes) == 0 || std::get<1>(sizes) == 0) && test_common_utils::use_hmm())
+        if((std::get<0>(sizes) == 0 || std::get<1>(sizes) == 0) && test_common_utils::use_hmm())
         {
             // hipMallocManaged() currently doesnt support zero byte allocation
             continue;
         }
+
         SCOPED_TRACE(
             testing::Message() << "with sizes = {" <<
             std::get<0>(sizes) << ", " << std::get<1>(sizes) << "}"
@@ -314,111 +331,77 @@ TYPED_TEST(RocprimDeviceMergeTests, MergeKeyValue)
 
             test_utils::out_of_bounds_flag out_of_bounds;
 
-            key_type * d_keys_input1;
-            key_type * d_keys_input2;
-            key_type * d_keys_output;
-            value_type * d_values_input1;
-            value_type * d_values_input2;
-            value_type * d_values_output;
-            HIP_CHECK(test_common_utils::hipMallocHelper(&d_keys_input1, keys_input1.size() * sizeof(key_type)));
-            HIP_CHECK(test_common_utils::hipMallocHelper(&d_keys_input2, keys_input2.size() * sizeof(key_type)));
-            HIP_CHECK(test_common_utils::hipMallocHelper(&d_keys_output, keys_output.size() * sizeof(key_type)));
-            HIP_CHECK(test_common_utils::hipMallocHelper(&d_values_input1, values_input1.size() * sizeof(value_type)));
-            HIP_CHECK(test_common_utils::hipMallocHelper(&d_values_input2, values_input2.size() * sizeof(value_type)));
-            HIP_CHECK(test_common_utils::hipMallocHelper(&d_values_output, values_output.size() * sizeof(value_type)));
-            HIP_CHECK(
-                hipMemcpy(
-                    d_keys_input1, keys_input1.data(),
-                    keys_input1.size() * sizeof(key_type),
-                    hipMemcpyHostToDevice
-                )
-            );
-            HIP_CHECK(
-                hipMemcpy(
-                    d_keys_input2, keys_input2.data(),
-                    keys_input2.size() * sizeof(key_type),
-                    hipMemcpyHostToDevice
-                )
-            );
-            HIP_CHECK(
-                hipMemcpy(
-                    d_values_input1, values_input1.data(),
-                    values_input1.size() * sizeof(value_type),
-                    hipMemcpyHostToDevice
-                )
-            );
-            HIP_CHECK(
-                hipMemcpy(
-                    d_values_input2, values_input2.data(),
-                    values_input2.size() * sizeof(value_type),
-                    hipMemcpyHostToDevice
-                )
-            );
+            common::device_ptr<key_type>   d_keys_input1(keys_input1);
+            common::device_ptr<key_type>   d_keys_input2(keys_input2);
+            common::device_ptr<key_type>   d_keys_output(keys_output.size());
+            common::device_ptr<value_type> d_values_input1(values_input1);
+            common::device_ptr<value_type> d_values_input2(values_input2);
+            common::device_ptr<value_type> d_values_output(values_output.size());
 
             test_utils::bounds_checking_iterator<key_type> d_keys_checking_output(
-                d_keys_output,
+                d_keys_output.get(),
                 out_of_bounds.device_pointer(),
-                size1 + size2
-            );
+                size1 + size2);
             test_utils::bounds_checking_iterator<value_type> d_values_checking_output(
-                d_values_output,
+                d_values_output.get(),
                 out_of_bounds.device_pointer(),
-                size1 + size2
-            );
+                size1 + size2);
 
-            // temp storage
-            size_t temp_storage_size_bytes;
-            void * d_temp_storage = nullptr;
             // Get size of d_temp_storage
-            HIP_CHECK(
-                rocprim::merge(
-                    d_temp_storage, temp_storage_size_bytes,
-                    d_keys_input1, d_keys_input2,
-                    d_keys_checking_output,
-                    d_values_input1, d_values_input2,
-                    d_values_checking_output,
-                    keys_input1.size(), keys_input2.size(),
-                    compare_op, stream, debug_synchronous
-                )
-            );
+            size_t temp_storage_size_bytes;
+            HIP_CHECK(rocprim::merge<config>(nullptr,
+                                             temp_storage_size_bytes,
+                                             d_keys_input1.get(),
+                                             d_keys_input2.get(),
+                                             d_keys_checking_output,
+                                             d_values_input1.get(),
+                                             d_values_input2.get(),
+                                             d_values_checking_output,
+                                             keys_input1.size(),
+                                             keys_input2.size(),
+                                             compare_op,
+                                             stream,
+                                             TestFixture::debug_synchronous));
 
             // temp_storage_size_bytes must be >0
             ASSERT_GT(temp_storage_size_bytes, 0);
 
             // allocate temporary storage
-            HIP_CHECK(test_common_utils::hipMallocHelper(&d_temp_storage, temp_storage_size_bytes));
+            common::device_ptr<void> d_temp_storage(temp_storage_size_bytes);
+
+            test_utils::GraphHelper gHelper;
+            if(TestFixture::use_graphs)
+            {
+                gHelper.startStreamCapture(stream);
+            }
 
             // Run
-            HIP_CHECK(
-                rocprim::merge(
-                    d_temp_storage, temp_storage_size_bytes,
-                    d_keys_input1, d_keys_input2,
-                    d_keys_checking_output,
-                    d_values_input1, d_values_input2,
-                    d_values_checking_output,
-                    keys_input1.size(), keys_input2.size(),
-                    compare_op, stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(rocprim::merge<config>(d_temp_storage.get(),
+                                             temp_storage_size_bytes,
+                                             d_keys_input1.get(),
+                                             d_keys_input2.get(),
+                                             d_keys_checking_output,
+                                             d_values_input1.get(),
+                                             d_values_input2.get(),
+                                             d_values_checking_output,
+                                             keys_input1.size(),
+                                             keys_input2.size(),
+                                             compare_op,
+                                             stream,
+                                             TestFixture::debug_synchronous));
+
+            if(TestFixture::use_graphs)
+            {
+                gHelper.createAndLaunchGraph(stream, true, false);
+            }
+
             HIP_CHECK(hipGetLastError());
             HIP_CHECK(hipDeviceSynchronize());
 
             ASSERT_FALSE(out_of_bounds.get());
 
-            HIP_CHECK(
-                hipMemcpy(
-                    keys_output.data(), d_keys_output,
-                    keys_output.size() * sizeof(key_type),
-                    hipMemcpyDeviceToHost
-                )
-            );
-            HIP_CHECK(
-                hipMemcpy(
-                    values_output.data(), d_values_output,
-                    values_output.size() * sizeof(value_type),
-                    hipMemcpyDeviceToHost
-                )
-            );
+            keys_output   = d_keys_output.load();
+            values_output = d_values_output.load();
 
             // Check if keys_output values are as expected
             std::vector<key_type> expected_key(expected.size());
@@ -431,19 +414,19 @@ TYPED_TEST(RocprimDeviceMergeTests, MergeKeyValue)
             ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(keys_output, expected_key));
             ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(values_output, expected_value));
 
-            hipFree(d_keys_input1);
-            hipFree(d_keys_input2);
-            hipFree(d_keys_output);
-            hipFree(d_values_input1);
-            hipFree(d_values_input2);
-            hipFree(d_values_output);
-            hipFree(d_temp_storage);
+            if (TestFixture::use_graphs)
+                gHelper.cleanupGraphHelper();
         }
+    }
 
+    if (TestFixture::use_graphs)
+    {
+        HIP_CHECK(hipStreamDestroy(stream));
     }
 }
 
-TEST(RocprimDeviceMergeTests, MergeMismatchedIteratorTypes)
+template<bool UseGraphs = false, typename config = rocprim::default_config>
+void testMergeMismatchedIteratorTypes()
 {
     const int device_id = test_common_utils::obtain_device_from_ctest();
     SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
@@ -462,62 +445,77 @@ TEST(RocprimDeviceMergeTests, MergeMismatchedIteratorTypes)
     std::vector<int> expected_keys_output(2 * keys_input1.size());
     std::iota(expected_keys_output.begin(), expected_keys_output.end(), 0);
 
-    int* d_keys_input1 = nullptr;
-    int* d_keys_output = nullptr;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&d_keys_input1,
-                                                 keys_input1.size() * sizeof(keys_input1[0])));
-    HIP_CHECK(
-        test_common_utils::hipMallocHelper(&d_keys_output,
-                                           expected_keys_output.size() * sizeof(keys_input1[0])));
+    common::device_ptr<int> d_keys_input1(keys_input1);
+    common::device_ptr<int> d_keys_output(expected_keys_output.size());
 
-    HIP_CHECK(hipMemcpy(d_keys_input1,
-                        keys_input1.data(),
-                        keys_input1.size() * sizeof(keys_input1[0]),
-                        hipMemcpyHostToDevice));
-
-    const auto d_keys_input2 = rocprim::make_transform_iterator(rocprim::make_counting_iterator(0),
-                                                                [] __host__ __device__(int value)
-                                                                { return value * 2 + 1; });
+    const auto d_keys_input2
+        = rocprim::make_transform_iterator(rocprim::make_counting_iterator(0),
+                                           [](int value) { return value * 2 + 1; });
 
     static constexpr bool debug_synchronous = false;
 
+    hipStream_t stream = 0; // default
+    if (UseGraphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
+
     size_t temp_storage_size_bytes = 0;
-    HIP_CHECK(rocprim::merge(nullptr,
-                             temp_storage_size_bytes,
-                             d_keys_input1,
-                             d_keys_input2,
-                             d_keys_output,
-                             keys_input1.size(),
-                             keys_input1.size(),
-                             rocprim::less<int>{},
-                             hipStreamDefault,
-                             debug_synchronous));
+    HIP_CHECK(rocprim::merge<config>(nullptr,
+                                     temp_storage_size_bytes,
+                                     d_keys_input1.get(),
+                                     d_keys_input2,
+                                     d_keys_output.get(),
+                                     keys_input1.size(),
+                                     keys_input1.size(),
+                                     rocprim::less<int>{},
+                                     stream,
+                                     debug_synchronous));
 
     ASSERT_GT(temp_storage_size_bytes, 0);
 
-    void* d_temp_storage = nullptr;
-    HIP_CHECK(test_common_utils::hipMallocHelper(&d_temp_storage, temp_storage_size_bytes));
+    common::device_ptr<void> d_temp_storage(temp_storage_size_bytes);
 
-    HIP_CHECK(rocprim::merge(d_temp_storage,
-                             temp_storage_size_bytes,
-                             d_keys_input1,
-                             d_keys_input2,
-                             d_keys_output,
-                             keys_input1.size(),
-                             keys_input1.size(),
-                             rocprim::less<int>{},
-                             hipStreamDefault,
-                             debug_synchronous));
+    test_utils::GraphHelper gHelper;
+    if(UseGraphs)
+    {
+        gHelper.startStreamCapture(stream);
+    }
 
-    std::vector<int> keys_output(expected_keys_output.size());
-    HIP_CHECK(hipMemcpy(keys_output.data(),
-                        d_keys_output,
-                        keys_output.size() * sizeof(keys_output[0]),
-                        hipMemcpyDeviceToHost));
+    HIP_CHECK(rocprim::merge<config>(d_temp_storage.get(),
+                                     temp_storage_size_bytes,
+                                     d_keys_input1.get(),
+                                     d_keys_input2,
+                                     d_keys_output.get(),
+                                     keys_input1.size(),
+                                     keys_input1.size(),
+                                     rocprim::less<int>{},
+                                     hipStreamDefault,
+                                     debug_synchronous));
+
+    if(UseGraphs)
+    {
+        gHelper.createAndLaunchGraph(stream);
+    }
+
+    const auto keys_output = d_keys_output.load();
 
     ASSERT_NO_FATAL_FAILURE(test_utils::assert_eq(keys_output, expected_keys_output));
 
-    HIP_CHECK(hipFree(d_temp_storage));
-    HIP_CHECK(hipFree(d_keys_output));
-    HIP_CHECK(hipFree(d_keys_input1));
+    if (UseGraphs)
+    {
+        gHelper.cleanupGraphHelper();
+        HIP_CHECK(hipStreamDestroy(stream));
+    }
+}
+
+TEST(RocprimDeviceMergeTests, MergeMismatchedIteratorTypes)
+{
+    testMergeMismatchedIteratorTypes<false, DefaultConfig>();
+}
+
+TEST(RocprimDeviceMergeTests, MergeMismatchedIteratorTypesWithGraphs)
+{
+    testMergeMismatchedIteratorTypes<true, DefaultConfig>();
 }
