@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -40,7 +40,8 @@ template<
     // Tests output iterator with void value_type (OutputIterator concept)
     // Segmented scan primitives which use head flags do not support this kind
     // of output iterators.
-    bool UseIdentityIterator = false
+    bool UseIdentityIterator = false,
+    bool UseGraphs = false
 >
 struct params
 {
@@ -51,6 +52,7 @@ struct params
     static constexpr unsigned int min_segment_length = MinSegmentLength;
     static constexpr unsigned int max_segment_length = MaxSegmentLength;
     static constexpr bool use_identity_iterator = UseIdentityIterator;
+    static constexpr bool use_graphs = UseGraphs;
 };
 
 template<class Params>
@@ -68,15 +70,17 @@ using bfloat16       = rocprim::bfloat16;
 typedef ::testing::Types<
     params<unsigned char, unsigned int, rocprim::plus<unsigned int>>,
     params<int, int, rocprim::plus<int>, -100, 0, 10000>,
+    params<int8_t, int8_t, rocprim::plus<int8_t>, -100, 0, 10000>,
     params<custom_double2, custom_double2, rocprim::minimum<custom_double2>, 1000, 0, 10000>,
     params<custom_int2, custom_short2, rocprim::maximum<custom_int2>, 10, 1000, 10000>,
-    params<float, double, rocprim::maximum<double>, 50, 2, 10>,
+    params<double, double, rocprim::maximum<double>, 50, 2, 10>,
     params<float, float, rocprim::plus<float>, 123, 100, 200, true>,
     params<bfloat16, float, rocprim::plus<bfloat16>, 0, 3, 50, true>,
     params<bfloat16, bfloat16, rocprim::minimum<bfloat16>, 0, 1000, 30000>,
     params<half, float, rocprim::plus<float>, 0, 10, 200, true>,
     params<half, half, rocprim::minimum<half>, 0, 1000, 30000>,
-    params<unsigned char, long long, rocprim::plus<int>, 10, 3000, 4000>>
+    params<unsigned char, long long, rocprim::plus<int>, 10, 3000, 4000>,
+    params<int, int, ::rocprim::plus<int>, 0, 0, 1000, false, true>>
     Params;
 
 TYPED_TEST_SUITE(RocprimDeviceSegmentedScan, Params);
@@ -107,6 +111,11 @@ TYPED_TEST(RocprimDeviceSegmentedScan, InclusiveScan)
     );
 
     hipStream_t stream = 0; // default stream
+    if (TestFixture::params::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
@@ -186,22 +195,28 @@ TYPED_TEST(RocprimDeviceSegmentedScan, InclusiveScan)
             HIP_CHECK(hipDeviceSynchronize());
 
             size_t temporary_storage_bytes;
-            HIP_CHECK(
-                rocprim::segmented_inclusive_scan(
-                    nullptr, temporary_storage_bytes,
-                    d_values_input,
-                    test_utils::wrap_in_identity_iterator<use_identity_iterator>(d_values_output),
-                    segments_count,
-                    d_offsets, d_offsets + 1,
-                    scan_op,
-                    stream, debug_synchronous
-                )
-            );
+            HIP_CHECK(rocprim::segmented_inclusive_scan(
+                nullptr,
+                temporary_storage_bytes,
+                d_values_input,
+                test_utils::wrap_in_identity_iterator<use_identity_iterator>(d_values_output),
+                segments_count,
+                d_offsets,
+                d_offsets + 1,
+                scan_op,
+                stream,
+                debug_synchronous));
 
             ASSERT_GT(temporary_storage_bytes, 0);
             void * d_temporary_storage;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
+            test_utils::GraphHelper gHelper;
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.startStreamCapture(stream);
+            }
+
             HIP_CHECK(
                 rocprim::segmented_inclusive_scan(
                     d_temporary_storage, temporary_storage_bytes,
@@ -213,6 +228,13 @@ TYPED_TEST(RocprimDeviceSegmentedScan, InclusiveScan)
                     stream, debug_synchronous
                 )
             );
+
+            
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.createAndLaunchGraph(stream, true, false);
+            }
+
             HIP_CHECK(hipDeviceSynchronize());
 
             std::vector<output_type> values_output(size);
@@ -232,9 +254,18 @@ TYPED_TEST(RocprimDeviceSegmentedScan, InclusiveScan)
             HIP_CHECK(hipFree(d_values_input));
             HIP_CHECK(hipFree(d_offsets));
             HIP_CHECK(hipFree(d_values_output));
+
+            if(TestFixture::params::use_graphs)
+            {
+               gHelper.cleanupGraphHelper();
+            }
         }
     }
 
+    if(TestFixture::params::use_graphs)
+    {
+        HIP_CHECK(hipStreamDestroy(stream));
+    }
 }
 
 TYPED_TEST(RocprimDeviceSegmentedScan, ExclusiveScan)
@@ -265,6 +296,11 @@ TYPED_TEST(RocprimDeviceSegmentedScan, ExclusiveScan)
     );
 
     hipStream_t stream = 0; // default stream
+    if (TestFixture::params::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
@@ -355,12 +391,19 @@ TYPED_TEST(RocprimDeviceSegmentedScan, ExclusiveScan)
                     stream, debug_synchronous
                 )
             );
+
             HIP_CHECK(hipDeviceSynchronize());
 
             ASSERT_GT(temporary_storage_bytes, 0);
             void * d_temporary_storage;
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temporary_storage, temporary_storage_bytes));
 
+            test_utils::GraphHelper gHelper;
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.startStreamCapture(stream);
+            }
+
             HIP_CHECK(
                 rocprim::segmented_exclusive_scan(
                     d_temporary_storage, temporary_storage_bytes,
@@ -372,6 +415,13 @@ TYPED_TEST(RocprimDeviceSegmentedScan, ExclusiveScan)
                     stream, debug_synchronous
                 )
             );
+
+            
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.createAndLaunchGraph(stream, true, false);
+            }
+
             HIP_CHECK(hipDeviceSynchronize());
 
             std::vector<output_type> values_output(size);
@@ -391,9 +441,18 @@ TYPED_TEST(RocprimDeviceSegmentedScan, ExclusiveScan)
             HIP_CHECK(hipFree(d_values_input));
             HIP_CHECK(hipFree(d_offsets));
             HIP_CHECK(hipFree(d_values_output));
+
+            if(TestFixture::params::use_graphs)
+            {
+               gHelper.cleanupGraphHelper();
+            }
         }
     }
 
+    if(TestFixture::params::use_graphs)
+    {
+        HIP_CHECK(hipStreamDestroy(stream));
+    }
 }
 
 TYPED_TEST(RocprimDeviceSegmentedScan, InclusiveScanUsingHeadFlags)
@@ -415,6 +474,11 @@ TYPED_TEST(RocprimDeviceSegmentedScan, InclusiveScanUsingHeadFlags)
     scan_op_type scan_op;
 
     hipStream_t stream = 0; // default stream
+    if (TestFixture::params::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
@@ -512,6 +576,7 @@ TYPED_TEST(RocprimDeviceSegmentedScan, InclusiveScanUsingHeadFlags)
                     debug_synchronous
                 )
             );
+
             HIP_CHECK(hipDeviceSynchronize());
 
             // temp_storage_size_bytes must be >0
@@ -522,6 +587,12 @@ TYPED_TEST(RocprimDeviceSegmentedScan, InclusiveScanUsingHeadFlags)
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temp_storage, temp_storage_size_bytes));
             HIP_CHECK(hipDeviceSynchronize());
 
+            test_utils::GraphHelper gHelper;
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.startStreamCapture(stream);
+            }
+
             // Run
             HIP_CHECK(
                 rocprim::segmented_inclusive_scan(
@@ -531,6 +602,13 @@ TYPED_TEST(RocprimDeviceSegmentedScan, InclusiveScanUsingHeadFlags)
                     debug_synchronous
                 )
             );
+
+            
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.createAndLaunchGraph(stream, true, false);
+            }
+
             HIP_CHECK(hipDeviceSynchronize());
 
             // Check if output values are as expected
@@ -550,9 +628,18 @@ TYPED_TEST(RocprimDeviceSegmentedScan, InclusiveScanUsingHeadFlags)
             HIP_CHECK(hipFree(d_input));
             HIP_CHECK(hipFree(d_flags));
             HIP_CHECK(hipFree(d_output));
+
+            if(TestFixture::params::use_graphs)
+            {
+               gHelper.cleanupGraphHelper();
+            }
         }
     }
 
+    if(TestFixture::params::use_graphs)
+    {
+        HIP_CHECK(hipStreamDestroy(stream));
+    }
 }
 
 TYPED_TEST(RocprimDeviceSegmentedScan, ExclusiveScanUsingHeadFlags)
@@ -576,6 +663,11 @@ TYPED_TEST(RocprimDeviceSegmentedScan, ExclusiveScanUsingHeadFlags)
     scan_op_type scan_op;
 
     hipStream_t stream = 0; // default stream
+    if (TestFixture::params::use_graphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
@@ -673,6 +765,7 @@ TYPED_TEST(RocprimDeviceSegmentedScan, ExclusiveScanUsingHeadFlags)
                     input.size(), scan_op, stream, debug_synchronous
                 )
             );
+
             HIP_CHECK(hipDeviceSynchronize());
 
             // temp_storage_size_bytes must be >0
@@ -683,6 +776,12 @@ TYPED_TEST(RocprimDeviceSegmentedScan, ExclusiveScanUsingHeadFlags)
             HIP_CHECK(test_common_utils::hipMallocHelper(&d_temp_storage, temp_storage_size_bytes));
             HIP_CHECK(hipDeviceSynchronize());
 
+            test_utils::GraphHelper gHelper;
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.startStreamCapture(stream);
+            }
+
             // Run
             HIP_CHECK(
                 rocprim::segmented_exclusive_scan(
@@ -691,6 +790,13 @@ TYPED_TEST(RocprimDeviceSegmentedScan, ExclusiveScanUsingHeadFlags)
                     input.size(), scan_op, stream, debug_synchronous
                 )
             );
+
+            
+            if(TestFixture::params::use_graphs)
+            {
+                gHelper.createAndLaunchGraph(stream, true, false);
+            }
+
             HIP_CHECK(hipDeviceSynchronize());
 
             // Check if output values are as expected
@@ -702,6 +808,12 @@ TYPED_TEST(RocprimDeviceSegmentedScan, ExclusiveScanUsingHeadFlags)
                     hipMemcpyDeviceToHost
                 )
             );
+
+            if(TestFixture::params::use_graphs)
+            {
+               gHelper.cleanupGraphHelper();
+            }
+
             HIP_CHECK(hipDeviceSynchronize());
 
             ASSERT_NO_FATAL_FAILURE(test_utils::assert_near(output, expected, precision));
@@ -713,4 +825,8 @@ TYPED_TEST(RocprimDeviceSegmentedScan, ExclusiveScanUsingHeadFlags)
         }
     }
 
+    if(TestFixture::params::use_graphs)
+    {
+        HIP_CHECK(hipStreamDestroy(stream));
+    }
 }
