@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -35,7 +35,8 @@ template<
     class InputType,
     class OutputType = InputType,
     bool UseIdentityIterator = false,
-    unsigned int SizeLimit = ROCPRIM_GRID_SIZE_LIMIT
+    unsigned int SizeLimit = ROCPRIM_GRID_SIZE_LIMIT,
+    bool UseGraphs = false
 >
 struct DeviceTransformParams
 {
@@ -43,6 +44,7 @@ struct DeviceTransformParams
     using output_type = OutputType;
     static constexpr bool use_identity_iterator = UseIdentityIterator;
     static constexpr size_t size_limit = SizeLimit;
+    static constexpr bool use_graphs = UseGraphs;
 };
 
 // ---------------------------------------------------------
@@ -58,29 +60,30 @@ public:
     static constexpr bool use_identity_iterator = Params::use_identity_iterator;
     static constexpr bool debug_synchronous = false;
     static constexpr size_t size_limit = Params::size_limit;
+    static constexpr bool use_graphs = Params::use_graphs;
 };
 
 using custom_short2  = test_utils::custom_test_type<short>;
 using custom_int2    = test_utils::custom_test_type<int>;
 using custom_double2 = test_utils::custom_test_type<double>;
 
-typedef ::testing::Types<
-    DeviceTransformParams<int, int, true>,
-    DeviceTransformParams<int8_t, int8_t>,
-    DeviceTransformParams<uint8_t, uint8_t>,
-    DeviceTransformParams<rocprim::half, rocprim::half>,
-    DeviceTransformParams<rocprim::bfloat16, rocprim::bfloat16>,
-    DeviceTransformParams<unsigned long>,
-    DeviceTransformParams<short, int, true>,
-    DeviceTransformParams<custom_short2, custom_int2, true>,
-    DeviceTransformParams<int, float>,
-    DeviceTransformParams<custom_double2, custom_double2>,
-    DeviceTransformParams<int, int, false, 512>,
-    DeviceTransformParams<float, float, false, 2048>,
-    DeviceTransformParams<int, int, false, 4096>,
-    DeviceTransformParams<int, int, false, 2097152>,
-    DeviceTransformParams<int, int, false, 1073741824>
-> RocprimDeviceTransformTestsParams;
+typedef ::testing::Types<DeviceTransformParams<int, int, true>,
+                         DeviceTransformParams<int8_t, int8_t>,
+                         DeviceTransformParams<uint8_t, uint8_t>,
+                         DeviceTransformParams<rocprim::half, rocprim::half>,
+                         DeviceTransformParams<rocprim::bfloat16, rocprim::bfloat16>,
+                         DeviceTransformParams<unsigned long>,
+                         DeviceTransformParams<short, int, true>,
+                         DeviceTransformParams<custom_short2, custom_int2, true>,
+                         DeviceTransformParams<int, float>,
+                         DeviceTransformParams<custom_double2, custom_double2>,
+                         DeviceTransformParams<int, int, false, 512>,
+                         DeviceTransformParams<float, float, false, 2048>,
+                         DeviceTransformParams<double, double, false, 4096>,
+                         DeviceTransformParams<int, int, false, 2097152>,
+                         DeviceTransformParams<int, int, false, 1073741824>,
+                         DeviceTransformParams<int, int, false, ROCPRIM_GRID_SIZE_LIMIT, true>>
+    RocprimDeviceTransformTestsParams;
 
 template <unsigned int SizeLimit>
 struct size_limit_config {
@@ -116,7 +119,6 @@ TYPED_TEST(RocprimDeviceTransformTests, Transform)
     using T = typename TestFixture::input_type;
     using U = typename TestFixture::output_type;
     static constexpr bool use_identity_iterator = TestFixture::use_identity_iterator;
-    const bool debug_synchronous = TestFixture::debug_synchronous;
     using Config = size_limit_config_t<TestFixture::size_limit>;
 
     for (size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
@@ -127,6 +129,11 @@ TYPED_TEST(RocprimDeviceTransformTests, Transform)
         for(auto size : test_utils::get_sizes(seed_value))
         {
             hipStream_t stream = 0; // default
+            if (TestFixture::use_graphs)
+            {
+                // Default stream does not support hipGraph stream capture, so create one
+                HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+            }
 
             SCOPED_TRACE(testing::Message() << "with size = " << size);
 
@@ -151,14 +158,27 @@ TYPED_TEST(RocprimDeviceTransformTests, Transform)
             std::vector<U> expected(input.size());
             std::transform(input.begin(), input.end(), expected.begin(), transform<U>());
 
+            test_utils::GraphHelper gHelper;
+            if(TestFixture::use_graphs)
+            {
+                gHelper.startStreamCapture(stream);
+            }
+
             // Run
             HIP_CHECK(
                 rocprim::transform<Config>(
                     d_input,
                     test_utils::wrap_in_identity_iterator<use_identity_iterator>(d_output),
-                    input.size(), transform<U>(), stream, debug_synchronous
+                    input.size(), transform<U>(), stream, TestFixture::debug_synchronous
                 )
             );
+
+            
+            if(TestFixture::use_graphs)
+            {
+                gHelper.createAndLaunchGraph(stream, true, false);
+            }
+
             HIP_CHECK(hipGetLastError());
             HIP_CHECK(hipDeviceSynchronize());
 
@@ -176,11 +196,16 @@ TYPED_TEST(RocprimDeviceTransformTests, Transform)
             ASSERT_NO_FATAL_FAILURE(
                 test_utils::assert_near(output, expected, test_utils::precision<U>));
 
-            hipFree(d_input);
-            hipFree(d_output);
+            HIP_CHECK(hipFree(d_input));
+            HIP_CHECK(hipFree(d_output));
+
+            if (TestFixture::use_graphs)
+            {
+                 gHelper.cleanupGraphHelper();
+                HIP_CHECK(hipStreamDestroy(stream));
+            }
         }
     }
-
 }
 
 template<class T1, class T2, class U>
@@ -214,6 +239,11 @@ TYPED_TEST(RocprimDeviceTransformTests, BinaryTransform)
         for(auto size : test_utils::get_sizes(seed_value))
         {
             hipStream_t stream = 0; // default
+            if (TestFixture::use_graphs)
+            {
+                // Default stream does not support hipGraph stream capture, so create one
+                HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+            }
 
             SCOPED_TRACE(testing::Message() << "with size = " << size);
 
@@ -251,6 +281,12 @@ TYPED_TEST(RocprimDeviceTransformTests, BinaryTransform)
                 expected.begin(), binary_transform<T1, T2, U>()
             );
 
+            test_utils::GraphHelper gHelper;
+            if(TestFixture::use_graphs)
+            {
+                gHelper.startStreamCapture(stream);
+            }
+
             // Run
             HIP_CHECK(
                 rocprim::transform<Config>(
@@ -259,6 +295,13 @@ TYPED_TEST(RocprimDeviceTransformTests, BinaryTransform)
                     input1.size(), binary_transform<T1, T2, U>(), stream, debug_synchronous
                 )
             );
+
+            
+            if(TestFixture::use_graphs)
+            {
+                gHelper.createAndLaunchGraph(stream, true, false);
+            }
+
             HIP_CHECK(hipGetLastError());
             HIP_CHECK(hipDeviceSynchronize());
 
@@ -276,15 +319,43 @@ TYPED_TEST(RocprimDeviceTransformTests, BinaryTransform)
             ASSERT_NO_FATAL_FAILURE(
                 test_utils::assert_near(output, expected, test_utils::precision<U>));
 
-            hipFree(d_input1);
-            hipFree(d_input2);
-            hipFree(d_output);
+            HIP_CHECK(hipFree(d_input1));
+            HIP_CHECK(hipFree(d_input2));
+            HIP_CHECK(hipFree(d_output));
+
+            if (TestFixture::use_graphs)
+            {
+                 gHelper.cleanupGraphHelper();
+                HIP_CHECK(hipStreamDestroy(stream));
+            }
         }
     }
-
 }
 
-TEST(RocprimDeviceTransformTests, LargeIndices)
+template<class T>
+struct flag_expected_op_t
+{
+    bool* d_flag;
+    T     expected;
+    T     expected_above_limit;
+
+    __device__
+    auto  operator()(const T& value) -> int
+    {
+        if(value == expected)
+        {
+            d_flag[0] = true;
+        }
+        if(value == expected_above_limit)
+        {
+            d_flag[1] = true;
+        }
+        return 0;
+    }
+};
+
+template<bool UseGraphs = false>
+void testLargeIndices()
 {
     const int device_id = test_common_utils::obtain_device_from_ctest();
     SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
@@ -295,7 +366,12 @@ TEST(RocprimDeviceTransformTests, LargeIndices)
     using OutputIterator         = rocprim::discard_iterator;
     const bool debug_synchronous = false;
 
-    const hipStream_t stream = 0; // default
+    hipStream_t stream = 0; // default
+    if (UseGraphs)
+    {
+        // Default stream does not support hipGraph stream capture, so create one
+        HIP_CHECK(hipStreamCreateWithFlags(&stream, hipStreamNonBlocking));
+    }
 
     for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
     {
@@ -324,21 +400,25 @@ TEST(RocprimDeviceTransformTests, LargeIndices)
             SCOPED_TRACE(testing::Message() << "expected = " << expected);
             SCOPED_TRACE(testing::Message() << "expected_above_limit = " << expected_above_limit);
 
-            const auto flag_expected = [=] __device__ (T value) -> int {
-                if(value == expected)
-                {
-                    d_flag[0] = true;
-                }
-                if(value == expected_above_limit)
-                {
-                    d_flag[1] = true;
-                }
-                return 0;
-            };
+            const auto flag_expected
+                = flag_expected_op_t<T>{d_flag, expected, expected_above_limit};
+
+            test_utils::GraphHelper gHelper;
+            if(UseGraphs)
+            {
+                gHelper.startStreamCapture(stream);
+            }
 
             // Run
             HIP_CHECK(
                 rocprim::transform(input, output, size, flag_expected, stream, debug_synchronous));
+
+            
+            if(UseGraphs)
+            {
+                gHelper.createAndLaunchGraph(stream, true, false);
+            }
+
             HIP_CHECK(hipGetLastError());
             HIP_CHECK(hipDeviceSynchronize());
 
@@ -348,8 +428,26 @@ TEST(RocprimDeviceTransformTests, LargeIndices)
 
             ASSERT_TRUE(flags[0]);
             ASSERT_TRUE(flags[1]);
+            
+            HIP_CHECK(hipFree(d_flag));
 
-            hipFree(d_flag);
+            if (UseGraphs)
+                 gHelper.cleanupGraphHelper();
         }
     }
+
+    if(UseGraphs)
+    {
+        HIP_CHECK(hipStreamDestroy(stream));
+    }
+}
+
+TEST(RocprimDeviceTransformTests, LargeIndices)
+{
+    testLargeIndices();
+}
+
+TEST(RocprimDeviceTransformTests, LargeIndicesWithGraphs)
+{
+    testLargeIndices<true>();
 }
