@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,11 +21,13 @@
 #ifndef TEST_TEST_UTILS_HPP_
 #define TEST_TEST_UTILS_HPP_
 
-#include <rocprim/types.hpp>
+#include <rocprim/device/config_types.hpp>
 #include <rocprim/functional.hpp>
 #include <rocprim/intrinsics.hpp>
 #include <rocprim/type_traits.hpp>
-#include <rocprim/detail/match_result_type.hpp>
+#include <rocprim/types.hpp>
+
+#include "../common_test_header.hpp"
 
 // Identity iterator
 #include "identity_iterator.hpp"
@@ -39,21 +41,7 @@
 #include "test_utils_custom_test_types.hpp"
 #include "test_utils_data_generation.hpp"
 #include "test_utils_assertions.hpp"
-
-// Helper macros to disable warnings in clang
-#ifdef __clang__
-#define ROCPRIM_PRAGMA_TO_STR(x) _Pragma(#x)
-#define ROCPRIM_CLANG_SUPPRESS_WARNING_PUSH _Pragma("clang diagnostic push")
-#define ROCPRIM_CLANG_SUPPRESS_WARNING(w) ROCPRIM_PRAGMA_TO_STR(clang diagnostic ignored w)
-#define ROCPRIM_CLANG_SUPPRESS_WARNING_POP _Pragma("clang diagnostic pop")
-#define ROCPRIM_CLANG_SUPPRESS_WARNING_WITH_PUSH(w) \
-    ROCPRIM_CLANG_SUPPRESS_WARNING_PUSH ROCPRIM_CLANG_SUPPRESS_WARNING(w)
-#else // __clang__
-#define ROCPRIM_CLANG_SUPPRESS_WARNING_PUSH
-#define ROCPRIM_CLANG_SUPPRESS_WARNING(w)
-#define ROCPRIM_CLANG_SUPPRESS_WARNING_POP
-#define ROCPRIM_CLANG_SUPPRESS_WARNING_WITH_PUSH(w)
-#endif // __clang__
+#include "test_utils_hipgraphs.hpp"
 
 namespace test_utils
 {
@@ -139,27 +127,21 @@ template<typename T>
 struct select_plus_operator_host
 {
     typedef ::rocprim::plus<T> type;
-    typedef T acc_type;
-    // #156 temporarily disable half test due to known issue with converting from double to half
-    //      cast_type is the type that should be used to cast acc_type to T.
-    //      overload needed temporarily due to compiler bug in half conversions
-    typedef T cast_type;
+    typedef T                  acc_type;
 };
 
 template<>
 struct select_plus_operator_host<::rocprim::half>
 {
     typedef ::rocprim::plus<double> type;
-    typedef double acc_type;
-    typedef float                   cast_type;
+    typedef double                  acc_type;
 };
 
 template<>
 struct select_plus_operator_host<::rocprim::bfloat16>
 {
     typedef ::rocprim::plus<double> type;
-    typedef double acc_type;
-    typedef ::rocprim::bfloat16     cast_type;
+    typedef double                  acc_type;
 };
 
 template<class InputIt, class T,
@@ -432,24 +414,77 @@ void iota(ForwardIt first, ForwardIt last, T value)
     }
 }
 
-#define SKIP_IF_UNSUPPORTED_WARP_SIZE(test_warp_size) { \
-    const auto host_warp_size = ::rocprim::host_warp_size(); \
-    if (host_warp_size < (test_warp_size)) \
-    { \
-        GTEST_SKIP() << "Cannot run test of warp size " \
-            << (test_warp_size) \
-            << " on a device with warp size " \
-            << host_warp_size; \
-    } \
+// Like test_utils::iota but applies module 'ubound' to the values generated.
+template<class ForwardIt,
+         class T,
+         typename std::enable_if<!std::is_same<typename std::iterator_traits<ForwardIt>::value_type,
+                                               rocprim::half>::value,
+                                 bool>::type
+         = false>
+void iota_modulo(ForwardIt first, ForwardIt last, T lbound, const size_t ubound)
+{
+    const T value_mod = static_cast<size_t>(lbound) < ubound ? lbound : 0;
+    using value_type  = typename std::iterator_traits<ForwardIt>::value_type;
+
+    for(T value = value_mod; first != last; value++, *first++)
+    {
+        if(static_cast<size_t>(value) >= ubound)
+        {
+            value = value_mod;
+        }
+        *first = static_cast<value_type>(value);
+    }
 }
 
-template<unsigned int LogicalWarpSize>
-struct DeviceSelectWarpSize
+// Necessary because for rocprim::half even though lbound < ubound it gets cast as a greater
+// value, as precision is bigger for values closer to the maximum.
+template<class ForwardIt,
+         class T,
+         typename std::enable_if<std::is_same<typename std::iterator_traits<ForwardIt>::value_type,
+                                              rocprim::half>::value,
+                                 bool>::type
+         = true>
+void iota_modulo(ForwardIt first, ForwardIt last, T lbound, const size_t ubound)
 {
-    static constexpr unsigned value = ::rocprim::device_warp_size() >= LogicalWarpSize
-        ? LogicalWarpSize
-        : ::rocprim::device_warp_size();
-};
+    const T value_mod = static_cast<size_t>(lbound) < ubound ? lbound : 0;
+    using value_type  = rocprim::half;
+
+    for(T value = value_mod; first != last; value++, *first++)
+    {
+        if(static_cast<float>(static_cast<value_type>(value)) >= ubound)
+        {
+            value = value_mod;
+        }
+        *first = static_cast<value_type>(value);
+    }
+}
+
+#define SKIP_IF_UNSUPPORTED_WARP_SIZE(test_warp_size, device_id)                \
+    {                                                                           \
+        unsigned int host_warp_size;                                            \
+        HIP_CHECK(::rocprim::host_warp_size(device_id, host_warp_size));        \
+        if(host_warp_size < (test_warp_size))                                   \
+        {                                                                       \
+            GTEST_SKIP() << "Cannot run test of warp size " << (test_warp_size) \
+                         << " on a device with warp size " << host_warp_size;   \
+        }                                                                       \
+    }
+
+template<unsigned int LogicalWarpSize>
+__device__ constexpr bool device_test_enabled_for_warp_size_v
+    = ::rocprim::arch::wavefront::min_size() >= LogicalWarpSize;
+
+template<bool MakeConst, typename T>
+inline auto wrap_in_const(T* ptr) -> typename std::enable_if_t<MakeConst, const T*>
+{
+    return ptr;
+}
+
+template<bool MakeConst, typename T>
+inline auto wrap_in_const(T* ptr) -> typename std::enable_if_t<!MakeConst, T*>
+{
+    return ptr;
+}
 
 } // end test_utils namespace
 
