@@ -1,6 +1,6 @@
 // MIT License
 //
-// Copyright (c) 2017-2022 Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2017-2024 Advanced Micro Devices, Inc. All rights reserved.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -35,7 +35,6 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScan)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
-    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
     static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
@@ -56,7 +55,8 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScan)
             ? rocprim::max<size_t>(ws64, logical_warp_size * 4)
             : rocprim::max<size_t>((ws64/logical_warp_size), 1) * logical_warp_size;
 
-    const unsigned int current_device_warp_size = rocprim::host_warp_size();
+    unsigned int current_device_warp_size;
+    HIP_CHECK(::rocprim::host_warp_size(device_id, current_device_warp_size));
 
     const size_t block_size = current_device_warp_size == ws32 ? block_size_ws32 : block_size_ws64;
     const unsigned int grid_size = 4;
@@ -66,7 +66,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScan)
     if( (logical_warp_size > current_device_warp_size) ||
         (current_device_warp_size != ws32 && current_device_warp_size != ws64) ) // Only WarpSize 32 and 64 is supported
     {
-        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %d.    Skipping test\n",
+        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %u.    Skipping test\n",
             logical_warp_size, block_size, current_device_warp_size);
         GTEST_SKIP();
     }
@@ -89,7 +89,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScan)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator = binary_op_host(input[idx], accumulator);
-                expected[idx] = static_cast<cast_type>(accumulator);
+                expected[idx] = static_cast<T>(accumulator);
             }
         }
 
@@ -157,7 +157,6 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanReduce)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
-    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
     static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
@@ -178,7 +177,8 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanReduce)
             ? rocprim::max<size_t>(ws64, logical_warp_size * 4)
             : rocprim::max<size_t>((ws64/logical_warp_size), 1) * logical_warp_size;
 
-    const unsigned int current_device_warp_size = rocprim::host_warp_size();
+    unsigned int current_device_warp_size;
+    HIP_CHECK(::rocprim::host_warp_size(device_id, current_device_warp_size));
 
     const size_t block_size = current_device_warp_size == ws32 ? block_size_ws32 : block_size_ws64;
     const unsigned int grid_size = 4;
@@ -188,7 +188,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanReduce)
     if( (logical_warp_size > current_device_warp_size) ||
         (current_device_warp_size != ws32 && current_device_warp_size != ws64) ) // Only WarpSize 32 and 64 is supported
     {
-        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %d.    Skipping test\n",
+        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %u.    Skipping test\n",
             logical_warp_size, block_size, current_device_warp_size);
         GTEST_SKIP();
     }
@@ -213,7 +213,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanReduce)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator = binary_op_host(input[idx],accumulator);
-                expected[idx] = static_cast<cast_type>(accumulator);
+                expected[idx] = static_cast<T>(accumulator);
             }
             expected_reductions[i] = expected[(i+1) * logical_warp_size - 1];
         }
@@ -301,7 +301,6 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveScan)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
-    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
     static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
@@ -322,7 +321,8 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveScan)
             ? rocprim::max<size_t>(ws64, logical_warp_size * 4)
             : rocprim::max<size_t>((ws64/logical_warp_size), 1) * logical_warp_size;
 
-    const unsigned int current_device_warp_size = rocprim::host_warp_size();
+    unsigned int current_device_warp_size;
+    HIP_CHECK(::rocprim::host_warp_size(device_id, current_device_warp_size));
 
     const size_t block_size = current_device_warp_size == ws32 ? block_size_ws32 : block_size_ws64;
     const unsigned int grid_size = 4;
@@ -332,7 +332,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveScan)
     if( (logical_warp_size > current_device_warp_size) ||
         (current_device_warp_size != ws32 && current_device_warp_size != ws64) ) // Only WarpSize 32 and 64 is supported
     {
-        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %d.    Skipping test\n",
+        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %u.    Skipping test\n",
             logical_warp_size, block_size, current_device_warp_size);
         GTEST_SKIP();
     }
@@ -357,7 +357,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveScan)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator = binary_op_host(input[idx-1], accumulator);
-                expected[idx] = static_cast<cast_type>(accumulator);
+                expected[idx] = static_cast<T>(accumulator);
             }
         }
 
@@ -414,6 +414,148 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveScan)
 
 }
 
+typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveScanWoInit)
+{
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
+    using T = typename TestFixture::params::type;
+    // for bfloat16 and half we use double for host-side accumulation
+    using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
+    binary_op_type_host binary_op_host;
+    using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+
+    // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
+    static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
+
+    // The different warp sizes
+    static constexpr size_t ws32{ROCPRIM_WARP_SIZE_32};
+    static constexpr size_t ws64{ROCPRIM_WARP_SIZE_64};
+
+    // Block size of warp size 32
+    static constexpr size_t block_size_ws32
+        = rocprim::detail::is_power_of_two(logical_warp_size)
+              ? rocprim::max<size_t>(ws32, logical_warp_size * 4)
+              : rocprim::max<size_t>((ws32 / logical_warp_size), 1) * logical_warp_size;
+
+    // Block size of warp size 64
+    static constexpr size_t block_size_ws64
+        = rocprim::detail::is_power_of_two(logical_warp_size)
+              ? rocprim::max<size_t>(ws64, logical_warp_size * 4)
+              : rocprim::max<size_t>((ws64 / logical_warp_size), 1) * logical_warp_size;
+
+    unsigned int current_device_warp_size;
+    HIP_CHECK(::rocprim::host_warp_size(device_id, current_device_warp_size));
+
+    const size_t block_size = current_device_warp_size == ws32 ? block_size_ws32 : block_size_ws64;
+    const unsigned int grid_size = 4;
+    const size_t       size      = block_size * grid_size;
+
+    // Check if warp size is supported
+    if((logical_warp_size > current_device_warp_size)
+       || (current_device_warp_size != ws32
+           && current_device_warp_size != ws64)) // Only WarpSize 32 and 64 is supported
+    {
+        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: "
+               "%d.    Skipping test\n",
+               logical_warp_size,
+               block_size,
+               current_device_warp_size);
+        GTEST_SKIP();
+    }
+
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
+
+        // Generate data
+        std::vector<T> input = test_utils::get_random_data<T>(size, 2, 50, seed_value);
+        std::vector<T> output(size);
+        std::vector<T> expected(input.size(), T(0));
+
+        // Calculate expected results on host
+        for(size_t i = 0; i < input.size() / logical_warp_size; i++)
+        {
+            // expected[i * logical_warp_size] is unspecified because init is not passed
+            acc_type accumulator(input[i * logical_warp_size]);
+
+            static_assert(logical_warp_size > 2, "logical_warp_size assumed to be at least 2.");
+            expected[i * logical_warp_size + 1] = static_cast<T>(accumulator);
+
+            for(size_t j = 2; j < logical_warp_size; j++)
+            {
+                auto idx      = i * logical_warp_size + j;
+                accumulator   = binary_op_host(input[idx - 1], accumulator);
+                expected[idx] = static_cast<T>(accumulator);
+            }
+        }
+
+        // Writing to device memory
+        T* device_input;
+        HIP_CHECK(test_common_utils::hipMallocHelper(
+            &device_input,
+            input.size() * sizeof(typename decltype(input)::value_type)));
+        T* device_output;
+        HIP_CHECK(test_common_utils::hipMallocHelper(
+            &device_output,
+            output.size() * sizeof(typename decltype(output)::value_type)));
+
+        HIP_CHECK(
+            hipMemcpy(device_input, input.data(), input.size() * sizeof(T), hipMemcpyHostToDevice));
+
+        // Launching kernel
+        if(current_device_warp_size == ws32)
+        {
+            hipLaunchKernelGGL(
+                HIP_KERNEL_NAME(
+                    warp_exclusive_scan_wo_init_kernel<T, block_size_ws32, logical_warp_size>),
+                dim3(grid_size),
+                dim3(block_size_ws32),
+                0,
+                0,
+                device_input,
+                device_output);
+        }
+        else if(current_device_warp_size == ws64)
+        {
+            hipLaunchKernelGGL(
+                HIP_KERNEL_NAME(
+                    warp_exclusive_scan_wo_init_kernel<T, block_size_ws64, logical_warp_size>),
+                dim3(grid_size),
+                dim3(block_size_ws64),
+                0,
+                0,
+                device_input,
+                device_output);
+        }
+
+        HIP_CHECK(hipGetLastError());
+        HIP_CHECK(hipDeviceSynchronize());
+
+        // Read from device memory
+        HIP_CHECK(hipMemcpy(output.data(),
+                            device_output,
+                            output.size() * sizeof(T),
+                            hipMemcpyDeviceToHost));
+
+        // The first value of each logical warp has an unspecified result, expect whatever we got
+        // for those values to not fail the test.
+        for(size_t i = 0; i < input.size() / logical_warp_size; i++)
+        {
+            expected[i * logical_warp_size] = output[i * logical_warp_size];
+        }
+
+        // Validating results
+        test_utils::assert_near(output, expected, test_utils::precision<T> * logical_warp_size);
+
+        HIP_CHECK(hipFree(device_input));
+        HIP_CHECK(hipFree(device_output));
+    }
+}
+
 //typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScan)
 typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScan)
 {
@@ -426,7 +568,6 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScan)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
-    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
     static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
@@ -447,7 +588,8 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScan)
             ? rocprim::max<size_t>(ws64, logical_warp_size * 4)
             : rocprim::max<size_t>((ws64/logical_warp_size), 1) * logical_warp_size;
 
-    const unsigned int current_device_warp_size = rocprim::host_warp_size();
+    unsigned int current_device_warp_size;
+    HIP_CHECK(::rocprim::host_warp_size(device_id, current_device_warp_size));
 
     const size_t block_size = current_device_warp_size == ws32 ? block_size_ws32 : block_size_ws64;
     const unsigned int grid_size = 4;
@@ -457,7 +599,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScan)
     if( (logical_warp_size > current_device_warp_size) ||
         (current_device_warp_size != ws32 && current_device_warp_size != ws64) ) // Only WarpSize 32 and 64 is supported
     {
-        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %d.    Skipping test\n",
+        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %u.    Skipping test\n",
             logical_warp_size, block_size, current_device_warp_size);
         GTEST_SKIP();
     }
@@ -484,7 +626,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScan)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator = binary_op_host(input[idx-1], accumulator);
-                expected[idx] = static_cast<cast_type>(accumulator);
+                expected[idx] = static_cast<T>(accumulator);
             }
 
             acc_type accumulator_reductions(0);
@@ -492,7 +634,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScan)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator_reductions = binary_op_host(input[idx], accumulator_reductions);
-                expected_reductions[i] = static_cast<cast_type>(accumulator_reductions);
+                expected_reductions[i] = static_cast<T>(accumulator_reductions);
             }
         }
 
@@ -567,6 +709,174 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScan)
 
 }
 
+typed_test_def(RocprimWarpScanTests, name_suffix, ExclusiveReduceScanWoInit)
+{
+    int device_id = test_common_utils::obtain_device_from_ctest();
+    SCOPED_TRACE(testing::Message() << "with device_id = " << device_id);
+    HIP_CHECK(hipSetDevice(device_id));
+
+    using T = typename TestFixture::params::type;
+    // for bfloat16 and half we use double for host-side accumulation
+    using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
+    binary_op_type_host binary_op_host;
+    using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
+
+    // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
+    static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
+
+    // The different warp sizes
+    static constexpr size_t ws32 = size_t(ROCPRIM_WARP_SIZE_32);
+    static constexpr size_t ws64 = size_t(ROCPRIM_WARP_SIZE_64);
+
+    // Block size of warp size 32
+    static constexpr size_t block_size_ws32
+        = rocprim::detail::is_power_of_two(logical_warp_size)
+              ? rocprim::max<size_t>(ws32, logical_warp_size * 4)
+              : rocprim::max<size_t>((ws32 / logical_warp_size), 1) * logical_warp_size;
+
+    // Block size of warp size 64
+    static constexpr size_t block_size_ws64
+        = rocprim::detail::is_power_of_two(logical_warp_size)
+              ? rocprim::max<size_t>(ws64, logical_warp_size * 4)
+              : rocprim::max<size_t>((ws64 / logical_warp_size), 1) * logical_warp_size;
+
+    unsigned int current_device_warp_size;
+    HIP_CHECK(::rocprim::host_warp_size(device_id, current_device_warp_size));
+
+    const size_t block_size = current_device_warp_size == ws32 ? block_size_ws32 : block_size_ws64;
+    const unsigned int grid_size = 4;
+    const size_t       size      = block_size * grid_size;
+
+    // Check if warp size is supported
+    if((logical_warp_size > current_device_warp_size)
+       || (current_device_warp_size != ws32
+           && current_device_warp_size != ws64)) // Only WarpSize 32 and 64 is supported
+    {
+        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: "
+               "%d.    Skipping test\n",
+               logical_warp_size,
+               block_size,
+               current_device_warp_size);
+        GTEST_SKIP();
+    }
+
+    for(size_t seed_index = 0; seed_index < random_seeds_count + seed_size; seed_index++)
+    {
+        unsigned int seed_value
+            = seed_index < random_seeds_count ? rand() : seeds[seed_index - random_seeds_count];
+        SCOPED_TRACE(testing::Message() << "with seed = " << seed_value);
+
+        // Generate data
+        std::vector<T> input = test_utils::get_random_data<T>(size, 2, 50, seed_value);
+        std::vector<T> output(size);
+        std::vector<T> output_reductions(size / logical_warp_size);
+        std::vector<T> expected(input.size(), T(0));
+        std::vector<T> expected_reductions(output_reductions.size(), T(0));
+
+        // Calculate expected results on host
+        for(size_t i = 0; i < input.size() / logical_warp_size; i++)
+        {
+            // expected[i * logical_warp_size] is unspecified because init is not passed
+            acc_type accumulator(input[i * logical_warp_size]);
+
+            static_assert(logical_warp_size > 2, "logical_warp_size assumed to be at least 2.");
+            expected[i * logical_warp_size + 1] = static_cast<T>(accumulator);
+
+            for(size_t j = 2; j < logical_warp_size; j++)
+            {
+                auto idx      = i * logical_warp_size + j;
+                accumulator   = binary_op_host(input[idx - 1], accumulator);
+                expected[idx] = static_cast<T>(accumulator);
+            }
+
+            acc_type accumulator_reductions(0);
+            for(size_t j = 0; j < logical_warp_size; j++)
+            {
+                auto idx               = i * logical_warp_size + j;
+                accumulator_reductions = binary_op_host(input[idx], accumulator_reductions);
+                expected_reductions[i] = static_cast<T>(accumulator_reductions);
+            }
+        }
+
+        // Writing to device memory
+        T* device_input;
+        HIP_CHECK(test_common_utils::hipMallocHelper(
+            &device_input,
+            input.size() * sizeof(typename decltype(input)::value_type)));
+        T* device_output;
+        HIP_CHECK(test_common_utils::hipMallocHelper(
+            &device_output,
+            output.size() * sizeof(typename decltype(output)::value_type)));
+        T* device_output_reductions;
+        HIP_CHECK(test_common_utils::hipMallocHelper(
+            &device_output_reductions,
+            output_reductions.size() * sizeof(typename decltype(output_reductions)::value_type)));
+
+        HIP_CHECK(
+            hipMemcpy(device_input, input.data(), input.size() * sizeof(T), hipMemcpyHostToDevice));
+
+        // Launching kernel
+        if(current_device_warp_size == ws32)
+        {
+            hipLaunchKernelGGL(
+                HIP_KERNEL_NAME(warp_exclusive_scan_reduce_wo_init_kernel<T,
+                                                                          block_size_ws32,
+                                                                          logical_warp_size>),
+                dim3(grid_size),
+                dim3(block_size_ws32),
+                0,
+                0,
+                device_input,
+                device_output,
+                device_output_reductions);
+        }
+        else if(current_device_warp_size == ws64)
+        {
+            hipLaunchKernelGGL(
+                HIP_KERNEL_NAME(warp_exclusive_scan_reduce_wo_init_kernel<T,
+                                                                          block_size_ws64,
+                                                                          logical_warp_size>),
+                dim3(grid_size),
+                dim3(block_size_ws64),
+                0,
+                0,
+                device_input,
+                device_output,
+                device_output_reductions);
+        }
+        HIP_CHECK(hipGetLastError());
+        HIP_CHECK(hipDeviceSynchronize());
+
+        // Read from device memory
+        HIP_CHECK(hipMemcpy(output.data(),
+                            device_output,
+                            output.size() * sizeof(T),
+                            hipMemcpyDeviceToHost));
+
+        HIP_CHECK(hipMemcpy(output_reductions.data(),
+                            device_output_reductions,
+                            output_reductions.size() * sizeof(T),
+                            hipMemcpyDeviceToHost));
+
+        // The first value of each logical warp has an unspecified result, expect whatever we got
+        // for those values to not fail the test.
+        for(size_t i = 0; i < input.size() / logical_warp_size; i++)
+        {
+            expected[i * logical_warp_size] = output[i * logical_warp_size];
+        }
+
+        // Validating results
+        test_utils::assert_near(output, expected, test_utils::precision<T> * logical_warp_size);
+        test_utils::assert_near(output_reductions,
+                                expected_reductions,
+                                test_utils::precision<T> * logical_warp_size);
+
+        HIP_CHECK(hipFree(device_input));
+        HIP_CHECK(hipFree(device_output));
+        HIP_CHECK(hipFree(device_output_reductions));
+    }
+}
+
 typed_test_def(RocprimWarpScanTests, name_suffix, Scan)
 {
     int device_id = test_common_utils::obtain_device_from_ctest();
@@ -578,7 +888,6 @@ typed_test_def(RocprimWarpScanTests, name_suffix, Scan)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
-    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
     static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
@@ -599,7 +908,8 @@ typed_test_def(RocprimWarpScanTests, name_suffix, Scan)
             ? rocprim::max<size_t>(ws64, logical_warp_size * 4)
             : rocprim::max<size_t>((ws64/logical_warp_size), 1) * logical_warp_size;
 
-    const unsigned int current_device_warp_size = rocprim::host_warp_size();
+    unsigned int current_device_warp_size;
+    HIP_CHECK(::rocprim::host_warp_size(device_id, current_device_warp_size));
 
     const size_t block_size = current_device_warp_size == ws32 ? block_size_ws32 : block_size_ws64;
     const unsigned int grid_size = 4;
@@ -609,7 +919,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, Scan)
     if( (logical_warp_size > current_device_warp_size) ||
         (current_device_warp_size != ws32 && current_device_warp_size != ws64) ) // Only WarpSize 32 and 64 is supported
     {
-        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %d.    Skipping test\n",
+        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %u.    Skipping test\n",
             logical_warp_size, block_size, current_device_warp_size);
         GTEST_SKIP();
     }
@@ -637,11 +947,11 @@ typed_test_def(RocprimWarpScanTests, name_suffix, Scan)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator_inclusive = binary_op_host(input[idx], accumulator_inclusive);
-                expected_inclusive[idx] = static_cast<cast_type>(accumulator_inclusive);
+                expected_inclusive[idx] = static_cast<T>(accumulator_inclusive);
                 if(j > 0)
                 {
                     accumulator_exclusive = binary_op_host(input[idx-1], accumulator_exclusive);
-                    expected_exclusive[idx] = static_cast<cast_type>(accumulator_exclusive);
+                    expected_exclusive[idx] = static_cast<T>(accumulator_exclusive);
                 }
             }
         }
@@ -736,7 +1046,6 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ScanReduce)
     using binary_op_type_host = typename test_utils::select_plus_operator_host<T>::type;
     binary_op_type_host binary_op_host;
     using acc_type = typename test_utils::select_plus_operator_host<T>::acc_type;
-    using cast_type = typename test_utils::select_plus_operator_host<T>::cast_type;
 
     // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
     static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
@@ -757,7 +1066,8 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ScanReduce)
             ? rocprim::max<size_t>(ws64, logical_warp_size * 4)
             : rocprim::max<size_t>((ws64/logical_warp_size), 1) * logical_warp_size;
 
-    const unsigned int current_device_warp_size = rocprim::host_warp_size();
+    unsigned int current_device_warp_size;
+    HIP_CHECK(::rocprim::host_warp_size(device_id, current_device_warp_size));
 
     const size_t block_size = current_device_warp_size == ws32 ? block_size_ws32 : block_size_ws64;
     const unsigned int grid_size = 4;
@@ -767,7 +1077,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ScanReduce)
     if( (logical_warp_size > current_device_warp_size) ||
         (current_device_warp_size != ws32 && current_device_warp_size != ws64) ) // Only WarpSize 32 and 64 is supported
     {
-        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %d.    Skipping test\n",
+        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %u.    Skipping test\n",
             logical_warp_size, block_size, current_device_warp_size);
         GTEST_SKIP();
     }
@@ -797,11 +1107,11 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ScanReduce)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator_inclusive = binary_op_host(input[idx], accumulator_inclusive);
-                expected_inclusive[idx] = static_cast<cast_type>(accumulator_inclusive);
+                expected_inclusive[idx] = static_cast<T>(accumulator_inclusive);
                 if(j > 0)
                 {
                     accumulator_exclusive = binary_op_host(input[idx-1], accumulator_exclusive);
-                    expected_exclusive[idx] = static_cast<cast_type>(accumulator_exclusive);
+                    expected_exclusive[idx] = static_cast<T>(accumulator_exclusive);
                 }
             }
             expected_reductions[i] = expected_inclusive[(i+1) * logical_warp_size - 1];
@@ -902,6 +1212,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, ScanReduce)
         HIP_CHECK(hipFree(device_input));
         HIP_CHECK(hipFree(device_inclusive_output));
         HIP_CHECK(hipFree(device_exclusive_output));
+        HIP_CHECK(hipFree(device_output_reductions));
     }
 
 }
@@ -914,8 +1225,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanCustomType)
 
     using base_type = typename TestFixture::params::type;
     using T = test_utils::custom_test_type<base_type>;
-    using acc_type = typename test_utils::select_plus_operator_host<base_type>::acc_type;
-    using cast_type = typename test_utils::select_plus_operator_host<base_type>::cast_type;
+    using acc_type  = typename test_utils::select_plus_operator_host<base_type>::acc_type;
 
     // logical warp side for warp primitive, execution warp size is always rocprim::warp_size()
     static constexpr size_t logical_warp_size = TestFixture::params::warp_size;
@@ -936,7 +1246,8 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanCustomType)
             ? rocprim::max<size_t>(ws64, logical_warp_size * 4)
             : rocprim::max<size_t>((ws64/logical_warp_size), 1) * logical_warp_size;
 
-    const unsigned int current_device_warp_size = rocprim::host_warp_size();
+    unsigned int current_device_warp_size;
+    HIP_CHECK(::rocprim::host_warp_size(device_id, current_device_warp_size));
 
     const size_t block_size = current_device_warp_size == ws32 ? block_size_ws32 : block_size_ws64;
     const unsigned int grid_size = 4;
@@ -946,7 +1257,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanCustomType)
     if( (logical_warp_size > current_device_warp_size) ||
         (current_device_warp_size != ws32 && current_device_warp_size != ws64) ) // Only WarpSize 32 and 64 is supported
     {
-        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %d.    Skipping test\n",
+        printf("Unsupported test warp size/computed block size: %zu/%zu. Current device warp size: %u.    Skipping test\n",
             logical_warp_size, block_size, current_device_warp_size);
         GTEST_SKIP();
     }
@@ -979,7 +1290,7 @@ typed_test_def(RocprimWarpScanTests, name_suffix, InclusiveScanCustomType)
             {
                 auto idx = i * logical_warp_size + j;
                 accumulator = static_cast<test_utils::custom_test_type<acc_type>>(input[idx]) + accumulator;
-                expected[idx] = static_cast<test_utils::custom_test_type<cast_type>>(accumulator);
+                expected[idx] = static_cast<T>(accumulator);
             }
         }
 
